/******************************************************************************
 * Licensed Materials - Property of IBM                                       *
 *                                                                            *
 * (C) Copyright by IBM 2003.  All Rights Reserved.                           *
 *****************************************************************************/

package com.BobCo.webobject;

import com.tivoli.twg.engine.AddressEntryFormatException;
import com.tivoli.twg.engine.*;
import com.tivoli.twg.engine.InvalidObjectIDException;
import com.tivoli.twg.engine.TWGManagedObject;
import com.tivoli.twg.engine.TWGNativeAddressEntryShadow;
import com.tivoli.twg.libs.QueueElement;
import com.tivoli.twg.libs.TWGStringArray;
import com.tivoli.twg.log.TWGRas;

import java.io.IOException;
import java.lang.SecurityException;
import java.net.InetAddress;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.Enumeration;
import java.util.Locale;


/**
 * The BCWebServerDiscoveryRequest class receives and processes requests
 * to discover web server managed objects.
 */
public class BCWebServerDiscoveryRequest implements QueueElement
{
   /* instance variables */
   private TWGManagedObject[] managedObjects = null;

   /**
    * Constructor
    *
    * @param objectList -- a list of managed objects to check for web servers
    */
   public BCWebServerDiscoveryRequest(TWGManagedObject[] objectList)
   {
     TWGRas.entryExit(TWGRas.EX7, "BCWebServerDiscoveryRequest:  in constructor");
     managedObjects = objectList;
   }

   /**
    * Process queue element
    *
    * @param int queue type (not used)
    */
   public void process( int queueType )
   {
     byte[] addressEntryRecord = null;
     int numberOfObjects = managedObjects.length;
     InetAddress address = null;
     String ipAddress = null;

     // Loop through each one of the Director managed objects, and loop through
     // each IP address for a managed object.  Ping port 80 for each IP address.
     // If we get a response, assume we've found a web server.  Make sure it's
     // not a web server we already know about.  If it isn't, create a new web
     // server managed object for it.
     for (int i = 0; i < numberOfObjects; i++)
     {
       // Get the IP addresses for a managed object
       TWGStringArray ipAddrs = (TWGStringArray)managedObjects[i].getAttributeValue(TWGManagedObjectConstants.ATTRIB_IP_ADDRS, Locale.getDefault());

       if(ipAddrs != null)
       {
         TWGRas.debug(TWGRas.EX7, "BCWebServerDiscoveryRequest.process():  There are " + ipAddrs.getLength() +
                            " IP addresses for server " + managedObjects[i].getName());

         // loop through all of the addresses for each managed object
         for (int j = 0; j < ipAddrs.getLength(); j++)
         {
           ipAddress = ipAddrs.getValueAt(j);
           TWGRas.debug(TWGRas.EX7, "BCWebServerDiscoveryRequest.process():  ipAddress = " + ipAddress);

           if ((ipAddress != null) && (ipAddress.toString() != ""))
           {
             // Convert the String address into an InetAddress address
             try
             {
               address = InetAddress.getByName(ipAddress);

               try
               {
                 // find out if the managed object is listening to port 80.
                 // if so, assume it is a web server.
                 Socket serverSocket = new Socket(address, 80);
                 if (serverSocket != null)
                 {
                   BCWebServerManagedObject webServer = BCWebServerManagedObject.getWebServer(ipAddress);

                   // find out if we already know about this web server
                   if ( webServer != null)
                   {
                     TWGRas.debug(TWGRas.EX7, "BCWebServerDiscoveryRequest.process() Found a web server we already know about: "
                                        + webServer.getName() + ", address: " + ipAddress);
                     // if we know about it, then verify that the state matches what we know
                     if (webServer.getState() != TWGManagedObjectConstants.MOSTATE_NORMAL_ONLINE)
                     {
                       // If the state doesn't match, then set the state to online
                       webServer.setState(TWGManagedObjectConstants.MOSTATE_NORMAL_ONLINE);
                     }
                   }
                   else
                   {
                     TWGRas.debug(TWGRas.EX7, "BCWebServerDiscoveryRequest.process() Found a web server we don't already know about: "
                                        + managedObjects[i].getName() + ", " + ipAddress);
                     // if we don't know about this web server, create a new BCWebServerManagedObject
                     // and set the state to online
                     webServer = new BCWebServerManagedObject(managedObjects[i].getName(), ipAddress);
                     webServer.setState(TWGManagedObjectConstants.MOSTATE_NORMAL_ONLINE);
                   }
                   try
                   {
                     webServer.save();
                   }
                   catch (TWGPersistentObjectSaveException pos)
                   {
                     TWGRas.error(TWGRas.EX7, "BCWebServerDiscoveryRequest.process: TWGPersistentObjectSave exception.");
                   }
                 }
               }
               catch (IOException ioe)
               {
                 TWGRas.debug(TWGRas.EX7, "BCWebServerDiscoveryRequest.process: Socket io exception - " + ioe.toString());

                 // find out if the managed object is in our web server list
                 // instanceof would also work
                 BCWebServerManagedObject webServer = BCWebServerManagedObject.getWebServer(ipAddress);
                 if ( webServer != null)
                 {
                   if (webServer.getState() != TWGManagedObjectConstants.MOSTATE_NORMAL_OFFLINE)
                   {
                     webServer.setState(TWGManagedObjectConstants.MOSTATE_NORMAL_OFFLINE);
                   }
                 }
               }
               catch (SecurityException se)
               {
                 TWGRas.debug(TWGRas.EX7, "BCWebServerDiscoveryRequest.process: Socket security exception - " + se.toString());

                 // find out if the managed object is in our web server list
                 // instanceof would also work
                 BCWebServerManagedObject webServer = BCWebServerManagedObject.getWebServer(ipAddress);
                 if ( webServer != null)
                 {
                   if (webServer.getState() != TWGManagedObjectConstants.MOSTATE_NORMAL_OFFLINE)
                   {
                     webServer.setState(TWGManagedObjectConstants.MOSTATE_NORMAL_OFFLINE);
                   }
                 }
               }
             }
             catch (UnknownHostException uhe)
             {
               TWGRas.error(TWGRas.EX7, "BCWebServerDiscoveryRequest.process: unknown host for address: " + ipAddress);
             }
           }
         }
       }
       else
       {
         TWGRas.error(TWGRas.EX7, "BCWebServerDiscoveryRequest.process():  The managed object " +
                            managedObjects[i].getName() + " doesn't support ipAddresses");
       }
     } // end for()
   }

} /* class BCWebServerDiscoveryRequest */
