/******************************************************************************
 * Licensed Materials - Property of IBM                                       *
 *                                                                            *
 * PID (5697-TKT)                                                             *
 *                                                                            *
 * (C) Copyright by IBM 2000. All Rights Reserved.                            *
 *****************************************************************************/

/* 
 * Pure java implementation of host (Director) inventory collector
 */

package com.BobCo.inventory;

import com.tivoli.twg.engine.*;
import com.tivoli.twg.libs.*;
import com.tivoli.twg.netipc.TWGSvcMgrServiceNode;
import com.tivoli.twg.netipc.TWGSvcMgrServiceNodeConstants;
import com.tivoli.twg.log.*;
import com.tivoli.twg.inventory.*;
import java.io.File;
import java.io.IOException;
import java.io.ObjectStreamException;
import java.lang.*;
import java.util.*;
import java.sql.Timestamp;

    /**
    *  BobCo Sample Inventory Session constructor.  The main driver for the collection,
    *  negotiates with the agent for the data.  Drives all, including the insertion into 
    *  the database.  In this case, items are entered into the database with the
    *  CommandComplete of the GatherInventoryCommand.
    */
public class BobCoSampleInventoryCollectorSession
{
    static final int RC_GENERAL_FAILURE  = TWGInvCollector.INV_COLLECTION_GENERAL_FAILURE;
    static final int RC_HARDWARE_FAILURE = TWGInvCollector.INV_COLLECTION_HARDWARE_FAILURE;
    static final int RC_SOFTWARE_FAILURE = TWGInvCollector.INV_COLLECTION_SOFTWARE_FAILURE;
    static final int RC_TIMEOUT          = TWGInvCollector.INV_COLLECTION_TIMEOUT;
    static final int RC_AUTH_FAILURE     = TWGInvCollector.INV_COLLECTION_AUTH_FAILURE;
    static final int RC_SUCCESS          = TWGInvCollector.INV_COLLECTION_SUCCESS;
    static final int TEN_SECOND_TIMEOUT  = 10000;
    
    static final long AGENT_INV_GATHER_DATA           = 0x0200;
    
    static final String AGENT_SVCNODE            = "BobCoInventoryServiceNode"; 
    


    private static int CommandCnt = 0;
    private static ServiceNode sn;
    private BobCoSampleInventoryCollector collector;
    private int objId;
    private String ipcPath;
    private int inventoryScanRc = RC_SUCCESS;
    private boolean inventoryInProgress = false;
    private boolean inventorySvcStarted = false;


    BobCoSampleInventoryCollectorSession(BobCoSampleInventoryCollector collector,
				  int objId,
				  String ipcPath)
    throws BobCoSampleCollectorException {
	this.collector = collector;
	this.objId = objId;
	this.ipcPath = ipcPath;
	TWGRas.debug(TWGRas.INVENTORY,"The BobCo collector session is starting");
	if (sn == null) {
	    try {
		sn = new ServiceNode("BobCoInventoryServer");
	    } catch (ServiceNodeException e) {
		throw new BobCoSampleCollectorException(" CS: Service node creation exception: " + e);
	    }
	}
	//  First we want to make sure the base inventory service has been started.
	long startInventoryRC = SendStartCommand(new StartInventoryCommand(ipcPath), TEN_SECOND_TIMEOUT);

	// In a real world example, we may query the agent to see what is down there, and then taylor
	// subsequent commands based on the return.
        // However, in this sample case, we know exactly what type of data the subagent will send back,
	// so will ask for it, and skip a possible negotiation phase.

	CommandCnt++; // talley the number of commands we send....
	long getInventoryData = SendGatherCommand(new GatherInventoryCommand(ipcPath), TEN_SECOND_TIMEOUT);


	TWGRas.exit(TWGRas.INVENTORY," CS: Exit, objId = "+objId);
    }



    /**
    *  Private command class for gathering data from the agent.
    */
    private class GatherInventoryCommand extends Command {
     public int inventory_rc = RC_SUCCESS;
     GatherInventoryCommand(String ipcPath) {
	 super(AGENT_INV_GATHER_DATA);

         if (ipcPath.length() == 0) {
	    SetDestinationAddress(AGENT_SVCNODE);
	 } else {
	    SetDestinationAddress(ipcPath + "::" + AGENT_SVCNODE);
	 }
     }

   /**
   *  This CommandComplete interprets the data returned from the agent and enters it into the database.
   */
    public void CommandComplete() {
	if (ReturnCode() == 0) {
	    // we got the command back, and there was no error, so process the data, and enter it into the
	    // database.
	    // The returning command has two output parms, the first one is the operating System data,
	    // and the second is the BobCoDevice table type....

	    if (NumOutputParms() == 2) {  // check to make sure we got enough data back
	      // get the data from the first output parm
	      IntelByteBuffer oper_sys_data = new IntelByteBuffer(OutputParm(0));
	      if (oper_sys_data != null) {

              // Read all the appropriate data items from the return IntelByteBuffer
	      int index = (int)oper_sys_data.ReadULONG();
              String osname = oper_sys_data.ReadASCIIZ();
              String ostype = oper_sys_data.ReadASCIIZ();
              String primary = oper_sys_data.ReadASCIIZ();
              String version = oper_sys_data.ReadASCIIZ();
              String revision = oper_sys_data.ReadASCIIZ();

              // A few lines of debug!
	      TWGRas.debug(TWGRas.INVENTORY,"OS index:" + index);
       	      TWGRas.debug(TWGRas.INVENTORY,"OS osname:" + osname);
      	      TWGRas.debug(TWGRas.INVENTORY,"OS ostype:" + ostype);
	      TWGRas.debug(TWGRas.INVENTORY,"OS primary:" + primary);
       	      TWGRas.debug(TWGRas.INVENTORY,"OS version:" + version);
      	      TWGRas.debug(TWGRas.INVENTORY,"OS revision:" + revision);

              // Now lets push this information to the database.....
	      // Create a replace managed obj command, which will delete
	      // all the information in the database pursuiant to this managed object
	      // and this table, to be replaced with the new data.
	      // We could instead use a Insertcommand, if we didn't want to remove all the
	      // old information, however, keep in mind that the database will remove any
	      // duplicates, based on Key fields(specified in the Table file)
              TWGDbStoreRow row = null;
              TWGDbReplaceManagedObjCmd dbCmd = new TWGDbReplaceManagedObjCmd(objId);
              // TWGDbStoreRowsCmd insertdbCmd = new TWGDbStoreRowsCmd(objId);

              // create a row, and add the data to it
   	      row = new TWGDbStoreRow(TWGOperatingSystemTable.TABLE_NAME);
	      row.addColumnValue(TWGOperatingSystemTable.MANAGED_OBJ_ID,objId);
              row.addColumnValue(TWGOperatingSystemTable.OP_SYS_INDEX, index);
  	      row.addColumnValue(TWGOperatingSystemTable.OP_SYS_NAME, osname);
  	      row.addColumnValue(TWGOperatingSystemTable.OP_SYS_TYPE, ostype);
      	      row.addColumnValue(TWGOperatingSystemTable.OP_SYS_PRIMARY, primary);
       	      row.addColumnValue(TWGOperatingSystemTable.OP_SYS_VERSION, version);
       	      row.addColumnValue(TWGOperatingSystemTable.OP_SYS_REVISION, revision);

              // add the row to our replace command
              dbCmd.addRow(row);
	      
              // and send it to the database.
              collector.sendToDb(TWGDbReplaceManagedObjCmd.TWG_DB_REPLACE_MANAGEDOBJ_CMD, dbCmd);

              }
	      else {
		TWGRas.debug(TWGRas.INVENTORY,"No Operating system Data returned....");
	      }

              // Get the data from the second outputparm
	      IntelByteBuffer device_data = new IntelByteBuffer(OutputParm(1));
	      if (device_data != null) {

	       // read the data from the buffer.
              int index = (int)device_data.ReadULONG();
	      String type = device_data.ReadASCIIZ();
              long date =  device_data.ReadULONG();
	      String chipset = device_data.ReadASCIIZ();
              double size = device_data.ReadDouble();


	      TWGRas.debug(TWGRas.INVENTORY,"Device Data index: " + index);
	      TWGRas.debug(TWGRas.INVENTORY,"Device Data type: " + type);
      	      TWGRas.debug(TWGRas.INVENTORY,"Device Data Date: " + date);
              TWGRas.debug(TWGRas.INVENTORY,"Device Data chipset: " + chipset);
              TWGRas.debug(TWGRas.INVENTORY,"Device Data size: " + size);


              // once again, create our DB command.
              TWGDbStoreRow row = null;
              TWGDbReplaceManagedObjCmd dbCmd = new TWGDbReplaceManagedObjCmd(objId);
              // TWGDbStoreRowsCmd insertdbCmd = new TWGDbStoreRowsCmd(objId);

   	      row = new TWGDbStoreRow(BobCoDeviceTable.TABLE_NAME);
	      row.addColumnValue(BobCoDeviceTable.MANAGED_OBJ_ID,objId);
      	      row.addColumnValue(BobCoDeviceTable.DEVICE_INDEX,index);
	      row.addColumnValue(BobCoDeviceTable.DEVICE_TYPE,type);
              row.addColumnValue(BobCoDeviceTable.DEVICE_DATE, TWGDbDates.toGMTDatetimeString(new Timestamp(date)));
	      row.addColumnValue(BobCoDeviceTable.DEVICE_CHIPSET, chipset);
	      row.addColumnValue(BobCoDeviceTable.DEVICE_SIZE, size);

	      // send the row to the database.
              dbCmd.addRow(row);
              collector.sendToDb(TWGDbReplaceManagedObjCmd.TWG_DB_REPLACE_MANAGEDOBJ_CMD, dbCmd);

	      }
	      else {
                TWGRas.debug(TWGRas.INVENTORY,"No Device Data returned....");
	      }

	    }
	    else {
                TWGRas.debug(TWGRas.INVENTORY,"We failed to recieve the proper data from the agent....");
	    }
	}
	else if (ReturnCode() == Command.CMDRET_SECURE_FAIL) {
	    inventory_rc = RC_AUTH_FAILURE;
	    TWGRas.debug(TWGRas.INVENTORY,"Security failure sending BobCo data to the database.");
	} else if (ReturnCode() == Command.CMDRET_SEND_TIMEOUT) {
	    inventory_rc = RC_TIMEOUT;
	    TWGRas.debug(TWGRas.INVENTORY,"Timeout failure sending BobCo data to the database.");
	} else {
	    TWGRas.debug(TWGRas.INVENTORY,"General failure sending BobCo data to the database. Return code is: " + ReturnCode());
	    inventory_rc = RC_GENERAL_FAILURE;
	}

          inventoryDone(inventory_rc);    
      }

    }
    /*
    *  Routine to send the GatherInventoryCommand to the Agent.
    */
    private int SendGatherCommand(GatherInventoryCommand cmd, int timeOut)
    throws BobCoSampleCollectorException {
	try {
	    cmd.SetTimeOut(timeOut);
	    sn.SendAsynchCommand(cmd);
	} catch (ServiceNodeException e) {
	    throw new BobCoSampleCollectorException(e.toString());
	} catch (InterruptedException e) {
	    throw new BobCoSampleCollectorException(e.toString());
	}
	return cmd.inventory_rc;
    }

    /*
    *  This method is used to keep tracking of all the outstanding inventory commands,
    *  and not to call the inventoryComplete method(which signals the server to shutdown and 
    *  kill thethread associated with this collector session) until we have finished processing.  
    */
    private synchronized void inventoryDone(int scanrc) {
     CommandCnt--;

      if (CommandCnt <= 0) {
	try {
	int rc = SendStopCommand(new StopInventoryCommand(ipcPath), TEN_SECOND_TIMEOUT);
	}
	catch (BobCoSampleCollectorException e) {
            TWGRas.error(TWGRas.INVENTORY,"Stop command failed: " + e.toString());
	}
	collector.inventoryComplete(objId, scanrc, System.currentTimeMillis());
      }

    }



    // The send Start command, which starts the BobCoInventory service node on the agent.
    private int SendStartCommand(StartInventoryCommand cmd, int timeOut)
    throws BobCoSampleCollectorException {
	try {
	    cmd.SetTimeOut(timeOut);
	    sn.SendCommand(cmd);
	} catch (ServiceNodeException e) {
	    throw new BobCoSampleCollectorException(e.toString());
	} catch (InterruptedException e) {
	    throw new BobCoSampleCollectorException(e.toString());
	}
	return cmd.inventory_rc;
    }


    class StartInventoryCommand extends Command {
        public int inventory_rc;
	StartInventoryCommand(String ipcPath) {
	    super(TWGSvcMgrServiceNodeConstants.SVCMGR_START_SVC);
	    IntelByteBuffer sn_name = new IntelByteBuffer(100);
	    sn_name.WriteASCIIZ(AGENT_SVCNODE,0);
	    AddInputParm(sn_name.GetBuffer());  // Tell it which service node to launch.....
	    if (ipcPath.length() == 0) {
		SetDestinationAddress("SvcMgr");
	    } else {
		SetDestinationAddress(ipcPath + "::" + "SvcMgr");
	    }
	}
	
    }

    // The send Start command, which stops the BobCoInventory service node on the agent.
    private int SendStopCommand(StopInventoryCommand cmd, int timeOut)
    throws BobCoSampleCollectorException {
	try {
	    cmd.SetTimeOut(timeOut);
	    // This command is sent asynchronously because it is called from
	    // within a commandcomplete routine.  There is no CommandComplete
	    // overload, because there is not much we can do if
	    // the command does not return......
	    sn.SendAsynchCommand(cmd);
	} catch (ServiceNodeException e) {
	    TWGRas.debug(TWGRas.INVENTORY,"Service node exception in Stop command");
	    throw new BobCoSampleCollectorException(e.toString());
	} catch (InterruptedException e) {
	    TWGRas.debug(TWGRas.INVENTORY,"Interrupted  exception in Stop command");
	    throw new BobCoSampleCollectorException(e.toString());
	}
	return cmd.inventory_rc;
    }

    class StopInventoryCommand extends Command {
	public int inventory_rc;
	StopInventoryCommand(String ipcPath) {
	    super(TWGSvcMgrServiceNodeConstants.SVCMGR_STOP_SVC);
	    IntelByteBuffer sn_name = new IntelByteBuffer(100);
	    sn_name.WriteASCIIZ(AGENT_SVCNODE,0);
	    AddInputParm(sn_name.GetBuffer());  // Tell it which service node to term.....
	    if (ipcPath.length() == 0) {
		SetDestinationAddress("SvcMgr");
	    } else {
		SetDestinationAddress(ipcPath + "::" + "SvcMgr");
	    }
	}
    }

}
