/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * 09N6996
 *
 * (C) Copyright IBM Corp. 1999, 2004 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */

package com.ibm.sysmgt.sdk.tools;

import com.tivoli.twg.engine.cli.*;

import com.tivoli.twg.libs.*;
import java.util.*;
import java.io.*;

/**
 * Generic CLI client<p>
 * The client is very thin providing just enough
 * logic to connect to the Director CLI server where
 * all the processing occurs.  The output from the server execution
 * is then passed back to the client where it's displayed completing
 * the request.
 *
 * @author Rob Warren
 * @version 1.1 - 6/8/05
 */
public class CliClient implements CommandReceivedListener, CliConstants {

    private String hostname = null;
    private String userid = null;
    private String password = null;
    private String bundle = null;
    private String fileName = null;
    private boolean readPipe = false;
    private String datalink = "com.tivoli.twg.libs.TWGTCPIPLink";
    private String linkparms = "2034";

    private ServiceNodeImplFactory fact = null;
    private ServiceNode sn = null;
    private boolean receivedExitSignal = false;
    private int returnCode = ICliBundle.FAIL;
    private int argProcIndex = 0;

    private PrintWriter clientOut;
    private ResourceBundle msg;
    private static final String NLSClass = "com.tivoli.twg.engine.cli.CliClientNLS";

    public CliClient(PrintWriter pw) {
        clientOut = pw;
    }

    public int execute(String[] sIn) {

        int code = ICliBundle.OK;

        try {
            msg = ResourceBundle.getBundle(NLSClass, Locale.getDefault());
        } catch (Exception e) {
            msg = ResourceBundle.getBundle(NLSClass, Locale.US);
        }

        if (sIn.length == 0) {
            printHelp();
            return ICliBundle.USAGE;
        }

        String sOut[] = processInputArguments(sIn);
        if (sOut == null) {
            return ICliBundle.USAGE;
        }

        code = connectToServer();
        if (code != ICliBundle.OK) {
            return code;
        }

        CliCommand cmd  = new CliCommand(Locale.getDefault(), bundle, sOut);
        cmd.SetCommandCode(CLI_INIT);

        if (sendCommand(cmd) == false) {
            return ICliBundle.FAIL;
        }

        // Handle case where user does not specify a command bundle

        if (cmd.ReturnCode() == ICliBundle.USAGE) {

            String defaultCliBundle = "server";

            // Try the server bundle first
            // create a new sOut
            String[]  sOut2 = new String[sOut.length+1];
            sOut2[0] = bundle;
            System.arraycopy (sOut,0,sOut2,1,sOut.length);

            bundle = defaultCliBundle;
            System.out.println("Trying bundle '" + bundle + "." + sOut2[0]);
            CliCommand cmd2  = new CliCommand(Locale.getDefault(), bundle, sOut2);
            cmd2.SetCommandCode(CLI_INIT);

            if (sendCommand(cmd2) == false) {
                return ICliBundle.FAIL;
            }

            if (cmd2.ReturnCode() != ICliBundle.USAGE) {
                // System.out.println("server bundle worked");
                cmd = cmd2;
            }

        }


        processCommandOutput(cmd);

        try {
            sn.DeregisterDefaultCommandListener();
            sn.Close();
        } catch (ServiceNodeException e) {
        }

        return returnCode;
    }

    /**
     * Handles task of sending IPC commands to the server
     * @param Command cmd - IPC command
     * @return boolean - true for success else false
     */
    boolean sendCommand(Command cmd) {
        boolean rtn = false;
        try {
            if (!sn.SendCommand(cmd)) {
                clientOut.println( msg.getString("ERROR.COMM") );
            } else {
                rtn = true;
            }
        } catch (Exception e) {
            clientOut.println( msg.getString("ERROR.COMM") );
        }

        return rtn;
    }

    /**
     * Process command line args<p>
     * The command line format is as shown below:
     * <li>==> director [options] bundle command [commandArguments]
     */
    protected String[] processInputArguments(String[] s) {

        List out = new LinkedList();
        String[] sOut = null;
        try {
            processOptions(s, out);
            while (argProcIndex < s.length) {
                out.add( readArg(s) );
            }
        } catch (ProgramErrorException e) {
            clientOut.println(msg.getString("ERROR.QUOTES") + "\n");
            printHelp();
            return null;
        } catch (Exception e) {
            printHelp();
            return null;
        }

        if (fileName != null) {
            String fileBuf = getFileBuffer(fileName);
            if (fileBuf == null) {
                return null;
            }
            out.add(fileBuf);
        }

        if (readPipe) {
            readPipedArgs(out);
        }


        if (out.size() > 0) {
            bundle = (String)out.remove(0);
            sOut = new String[out.size()];
            for (int i = 0; i < sOut.length; i++) {
                sOut[i] = (String)out.get(i);
            }
            out = null;
        } else {
            clientOut.println(msg.getString("ERROR.INCOMPLETE_CMD") + "\n");
            printHelp();
        }

        return sOut;
    }

    /**
     * Parse through the command line switches/options<p>
     * The possible options are:
     * <li>-s ServerName        Name of director server for connection
     * <li>-u UserName          Authentication UserName
     * <li>-p Password          Authentication password
     * <li>-k DataLinkOption    DataLink override for connection to server
     * <li>-o DataLinkParams    DataLink parameters override for connection to server
     * <li>-f Filename          Load specified file in commandArguments[0]
     * <li>-r                   Read incomming data pipe from stdin
     * <li>-u                   Display usage from server
     * <li>-l                   List all server CLI bundles
     * <li>-h [clear | size=N]  Server history options (list|clear|resize)
     *
     * @param String[] s - the command line argument list
     * @param List out - Output list
     */
    protected void processOptions(String[] s, List out) {

        int historyIndex = 0;
        boolean doingOptions = true;

        while (doingOptions && (argProcIndex < s.length)) {
            if (s[argProcIndex].equals("-s")) {
                hostname = s[argProcIndex+1];
                argProcIndex++;
            } else if (s[argProcIndex].equals("-u")) {
                userid = s[argProcIndex+1];
                argProcIndex++;
            } else if (s[argProcIndex].equals("-p")) {
                password = s[argProcIndex+1];
                argProcIndex++;
            } else if (s[argProcIndex].equals("-r")) {
                readPipe = true;
            } else if (s[argProcIndex].equals("-?")) {
                out.add(CliBaseBundleName);
                out.add(CliHelpMethod);
            } else if (s[argProcIndex].equals("-h")) {
                out.add(CliBaseBundleName);
                out.add(CliHelpMethod);
            } else if (s[argProcIndex].equals("-b")) {
                out.add(CliBaseBundleName);
                out.add(CliListMethod);
            } else if (s[argProcIndex].equals("-l")) {
                out.add(CliBaseBundleName);
                out.add(CliHistoryMethod);
                historyIndex = argProcIndex + 1;
            } else if (s[argProcIndex].equals("-f")) {
                fileName = s[argProcIndex+1];
                argProcIndex++;
            } else if (s[argProcIndex].equals("-k")) {
                datalink = s[argProcIndex+1];
                argProcIndex++;
            } else if (s[argProcIndex].equals("-o")) {
                linkparms = s[argProcIndex+1];
                argProcIndex++;
            } else if ((s[argProcIndex].equals(CliHistoryClearOpt)) &&
                       (historyIndex == argProcIndex) ) {
                out.add("-c");
            } else if ((s[argProcIndex].startsWith(CliHistorySizeOpt)) &&
                       (historyIndex == argProcIndex) ) {
                String size = s[argProcIndex].substring(CliHistorySizeOpt.length());
                out.add("-s");
                out.add(size);
            } else {
                doingOptions = false;
            }

            // Advance index
            if (doingOptions) {
                argProcIndex++;
            }
        }
    }

    /**
     * Read command line arguments with processing for multi-arg quoted params.
     * Utilizes and modifies class global variable argProcIndex<p>
     * Note: Many commnd shells do this processing automatically
     * @param String[] args - full command line argument list
     */
    protected String readArg(String[] args) throws ProgramErrorException {

        if (args[argProcIndex].startsWith("\"") == false) {
            // no " "
            return args[argProcIndex++];
        }
        if (args[argProcIndex].endsWith("\"")) {
            // 1 arg " "
            String arg =  args[argProcIndex].substring(1, args[argProcIndex].length()-1);
            argProcIndex++;
            return arg;
        }

        // multi arg " "
        boolean complete = false;
        StringBuffer sb = new StringBuffer(args[argProcIndex].substring(1));
        while (!complete) {
            if (++argProcIndex >= args.length) {
                throw new ProgramErrorException();
            }
            if (args[argProcIndex].endsWith("\"")) {
                sb.append(" " + args[argProcIndex].substring(0, args[argProcIndex].length()-1));
                complete = true;
            } else {
                sb.append(" " + args[argProcIndex]);
            }
        }
        argProcIndex++;
        return sb.toString();
    }

    /**
     * Read a file and load contents into String
     * @param String fileName - name of file to load data input
     * @return String - file contents
     */
    protected String getFileBuffer(String fileName) {

        StringBuffer buf = new StringBuffer();

        try {
            BufferedReader in = new BufferedReader( new FileReader(fileName) );
            int ch = in.read();
            while (ch > -1) {
                buf.append((char)ch);
                ch = in.read();
            }
            in.close();
        } catch (FileNotFoundException e) {
            clientOut.println( msg.getString("ERROR.FILE_NOT_FOUND") );
        } catch (IOException e) {
            clientOut.println( msg.getString("ERROR.FILE_READ") );
        }
        if (buf.length() == 0) {
            return null;
        }
        return buf.toString();
    }

    /**
     * Read incomming arguments from input pipe
     * @param List argList - list to add the arguments to
     */
    protected void readPipedArgs(List argLst) {

        try {
            BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
            String in = br.readLine();
            while (in != null) {
                int mrk1 = 0;
                int mrk2 = in.indexOf(' ');
                while (mrk2 >= 0) {
                    argLst.add( in.substring(mrk1, mrk2) );
                    mrk1 = mrk2+1;
                    mrk2 = in.indexOf(' ', mrk1);
                }
                String arg = in.substring(mrk1);
                if (arg.length() > 0) {
                    argLst.add( arg );
                }
                in = br.readLine();
            }
        } catch (IOException e) {
            clientOut.println( msg.getString("ERROR.PIPE") );
        }
    }

    /**
     * Display the output streams
     * @param CliCommand cmd - command returned from server execution
     */
    public void processCommandOutput(CliCommand cmd) {

        String stdOut = cmd.getOutputString();
        if ((stdOut != null) && (!stdOut.equals(""))) {
            clientOut.print(stdOut);
            clientOut.flush();
        }

        if (cmd.ReturnCode() == CLI_PROCESSING) {
            // Process the servers output
            synchronized(this) {
                try {
                    if (!receivedExitSignal) {
                        this.wait();
                    }
                } catch (InterruptedException e) {
                }
            }
        } else {
            returnCode = (int)cmd.ReturnCode();
        }
    }

    /**
     * Process incomming output streams
     */
    public boolean CommandReceived(ServiceNode sn, Command cmd) {

        String stdOut = IntelByteBuffer.ReadCompUnicode(cmd.InputParm(0), 0);

        if ((stdOut != null) && (!stdOut.equals(""))) {
            clientOut.print(stdOut);
            clientOut.flush();
        }

        if (cmd.CommandCode() != CLI_PROCESSING) {
            synchronized(this) {
                receivedExitSignal = true;
                clientOut.println("");
                clientOut.flush();
                returnCode = (int)cmd.CommandCode();
                this.notify();
            }
        }
        return true;
    }



    private static boolean isCurrentUserAuthorized()
    {

        boolean result = false;
        String  ivUsername = null;
        try {
            ivUsername = System.getProperty( "user.name" );
        } catch ( SecurityException sex ) {
            System.err.println( "TWGMainViewPlugIn.loadLoginPrefs - SecurityException caught " + sex );
        } catch ( Exception ex ) {
            System.err.println( "TWGMainViewPlugIn.loadLoginPrefs - SecurityException caught " + ex );
        }

        if (ivUsername != null) {
            // System.out.println("ivUsername = '" + ivUsername + "'");
            if (UserIDSecurity.isSuperUser(ivUsername)) {
                // System.out.println("isSuperUser("+ivUsername+")");
                result = true;
            } else {
                System.out.println("" + ivUsername+" is not a Director superuser");
            }
        } else {
            System.out.println("ivUsername == null");
        }

        return result;


    }


    /**
     * Create a local or remote server connection
     * <li>ProcessInputArguments() must be called before this method
     * @return int - true for success
     */
    int connectToServer() {
        int rtnCode = ICliBundle.OK;
        try {
            if (hostname != null) {

                fact = new ServiceNodeRemoteImplFactory(datalink, linkparms, hostname, userid, password);

            } else {

                if (isCurrentUserAuthorized()) {
                    fact = new ServiceNodeLocalImplFactory();
                } else {
                    clientOut.println( msg.getString("ERROR.ID") );
                    rtnCode = ICliBundle.SECURITY_FAILURE;
                }

            }

            ServiceNode.SetServiceNodeFactory(fact);
            sn = new ServiceNode();
            sn.RegisterDefaultCommandListener(this);

            if (!sn.Create()) {
                clientOut.println( msg.getString("ERROR.COMM") );
                rtnCode = ICliBundle.FAIL;
            }
        } catch (TWGExpiredPasswordException e) {
            clientOut.println( msg.getString("ERROR.PASSWORDEXP") );
            rtnCode = ICliBundle.SECURITY_FAILURE;
        } catch (TWGBadPasswordException e) {
            clientOut.println( msg.getString("ERROR.PASSWORD") );
            rtnCode = ICliBundle.SECURITY_FAILURE;
        } catch (TWGDisabledUserIDException e) {
            clientOut.println( msg.getString("ERROR.DISABLEDID") );
            rtnCode = ICliBundle.SECURITY_FAILURE;
        } catch (TWGUserIDCountExceededException e) {
            clientOut.println( msg.getString("ERROR.ID_EXCEEDED") );
            rtnCode = ICliBundle.SECURITY_FAILURE;
        } catch (TWGBadUserIDException e) {
            clientOut.println( msg.getString("ERROR.ID") );
            rtnCode = ICliBundle.SECURITY_FAILURE;
        }


        catch (TWGDownlevelConsoleException e) {
            clientOut.println( msg.getString("ERROR.UPLEVEL_LINK") );
            rtnCode = ICliBundle.FAIL;
        } catch (TWGUplevelConsoleException e) {
            clientOut.println( msg.getString("ERROR.DWNLEVEL_LINK") );
            rtnCode = ICliBundle.FAIL;
        }

        catch (TWGInitParmException e) {
            clientOut.println( msg.getString("ERROR.PARM") );
            rtnCode = ICliBundle.FAIL;
        } catch (TWGLinkNotSupportedException e) {
            clientOut.println( msg.getString("ERROR.LINK") );
            rtnCode = ICliBundle.FAIL;
        } catch (TWGLinkException e) {
            clientOut.println( msg.getString("ERROR.LINK") );
            rtnCode = ICliBundle.FAIL;
        } catch (java.lang.IllegalArgumentException e) {
            clientOut.println( msg.getString("ERROR.GEN_NETWORK") );
            rtnCode = ICliBundle.FAIL;
        }

        catch (ServiceNodeException e) {
            clientOut.println( msg.getString("ERROR.GEN_NETWORK") );
            rtnCode = ICliBundle.FAIL;
        }

        catch (java.net.UnknownHostException e) {
            clientOut.println( msg.getString("ERROR.HOST") );
            rtnCode = ICliBundle.FAIL;
        } catch (java.net.ConnectException e) {
            clientOut.println( msg.getString("ERROR.COMM_REFUSED") );
            rtnCode = ICliBundle.FAIL;
        } catch (java.net.SocketException e) {
            clientOut.println( msg.getString("ERROR.GEN_NETWORK") );
            rtnCode = ICliBundle.FAIL;
        } catch (java.io.IOException e) {
            clientOut.println( msg.getString("ERROR.GEN_NETWORK") );
            rtnCode = ICliBundle.FAIL;
        }

        catch (Exception e) {
            clientOut.println( msg.getString("ERROR.GEN_NETWORK") );
            rtnCode = ICliBundle.FAIL;
        }

        return rtnCode;
    }

    /*
     * Client execution help msg
     */
    private void printHelp() {
        try {
            int idx = 1;
            String str = msg.getString( "USAGE." + idx );
            while (str != null) {
                clientOut.println( str );
                str = msg.getString( "USAGE." +  (++idx) );
            }
        } catch (java.util.MissingResourceException e) {
        }
    }

    /**
     * Main entry point: command line arguments
     */
    public static void main(String[] args) {

        int rc = ICliBundle.OK;
        try {
            OutputStreamWriter osw = new OutputStreamWriter(System.out);
            if (osw.getEncoding().equals("Cp1252")) {
                osw = new OutputStreamWriter(System.out, "Cp850");
            }
            PrintWriter pw = new PrintWriter(osw, true);
            CliClient client = new CliClient(pw);
            rc = client.execute(args);
            pw.flush();
            // Note: do not close redirected System.out!
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
            rc = ICliBundle.FAIL;
        }

        System.exit(rc);
    }
}

