/******************************************************************************
 * Licensed Materials - Property of IBM                            *
 *                                                                            *
 * PID (5697-TKT)                                                             *
 *                                                                            *
 * (C) Copyright by IBM 1999. All Rights Reserved. *
 *****************************************************************************/

package com.BobCo.snapins;

import java.awt.*;
import java.beans.*;
import java.text.*;
import java.util.*;
import javax.swing.*;
import com.tivoli.twg.engine.*;
import com.tivoli.twg.dmi.*;
import com.tivoli.twg.libs.*;
import com.tivoli.twg.guilibs.*;

/**
 * The ComponentIDSnapIn class is a sample DMI group browser.
 * It implements the TWGDMIGroupBrowserInterface and is
 * registered to display instances of "DMTF|ComponentID|*".
 *
 * @version 1.1 - 6/8/05
 */
public class ComponentIDSnapIn implements TWGDMIGroupBrowserInterface,
                                          Runnable
{

   /* Private data */
   private JPanel                      panel            = null;
   private JPanel                      panel2           = null;
   private int                         status           = TWGDMIConstants.UNKNOWN;
   private String                      statusText       = null;
   private WindowAttributes            windowAttributes = null;
   private PropertyChangeSupport       prop             = null;
   private ListResourceBundle          bundle           = null;
   private ImageIcon                   icon             = null;
   private DMIGroup                    dmiGroup         = null;
   private long                        moid             = 0;
   private int                         component_id     = 0;
   private TWGDMIDefaultGroupBrowser   default_browser  = null;
   private ComponentAttributeResponse  response         = null;
   private JLabel                      icon_label       = null;
   private JLabel                      product_label    = null;
   private JLabel                      version_label    = null;
   private JLabel                      serial_label     = null;
   private JLabel                      company_label    = null;
   private JLabel                      install_label    = null;

   private static final String ibmIconPath       = "com/BobCo/snapins/images/ibm.gif";
   private static final String intelIconPath     = "com/BobCo/snapins/images/intel.gif";
   private static final String tivoliIconPath    = "com/BobCo/snapins/images/tivoli.gif";
   private static final String microsoftIconPath = "com/BobCo/snapins/images/microsoft.gif";

   /**
    * Constructor.  The snap-in interface requires a default
    * constructor that does not have any parameters.
    */
   public ComponentIDSnapIn ()
   {
      /* Create a Java Bean property change support object */
      /* to manage the property change listeners           */
      prop = new PropertyChangeSupport( this );
   }

   /**
    * The getDisplayWindow method is used to return a JPanel 
    * that will be displayed by the caller.  The snap-in is  
    * responsible for the contents and appearance of the panel.
    */
   public JPanel getDisplayWindow ()
   {
      /* If we have already created a panel, just re-use it*/
      if (panel == null)
      {

         /* Load the NLS resource bundle for display strings */
         if (bundle == null)
         {
            bundle = (ListResourceBundle)ListResourceBundle.getBundle
                     ( "com.BobCo.snapins.BobCoSnapInsResources",
                     Locale.getDefault() );
         }

         /* Create the panel with a Border layout */
         panel = new JPanel();
         panel.setLayout(new BorderLayout());
         panel2 = new JPanel();
         panel2.setLayout(new GridBagLayout());
         panel.add(panel2,BorderLayout.CENTER);
      }

      /* Return the panel */
      return panel;
   }

   /**
    * Retrieve the current status of the snap-in.  Status values 
    * should be one of the constants defined in TWGDMIConstants.
    */
   public int getStatus()
   {
      return status;
   }

   /**
    * Set the current status of the snap-in.  Status values 
    * should be one of the constants defined in TWGDMIConstants.
    * After changing the status, any property change listeners
    * are notified of the change.
    */
   public void setStatus( int newStatus )
   {
      /* Load the NLS resource bundle for display strings */
      if (bundle == null)
      {
         bundle = (ListResourceBundle)ListResourceBundle.getBundle
                  ( "com.BobCo.snapins.BobCoSnapInsResources",
                  Locale.getDefault() );
      }

      /* Take actions based on the new status */
      switch ( newStatus )
      {
         /* If we are closing, then remove all references to    */
         /* our data objects so that they are garbage collected */
         /* by the Java run-time.                               */
         case TWGDMIConstants.CLOSE:
            if (response != null)
            {
               /* If we are in the middle of a DMI call, then */
               /* cancel the response processing.             */
               response.cancel();
               response = null;
            }
            panel = null;
            bundle = null;
            icon = null;
            windowAttributes = null;
            default_browser = null;
            icon_label = null;
            product_label = null;
            version_label = null;
            serial_label = null;
            company_label = null;
            install_label = null;
            break;
            
         /* If we are in-progress, then update the status text */
         case TWGDMIConstants.IN_PROGRESS:
            setStatusText(bundle.getString("DMIQuery"));
            break;

         /* Otherwise we must be ready, so update the status text */
         default:
            setStatusText(bundle.getString("Ready"));
            // response = null;
            break;
      }
      
      /* Notify any property change listeners */
      int oldStatus = status;
      status = newStatus;
      if (newStatus != TWGDMIConstants.CLOSE)
      {
         prop.firePropertyChange( "statusChange",
                                  new Integer(oldStatus),
                                  new Integer(newStatus));
      }
   }

   /**
    * Retrieve a text string that describes the current status
    * of the snap-in.  This string may be used by the parent of
    * the snap-in to update the status area at the bottom of
    * the window.
    */
   public String getStatusText()
   {
      /* Load the NLS resource bundle for display strings */
      if (bundle == null)
      {
         bundle = (ListResourceBundle)ListResourceBundle.getBundle
                  ( "com.BobCo.snapins.BobCoSnapInsResources",
                  Locale.getDefault() );
      }

      /* If we have never set the status text, then assume */
      /* we are ready                                      */
      if ( statusText == null )
      {
         statusText = bundle.getString("Ready");
      }

      /* Return the latest status text string */
      return statusText;
   }

   /**
    * Set the current status text message for the snap-in.
    */
   public void setStatusText( String newStatusText )
   {
      statusText = newStatusText;
   }

   /**
    * Retrieve the DMI group object that is being displayed 
    * by the snap-in. 
    */
   public DMIGroup getDMIGroup()
   {
      return dmiGroup;
   }

   /**
    * Set the DMI group that the snap-in will display instance
    * for.  Calling this method will start the processing to
    * get information from the DMI service layer for display.
    */
   public void setDMIGroup( long new_moid, DMIGroup new_group, int new_id )
   {
      moid = new_moid;
      dmiGroup = new_group;
      component_id = new_id;

      /* Be sure that we have a display window to update */
      getDisplayWindow();

      /* If a previous DMI call, then cancel it */
      if (response != null)
      {
         response.cancel();
      }
      
      /* Start DMI processing on background thread */
      (new Thread(this)).start();
   }

   /**
    * This method is used to get an icon image to display for
    * a DMI group. 
    */
   public ImageIcon getIconForDMIGroup ( DMIGroup group )
   {
      if (default_browser == null)
      {
         default_browser = new TWGDMIDefaultGroupBrowser();
      }

      /* Return the default image */
      return default_browser.getIconForDMIGroup(group);
   }

   /**
    * This method is called by the parent of the snap-in to
    * update the window attributes so that they match the
    * rest of the current user console.
    */
   public void setWindowAttributes( WindowAttributes wa )
   {
      /* Save the new attributes */
      windowAttributes = wa;

      /* Ensure that we have a display window */
      getDisplayWindow();

      /* Apply the new attributes to our window */
      JComponent panels[] = {panel};
      ImageLabel labels[] = {};
      JComponent others[] = {panel2};
      windowAttributes.applyColors(panels, labels, others);

      /* Repaint the window */
      panel.invalidate();
      panel.validate();
      panel.repaint();
   }

   /**
    * Retrieve the current window attributes 
    */
   public WindowAttributes getWindowAttributes()
   {
      return windowAttributes;
   }

   /**
    * Add a property change listener to the snap-in.
    */
   public void addPropertyChangeListener( PropertyChangeListener listener )
   {
      prop.addPropertyChangeListener( listener );
   }

   /**
    * Remove a property change listener from the snap-in.
    */
   public void removePropertyChangeListener( PropertyChangeListener listener )
   {
      prop.removePropertyChangeListener( listener );
   }

   /**
    * This method is called by the ComponentAttributeResponse to
    * fill in the display fields with the attribute values.
    */
   public void displayAttributes ( DMIAttributeValue[] Attributes )
   {
      int x;

      /* Remove any existing data from the display */
      panel2.removeAll();
      icon_label = null;
      product_label = null;
      version_label = null;
      serial_label = null;
      company_label = null;
      install_label = null;

      /* Process Attributes */
      for (x = 0; x < Attributes.length; x++)
      {
         System.out.println("ID = "+Attributes[x].GetID());
         System.out.println("Value = "+Attributes[x].GetValueAsString());
         switch (Attributes[x].GetID())
         {
            /* Manufacturer */
            case 1:
               String manufacturer = Attributes[x].GetValueAsString();
               if (manufacturer.startsWith("IBM"))
               {
                  icon_label = new JLabel();
                  icon_label.setIcon(new ImageIcon
                                    (TWGRemoteSnapInManager.getImage(ibmIconPath)));
                  icon_label.setHorizontalAlignment(SwingConstants.CENTER);
               }
               if (manufacturer.startsWith("Tivoli"))
               {
                  icon_label = new JLabel();
                  icon_label.setIcon(new ImageIcon
                                    (TWGRemoteSnapInManager.getImage(tivoliIconPath)));
                  icon_label.setHorizontalAlignment(SwingConstants.CENTER);
               }
               if (manufacturer.startsWith("Intel"))
               {
                  icon_label = new JLabel();
                  icon_label.setIcon(new ImageIcon
                                    (TWGRemoteSnapInManager.getImage(intelIconPath)));
                  icon_label.setHorizontalAlignment(SwingConstants.CENTER);
               }
               if (manufacturer.startsWith("Microsoft"))
               {
                  icon_label = new JLabel();
                  icon_label.setIcon(new ImageIcon
                                    (TWGRemoteSnapInManager.getImage(microsoftIconPath)));
                  icon_label.setHorizontalAlignment(SwingConstants.CENTER);
               }
               if (manufacturer.length() > 0)
               {
                  company_label = new JLabel(bundle.getString("Manufacturer")+" "+
                                             manufacturer);
                  company_label.setHorizontalAlignment(SwingConstants.CENTER);
               }
               break;
            /* Product */
            case 2:
               product_label = new JLabel(Attributes[x].GetValueAsString());
               product_label.setFont(new Font("Dialog", Font.BOLD, 20));
               product_label.setHorizontalAlignment(SwingConstants.CENTER);
               break;
            /* Version */
            case 3:
               version_label = new JLabel(Attributes[x].GetValueAsString());
               version_label.setHorizontalAlignment(SwingConstants.CENTER);
               break;
            /* Serial Number */
            case 4:
               String serial = Attributes[x].GetValueAsString();
               if (serial.length() > 0)
               {
                  serial_label = new JLabel(bundle.getString("SerialNumber")+" "+
                                            serial);
                  serial_label.setHorizontalAlignment(SwingConstants.CENTER);
               }
               break;
            /* Installation Date */
            case 5:
               Object obj = Attributes[x].GetValue();
               if (obj instanceof DMI_DateString)
               {
                  DMI_DateString ds = (DMI_DateString) obj;
                  DateFormat df = DateFormat.getDateTimeInstance(DateFormat.SHORT,
                                                                 DateFormat.SHORT);
                  install_label = new JLabel(bundle.getString("InstallDate")+" "+
                                             df.format(ds.getDate()));
                  install_label.setHorizontalAlignment(SwingConstants.CENTER);
               }
               break;
         }
      }

      /* Layout window */
      int row = 0;
      Insets i = new Insets(0,5,0,5);
      makeGridBagLayoutItem(panel2,new JLabel(" "), 0, row++, 1, 1, 0, 1, i);
      if (icon_label != null)
      {
         makeGridBagLayoutItem(panel2,icon_label, 0, row++, 1, 1, 0, 0, i);
         makeGridBagLayoutItem(panel2,new JLabel(" "), 0, row++, 1, 1, 0, 0, i);
      }
      if (product_label != null)
      {
         makeGridBagLayoutItem(panel2,product_label, 0, row++, 1, 1, 0, 0, i);
      }
      makeGridBagLayoutItem(panel2,new JLabel(" "), 0, row++, 1, 1, 0, 0, i);
      if (version_label != null)
      {
         makeGridBagLayoutItem(panel2,version_label, 0, row++, 1, 1, 0, 0, i);
      }
      makeGridBagLayoutItem(panel2,new JLabel(" "), 0, row++, 1, 1, 0, 0, i);
      if (serial_label != null)
      {
         makeGridBagLayoutItem(panel2,serial_label, 0, row++, 1, 1, 0, 0, i);
      }
      if (company_label != null)
      {
         makeGridBagLayoutItem(panel2,company_label, 0, row++, 1, 1, 0, 0, i);
      }
      makeGridBagLayoutItem(panel2,new JLabel(" "), 0, row++, 1, 1, 0, 0, i);
      if (install_label != null)
      {
         makeGridBagLayoutItem(panel2,install_label, 0, row++, 1, 1, 0, 0, i);
      }
      makeGridBagLayoutItem(panel2,new JLabel(" "), 0, row++, 1, 1, 0, 1, i);

      /* Repaint the window */
      panel2.invalidate();
      panel2.validate();
      panel2.repaint();
   }

   /**
    * This is an internal method used to simplify the usage
    * of GridBagLayout.
    */
   private void makeGridBagLayoutItem (Container cont,
                                       Component comp,
                                       int       x,
                                       int       y,
                                       int       w,
                                       int       h,
                                       double    weightx,
                                       double    weighty,
                                       Insets    i)
   {
     GridBagLayout gbl = (GridBagLayout)cont.getLayout();
     GridBagConstraints c = new GridBagConstraints();
     c.anchor     = GridBagConstraints.NORTHWEST;
     c.fill       = GridBagConstraints.BOTH;
     c.gridx      = x;
     c.gridy      = y;
     c.gridwidth  = w;
     c.gridheight = h;
     c.weightx    = weightx;
     c.weighty    = weighty;
     if (i != null)
     {
        c.insets = i;
     }
     cont.add(comp);
     gbl.setConstraints(comp, c);
   }

   /**
    * Background thread method to start DMI processing.
    */
   public void run ()
   {
      /* Create a new DMI call to get the data for display */
      response = new ComponentAttributeResponse(moid,dmiGroup,component_id);
   }

   /**
    * ComponentAttributeResponse is an internal class used to query
    * attributes for a component ID.
    */
   class ComponentAttributeResponse implements iDMIResponse,
                                               Runnable
   {
      private DMIComponentData    systemNode  = null;
      private DMIInterface        ClientDMI   = null;
      private long                moid;
      private DMIGroup            dmiGroup;
      private int                 component_id;
      private DMIAttributeValue[] values;
      private boolean             cancel;

      /*
       * Constructor.
       */
      public ComponentAttributeResponse( long new_moid,
                                         DMIGroup new_group, 
                                         int new_id )
      {
         cancel = false;

         moid = new_moid;
         dmiGroup = new_group;
         component_id = new_id;

         setStatus( TWGDMIConstants.IN_PROGRESS );
         ClientDMI = new DMIInterface( moid, this );
         return;
      }

      /**
       * This method is used to cancel a response that is in-progress.
       */
      public void cancel()
      {
         cancel = true;
         return;
      }

      /**
       * This callback is called when the DMIInterface is available.
       * We use this method to make the actual attribute query.
       */
      public void DMIRegisterReady( DMIInterface Caller )
      {
         if ( cancel == false )
         {
            if ( Caller.DmiListAttributes( this,
                                           false,
                                           0,
                                           true,
                                           true,
                                           component_id,
                                           dmiGroup.GetID(),
                                           0 ) == false )
            {
               /* Error condition, just clear status indicator */
               setStatus( TWGDMIConstants.READY );
            }
         }
         return;
      }

      /**
       * This callback is not used in this sample snap-in.
       */
      public void DMIListComponent(DMIComponent[] Components,int RC,DMIInterface Caller)
      { 
         return; 
      }

      /**
       * This callback is not used in this sample snap-in.
       */
      public void DMIListGroup(DMIGroup[] Groups,int RC,DMIInterface Caller)
      { 
         return; 
      }  

      /** 
       * This callback is called when the attribute listis ready.
       * We now need to get the values.
       */
      public void DMIListAttribute( DMIAttribute[] Attributes,
                                    int RC,
                                    DMIInterface Caller )
      {
         if ( (Attributes != null) && (cancel == false) )
         {
            DMIAttributeValue[] values = new DMIAttributeValue[Attributes.length];
            int x;
            for (x = 0; x < Attributes.length; x++)
            {
               values[x] = new DMIAttributeValue(Attributes[x].GetID(),
                                                 Attributes[x].GetDataType(),
                                                 null);
            }
            DMI_MultiRow[] rows = new DMI_MultiRow[1];
            rows[0] = new DMI_MultiRow ( component_id, dmiGroup.GetID(), null, 
                                        DMIInterface.REQUEST_MODE_FIRST, 
                                        null, values );
            if (Caller.DmiGetMultiple(this, rows) == false)
            {
               /* Error condition, just clear status indicator */
               setStatus( TWGDMIConstants.READY );
            }
         }
         return;
      }

      /**
       * This callback is not used in this sample snap-in.
       */
      public void DMIListLanguage(String[] Languages,int RC,DMIInterface Caller)
      { 
         return; 
      }

      /**
       * This callback is not used in this sample snap-in.
       */
      public void DMIListClasses(DMIClassInfo[] Classes,int RC,DMIInterface Caller)
      { 
         return; 
      }

      /**
       * This callback is not used in this sample snap-in.
       */
      public void DMIGetResponse(DMIAttributeValue Value, int RC,DMIInterface Caller)
      { 
         return; 
      }

      /**
       * This callback is used the the attribute values are ready.
       */
      public void DMIGetResponse(DMI_MultiRow[] Values,int RC,DMIInterface Caller)
      { 
         if ( cancel == false) 
         {
            if (( RC == DMICodes.DMIERR_NO_ERROR ) || 
                ( RC == DMICodes.DMIERR_NO_ERROR_MORE_DATA ) || 
                ( RC == DMICodes.DMIERR_DEFAULT_LANGUAGE_RETURNED ))  
            {
               values = Values[0].GetValues();
               try
               {
                  SwingUtilities.invokeLater(this);
               }
               catch (Exception x)
               {
               }
            }
            else
            {
               setStatus( TWGDMIConstants.READY );
            }
         }
         return; 
      }

      /**
       * This callback is not used in this sample snap-in.
       */
      public void DMISetResponse(int RC,DMIInterface Caller)
      { 
         return; 
      }

      /**
       * This callback is used to handle DMI error conditions.
       */
      public void DMIError(DMIInterface Caller,int DMIStatus)
      {
         /* Error condition, just clear status indicator */
         setStatus( TWGDMIConstants.READY );
         return;
      }

      /**
       * Method invoked on the event dispatch thread to update 
       * the display after the attribute values are ready. 
       */
      public void run ()
      {
         displayAttributes(values);
         setStatus( TWGDMIConstants.READY );
      }
   } 
}
