package com.BobCo.FileSource;

import java.io.*;
import java.net.UnknownHostException;
import java.util.StringTokenizer;

import com.tivoli.twg.engine.*;
import com.tivoli.twg.filetransfer.*;
import com.tivoli.twg.libs.*;
import com.tivoli.twg.log.TWGOutput;

/**
 * This is the File Source example code.  This class illustrates how to instantiate
 * the various FileSource implementations and shows a variety of actions available
 * with FileSource objects.
 *
 * To compile this file:
 *
 *    From directory <SdkInstallDir>\samples\javasrc,
 *    javac com\BobCo\FileSource\FileSourceSample.java
 *
 * The class file resides in <SdkInstallDir>\samples\classes\com\BobCo\FileSource\
 *
 * Run this class from the command line:
 *
 *    From directory <SdkInstallDir>\samples\classes
 *    java com.BobCo.FileSource.FileSourceSample
 */
public class FileSourceSample implements TWGConObjectListener {

  // The file source we will communicate with
  private static FileSource fs;

  // Object to wait for update notification
  private static Object LOCK = new Object();

  // Stream to print output to. System.out is redirected to a file
  public static PrintStream interaction;

  /**
   * Default constructor.
   * We need to be able to create an object for
   * a callback from the server that lets us know
   * when our link is set up and it is okay to
   * communicate with the server.
   */
  public FileSourceSample() {

    // Register an object listener to get
    // notified when we are allowed access to
    // the server
    TWGConObject.RegisterListener(this);
  }

  /** Setup for remote contact.  This method logs the user
    * into the server where he will be validated for remote access.
    */
  private static boolean prepRemote () {

    boolean rc=false;
    try {

      BufferedReader stdIn =new BufferedReader(new InputStreamReader(System.in));
      interaction.print("Server:");
      String server = stdIn.readLine();
      interaction.print("User:");
      String user = stdIn.readLine();
      interaction.print("Password:");
      String passwd = stdIn.readLine();

      interaction.print("Connecting...");

      //Initialize the console service node
      TWGConsoleServiceNode.initialize();

      /* Logon to the server, and initialize the service node */
      TWGConsoleServiceNode.logonToServer( server, user, passwd, 0 );

      /* Request shadow update so we get access. */
      TWGConsoleServiceNode.startShadowRefresh();

      // Now we must wait for the shadow refresh to complete.
      // We are notified through our TWGConObjectListener
      // implementation
      synchronized (LOCK) {
        try {
           LOCK.wait();
        } catch (InterruptedException ie) {
          interaction.print("LOCK wait interrupted");
          // if we are interrupted for some reason,
          // let's just sleep a bit, and hope we
          // get the shadow refresh update before
          // we try to access the server
          try {
            Thread.sleep(3000);
          } catch (InterruptedException ix) {
            // noop
          }
        }
      }

      rc=true;

    } catch (TWGGatewayInitException gix) {
      interaction.println( "TWGErrorGatewayInit" );
    } catch (TWGInitParmException ipx) {
      interaction.println( "TWGErrorInitParm" );
    } catch (TWGLinkNotSupportedException lnsx) {
      interaction.println( "TWGErrorLinkNotSupported" );
      lnsx.printStackTrace();
    } catch (TWGBadUserIDException buix) {
      interaction.println( "TWGErrorBadUserIDPwd" );
    } catch (TWGBadPasswordException bpx) {
      interaction.println( "TWGErrorBadUserIDPwd" );
    } catch (UnknownHostException uhx) {
      interaction.println( "TWGErrorUnknownHost" );
    } catch (IOException iox) {
      interaction.println( "TWGErrorIO" );
    } catch (ServiceNodeException snx) {
      interaction.println( "TWGErrorServiceNode" );
    } catch (InterruptedException ie) {
      interaction.println( "InterruptedException" );
    }

    return rc;
  }

  /** Display a list of valid commands. */
  private static void help () {
    interaction.println("(L)ist Root Level");
    interaction.println("(Dir)ectory <dirctory to list>");
    interaction.println("(IsF)ile <file name>");
    interaction.println("(IsD)irectory <file name>");
    interaction.println("(CanW)rite <file name>");
    interaction.println("(CanR)ead  <file name>");
    interaction.println("(Del)ete File <file name>");
    interaction.println("File (S)ize <file name>");
    interaction.println("(M)ake Directory <directory name>");
    interaction.println("(Read) in file as text <file name>");
    interaction.println("(Ref)resh");
    interaction.println("(Ren)ame file <old file name> <new file name>");
    interaction.println("(REGINT) Retrieve an integer registry value <key name> <value name>");
    interaction.println("(REGSTR) Retrieve a string registry value <key name> <value name>");
    interaction.println("(REGBIN) Retrieve a binary registry value <key name> <value name>");
    interaction.println("(GetEnv)ironment variable value <variable name>");
    interaction.println("(H)elp, (?)");
    interaction.println("(E)xit");
  }

  /** Display proper usage information.  Called when this class is run improperly. */
  private static void argHelp() {
    interaction.println("Usage: com.BobCo.FileSource.FileSourceSample [source type]");
    interaction.println("\tc, con, console                  Local file source.");
    interaction.println("\tv1, v1server, v1serv             Version 1 server file source.");
    interaction.println("\tserver, serv                     Server file source.");
    interaction.println("\ta,agent <hostname>               Agent file source.");

  }

  /**
   * This is done to keep logging messages from clutering up the user interface. It can
   * be ignored.
   */
  private static void setUpLogging() throws IOException
      {
      try
          {
          System.out.println("Redirecting IO...");
          PrintStream logOut = new PrintStream(new FileOutputStream("error.out"));
          interaction = System.out;
          System.setErr(logOut);
          System.setOut(logOut);
          } 
      catch (java.security.AccessControlException acx)
          {
          System.out.println("Unable to redirect IO.  Continuing...\n");
          }
      }

  /** Lists the local and available remote drives. */
  private static void listRootLevel () {
    try {
      FSTObject [] rl= fs.getRootLevel();
      if (rl == null)
        interaction.println("Unkown root level.  Query failed.");
      else {
        for (int i=0; i<rl.length;i++) {
          interaction.println(i+ ":    "+ rl[i].toString());
        }
      }
    } catch (FTException e) {
      interaction.println("FTException Getting Drives!!!!");
      e.printStackTrace();
    }
  }

  /**
   * Lists the directory contents of the direcotry passed in.
   *
   * @parm String the entire command read in from the command line.
   *              This command must have a valid fully-qualified directory name
   *              as its argument.
   */
  private static void listDir (String listCommand) {
    int ind=listCommand.indexOf(" ");

    if (ind <=0) {
      interaction.println("No directory specified!!!");
      return;
    }

    String dirFor=listCommand.substring(ind+1).trim();
    interaction.println("Doing dir for: "+dirFor);

    ind=dirFor.lastIndexOf("/");
    FSTObject toFill;
    if (ind >=0)
      toFill = new FSTObject(dirFor.substring(0,ind),dirFor.substring(ind+1),FSTObject.DIRECTORY);
    else
      toFill = new FSTObject("/",dirFor,FSTObject.DIRECTORY);
    try {

      boolean b=fs.getDirectoryTree(toFill);

      if (b) {
        for (int i=0; i< toFill.getChildCount(); i++) {
          interaction.println(((FSTObject)toFill.getChildAt(i)).getName());
        }
      } else {
        interaction.println("Directory fill was unsuccessful.");
      }

    } catch (FTException e) {
      interaction.println("FTException Getting Directory!!!!");
      e.printStackTrace();
    }
  }

  /**
   * Determines whether the specified filename exists and is a file rather
   * than a directory.
   *
   * @parm String the entire command read in from the command line.
   *              This command must have a valid fully-qualified filename
   *              as its argument.
   */
  private static void isFile (String isFileCommand) {
    int ind =isFileCommand.indexOf(" ");

    if (ind <=0) {
      interaction.println("No file specified!!!");
      return;
    }

    String isFor=isFileCommand.substring(ind+1).trim();

    try {
      boolean b=fs.isFile(isFor);

      if (b) {
        interaction.println(isFor+" is a file.");
      } else {
        interaction.println(isFor+" is not a file.");
      }
    } catch (FTException e) {
      interaction.println("FTException doing isFile!!!!");
      e.printStackTrace();
    }

  }

  /**
   * Determines whether the specified directory name exists and is a directory rather
   * than a file.
   *
   * @parm String the entire command read in from the command line.
   *              This command must have a valid fully-qualified directory name
   *              as its argument.
   */
  private static void isDirectory (String isFileCommand) {
    int ind =isFileCommand.indexOf(" ");

    if (ind <=0) {
      interaction.println("No file specified!!!");
      return;
    }

    String isFor=isFileCommand.substring(ind+1).trim();

    try {
      boolean b=fs.isDirectory(isFor);

      if (b) {
        interaction.println(isFor+" is a directory.");
      } else {
        interaction.println(isFor+" is not a directory.");
      }
    } catch (FTException e) {
      interaction.println("FTException doing isFile!!!!");
      e.printStackTrace();
    }

  }

  /**
   * Determines whether the specified filename can be written to.
   *
   * @parm String the entire command read in from the command line.
   *              This command must have a fully-qualified filename
   *              as its argument.  The file does not have to exist.
   */
  private static void canWrite (String canWriteCommand) {
    int ind =canWriteCommand.indexOf(" ");

    if (ind <=0) {
      interaction.println("No file specified!!!");
      return;
    }

    String canFor=canWriteCommand.substring(ind+1).trim();

    try {
      boolean b=fs.canWrite(canFor);

      if (b) {
        interaction.println(canFor+" can be written to.");
      } else {
        interaction.println(canFor+" cannot be written to.");
      }
    } catch (FTException e) {
      interaction.println("FTException doing canWrite!!!!");
      e.printStackTrace();
    }

  }

  /**
   * Determines whether the specified file can be read.
   *
   * @parm String the entire command read in from the command line.
   *              This command must have a fully-qualified filename
   *              as its argument.
   */
  private static void canRead (String canReadCommand) {
    int ind =canReadCommand.indexOf(" ");

    if (ind <=0) {
      interaction.println("No file specified!!!");
      return;
    }

    String canFor=canReadCommand.substring(ind+1).trim();

    try {
      boolean b=fs.canRead(canFor);

      if (b) {
        interaction.println(canFor+" can be read from.");
      } else {
        interaction.println(canFor+" cannot be read from.");
      }
    } catch (FTException e) {
      interaction.println("FTException doing canRead!!!!");
      e.printStackTrace();
    }

  }

  /**
   * Deletes the specified file.
   *
   * @parm String the entire command read in from the command line.
   *              This command must have a valid fully-qualified filename
   *              as its argument.
   */
  private static void deleteFile (String deleteFileCmd) {
    int ind =deleteFileCmd.indexOf(" ");

    if (ind <=0) {
      interaction.println("No file specified!!!");
      return;
    }

    String toDel=deleteFileCmd.substring(ind+1).trim();

    try {
      boolean b=fs.deleteFile(toDel);

      if (b) {
        interaction.println(toDel+" successfully deleted.");
      } else {
        interaction.println(toDel+" could not be deleted.");
      }
    } catch (FTException e) {
      interaction.println("FTException doing deleteFile!!!!");
      e.printStackTrace();
    }

  }

  /** Refresh the file source. */
  private static void refresh() {
    try {
      fs.refresh();

    } catch (FTException e) {
      interaction.println("FTException doing refresh!!!!");
      e.printStackTrace();
    }
  }

  /**
   * Create a new directory.
   *
   * @parm String the entire command read in from the command line.
   *              This command must have a fully-qualified directory name
   *              as its argument.
   */
  private static void makeDirectory (String mkdirCmd) {
    int ind =mkdirCmd.indexOf(" ");

    if (ind <=0) {
      interaction.println("No file specified!!!");
      return;
    }

    String toMake=mkdirCmd.substring(ind+1).trim();

    try {
      boolean b=fs.mkdir(toMake);

      if (b) {
        interaction.println(toMake+" was successfully created.");
      } else {
        interaction.println(toMake+" could not be created.");
      }
    } catch (FTException e) {
      interaction.println("FTException doing makeDirectory!!!!");
      e.printStackTrace();
    }

  }

  /**
   * Read in a file from the file source and print it to the screen.
   *
   * @parm String the command string is passed in.  It must contain
   *              a fully qualified filename.
   */
  public static void readInFile (String command) {
    int ind =command.indexOf(" ");

    if (ind <=0) {
      interaction.println("No file specified!!!");
      return;
    }

    String textFor=command.substring(ind+1).trim();

    try {
      interaction.println("textFor: "+textFor);

      InputStream bis = fs.getInputStream(textFor);

      if (bis == null) {
        interaction.println("Could not find file: "+textFor);
      } else {
        // read in the file

        InputStreamReader isr = new InputStreamReader(bis);

        char [] cb = new char [1024];
        int read=0;
        while (read != -1) {
          interaction.print(new String(cb,0,read));
          read = isr.read(cb, 0 , 1024);
        }

      } // end if

      // add a space for ease of reading
      interaction.println();

    } catch (FTException e) {
      interaction.println("FTException doing read!!!!");
      e.printStackTrace();
    } catch (IOException e) {
      interaction.println("IOException doing read!!!!");
      e.printStackTrace();
    }

  }

  /**
   * Rename the specified file to the new name passed in.
   *
   * @parm String the entire command read in from the command line.
   *              This command must have a valid fully-qualified filename
   *              as its first argument, and a new file name as its second.
   */
  private static void renameFile (String renCmd) {
    StringTokenizer st = new StringTokenizer(renCmd," ");
    if (st.countTokens() < 3) {
      interaction.println("Command is missing parameters.");
      return;
    }

    st.nextToken();
    String origName=st.nextToken();
    String newName=st.nextToken();

    try {
      boolean b=fs.renameFile(origName, newName);

      if (b) {
        interaction.println(origName+" was successfully renamed to "+ newName+".");
      } else {
        interaction.println(origName+" could not be renamed.");
      }
    } catch (FTException e) {
      interaction.println("FTException doing renamefile!!!!");
      e.printStackTrace();
    }

  }

  /**
   * Returns the size of the specified file.
   *
   * @parm String the entire command read in from the command line.
   *              This command must have a valid fully-qualified filename
   *              as its argument.
   */
  private static void getSize (String sizeCommand) {
    int ind =sizeCommand.indexOf(" ");

    if (ind <=0) {
      interaction.println("No file specified!!!");
      return;
    }

    String sizeFor=sizeCommand.substring(ind+1).trim();

    try {
      long size=fs.getFileSize(sizeFor);

      if (size >= 0) {
        interaction.println(sizeFor+" = "+size);
      } else {
        interaction.println("Could not determine size of "+sizeFor+". File may not exist");
      }
    } catch (FTException e) {
      interaction.println("FTException doing get size!!!!");
      e.printStackTrace();
    }


  }

  /**
   * Initialize an agent file source.  If the file source we wish to communicate with
   * is an IT Director agent, then we must use this method to initialize it.  This
   * method takes a hostname and uses it to lookup the desitnation address (IPC Path)
   * of the agent we wish to communicate with.  Using this data, it instantiates the agent.
   *
   * @parm String hostname of an IT Director agent.  NOTE:  This hostname must be spelled
   *              exactly as it appears on the inventory sheet for each agent from the IT
   *              Director main console, capitalization matters.
   */
  public static boolean initAgent(String hostname) {
    boolean rc = false;

    interaction.println("Initializing agent...");
    LongValueSet ids = null;
    String[] attribList = {TWGManagedObjectConstants.ATTRIB_IP_HOSTS};
    String[] value = {hostname};
    TWGStringArray sArray = new TWGStringArray(value);
    DataValue[] dataValue = {sArray};
    FindObjectByAttribCmd cmd = new FindObjectByAttribCmd("com.tivoli.twg.engine.TWGNativeManagedObject", attribList, dataValue);

    try {
      // NOTE:  We use the console service node here for simplicity.  However,
      // when used within IT Director, this could cause dead locks, you should
      // always create your own service node to use within IT Director.
      if (TWGConsoleServiceNode.getConsoleServiceNode().SendCommand(cmd)) {
        if (cmd.ReturnCode() == FTConstants.FILE_TRANSFER_BASE_SUCCESS) {
          ids = cmd.getMatchingIDs();
        } else {
          interaction.println("Send Command Failed.  RC = "+cmd.ReturnCode());
        }
      } else {
        interaction.println("Send Command Failed.");
      }
    } catch (InterruptedException ie) {
      interaction.println("InterruptedException caught in initAgent: "+ie);
      ie.printStackTrace();
    } catch (ServiceNodeException sne) {
      interaction.println("ServiceNodeException in initAgent: "+sne);
      sne.printStackTrace();
    }

    if ((ids != null) && (ids.Length() > 0)) {
      String ipcPath = getClientDestinationAddress(ids.GetValue(0));

      try {

        fs = new FSAgent(ids.GetValue(0), ipcPath);

        // now we must refresh the agent, to insure population of the root level
        refresh();

        rc = true;

      } catch (FTException fte) {
        interaction.println("FTException caught while declaring: "+fte);
        fte.printStackTrace();
      }
      interaction.println("Agent initialized.");
    } else {
      interaction.println("Hostname not found.");
    }

    return rc;
  }

  /**
   * This method will return the destination address of the remote ITD agent.
   * This address is used to instantiate agent file sources.
   *
   * @param mo - the client managed object
   */
  public static String getClientDestinationAddress(long mo) {

    String ipcPath = null;
    try {

      TWGRemoteManagedObject rmo = TWGRemoteTaskManager.getRemoteManagedObject(
                                                                              TWGConsoleServiceNode.getConsoleServiceNode(), mo);

      // Shadow the address blob entry.
      TWGNativeAddressEntryShadow addressBlob = new TWGNativeAddressEntryShadow();

      // Try to pull the address out of the command for this
      // particular MOID.
      try {

        byte[] ber= rmo.getAddressEntryRecord();
        int berlen = rmo.getAddressEntryRecord().length;
        addressBlob.initAddressEntry(ber, 0, berlen);


      } catch (AddressEntryFormatException aefx) {
        // Could not get address blob, log it and skip.
        interaction.println("AddressEntryFormatException: "+aefx);
      }

      // destination address of our remote client.
      ipcPath = addressBlob.getIPCPath();

    } catch (ServiceNodeClosedException snce) {
      interaction.println("ServiceNodeClosed: "+snce);
    }

    return(ipcPath);

  }

  /**
   * Read in a registry value of type String.
   *
   * @parm String the entire command read in from the command line.
   *              This command must have a valid registry key.
   *              as its argument.
   */
  private static void readStringRegValue (String cmd) {
    StringTokenizer st = new StringTokenizer(cmd," ");

    if (st.countTokens() < 3) {
      interaction.println("Command is missing parameters.");
      return;
    }

    st.nextToken();
    String regKey= st.nextToken();
    String regValue=st.nextToken();


    try {
      interaction.println("RegistryValue= "+fs.getRegStringValue(regKey, regValue));
    } catch (FTException e) {
      e.printStackTrace();
    }


  }

  /**
   * Read in a registry value of type int.
   *
   * @parm String the entire command read in from the command line.
   *              This command must have a valid registry key.
   *              as its argument.
   */
  private static void readIntRegValue (String cmd) {
    StringTokenizer st = new StringTokenizer(cmd," ");

    if (st.countTokens() < 3) {
      interaction.println("Command is missing parameters.");
      return;
    }

    st.nextToken();
    String regKey= st.nextToken();
    String regValue=st.nextToken();


    try {
      interaction.println("RegistryValue= "+fs.getRegIntValue(regKey, regValue));
    } catch (FTException e) {
      e.printStackTrace();
    }


  }

  /**
   * Read in a registry value of type bytes.
   *
   * @parm String the entire command read in from the command line.
   *              This command must have a valid registry key.
   *              as its argument.
   */
  private static void readBytesRegValue (String cmd) {
    StringTokenizer st = new StringTokenizer(cmd," ");

    if (st.countTokens() < 3) {
      interaction.println("Command is missing parameters.");
      return;
    }

    st.nextToken();
    String regKey= st.nextToken();
    String regValue=st.nextToken();


    try {
      byte [] b = fs.getRegBytesValue(regKey, regValue);
      for (int i=0; i< b.length; i++) {
        interaction.print(Integer.toHexString(b[i]).substring(6));
      }
      interaction.println();


    } catch (FTException e) {
      e.printStackTrace();
    }


  }

  /**
   * Read in an environment variable.
   *
   * @parm String the entire command read in from the command line.
   *              This command must have a valid environment variable
   *              as its argument.
   */
  private static void getEnvironmentVar (String cmd) {
    StringTokenizer st = new StringTokenizer(cmd," ");

    if (st.countTokens() < 2) {
      interaction.println("Command is missing parameters.");
      return;
    }

    st.nextToken();
    String varName= st.nextToken();

    try {
      String val = fs.getEnvironmentVariable(varName);
      interaction.println("Var Value="+val);
    } catch (FTException e) {
      e.printStackTrace();
    }

  }

  /**
   * Start of notification set : this method is called just before
   * a group of adds, modifies, and deletes are passed to their
   * respective interfaces
   */
  public void BeginNotificationSet() {
  }
  /**
   * Method for notify an object delete
   * @param oid - object ID of the deleted object
   */
  public void DeleteObjectNotify(long oid) {
  }
  /**
   * Method for notify of object create
   * @param obj - TWGConObject reference to newly created object
   */
  public void CreateObjectNotify(TWGConObject obj) {
  }
  /**
   * Method for notify of object modifies
   * @param obj - TWGConObject reference to newly changed object
   */
  public void ModifiedObjectNotify(TWGConObject obj) {
  }
  /**
   * End of notification set : this method is called just after the last
   * notification in a set has been done.
   */
  public void EndNotificationSet() {
    synchronized (LOCK) {
      LOCK.notifyAll();
    }
  }

  /** The main method.  File Source instantiation takes place here. */
  public static void main (String args[]) {

    // Redirects System.out. There is no need to do this normally
    try {
        setUpLogging();
    } catch (Exception e) {
        e.printStackTrace();
        System.exit(1);
    }

    if (args.length==0) {
      interaction.println("You must specify the type of file source.");
      argHelp();
      return;
    }

    FileSourceSample fss = new FileSourceSample(); // create an object for callbacks.

    //first arg is type
    if (args[0].equalsIgnoreCase("console")
        || args[0].equalsIgnoreCase("con")
        || args[0].equalsIgnoreCase("c")) {
      try {
        fs=new FSConsole();
      } catch (FTException e) {
        //quit
        interaction.println("FTException initializing console file source");
        e.printStackTrace();
        return;
      }
    } else if (args[0].equalsIgnoreCase("v1server")
               || args[0].equalsIgnoreCase("v1serv")
               || args[0].equalsIgnoreCase("v1")) {

      if (!prepRemote()) {
        interaction.println("Unable to connect with server!");
        return;
      } else {
        interaction.println("Connected to Server.");
      }

      try {
        fs=new FSServerV1();
      } catch (FTException e) {
        //quit
        interaction.println("FTException initializing ServerV1 file source");
        e.printStackTrace();
        return;
      } catch (UserNotAuthorizedException e) {
        interaction.println("User is not authorized to access this file source..");
        //quit
        return;
      }
    } else if (args[0].equalsIgnoreCase("server")
               || args[0].equalsIgnoreCase("serv")) {

      if (!prepRemote()) {
        interaction.println("Unable to connect with server!");
        return;
      } else {
        interaction.println("Connected to Server.");
      }

      try {
        fs=new FSServer();
      } catch (FTException e) {
        //quit
        interaction.println("FTException initializing ServerV1 file source");
        e.printStackTrace();
        return;
      } catch (UserNotAuthorizedException e) {
        interaction.println("User is not authorized to access this file source..");
        //quit
        return;
      }
    } else if (args[0].equalsIgnoreCase("agent")
               || args[0].equalsIgnoreCase("a")) {
      /*First we need to log on to the server*/
      if (!prepRemote()) {
        interaction.println("Unable to connect with server!");
        return;
      } else {
        interaction.println("Connected to Server.");
        if (args.length < 2) {
          interaction.println("Need IP Address to connect to client.");
          TWGConsoleServiceNode.terminate();
          return;
        } else {
          //do stuff
          String hostname = args[1];
          if (!initAgent(hostname)) {
            interaction.println("Could not Initialize Agent.");
            TWGConsoleServiceNode.terminate();
            return;
          }
        }
      }


    } else {
      interaction.println("Unrecognized Argument...Exiting!");
      argHelp();
      return;
    }

    //test loop
    BufferedReader sysIn=new BufferedReader(new InputStreamReader(System.in));
    String command=null;

    while (true) {

      interaction.print(">");

      try {
        command=sysIn.readLine();
      } catch (IOException e) {
        interaction.println("Invalid Input. Exiting.");
        break;
      }

      if (command.toLowerCase().startsWith("e")) {
        try {
          fs.cleanup();
          TWGConsoleServiceNode.terminate();

        } catch (FTException e) {
          interaction.println("Exception cleaning up file source resources.");
          e.printStackTrace();
        }

        System.exit(0);

      } else if (command.toLowerCase().startsWith("h")
                 || command.startsWith("?")) {
        help();
      } else if (command.toLowerCase().startsWith("l")) {
        listRootLevel();
      } else if (command.toLowerCase().startsWith("dir")) {
        listDir(command);
      } else if (command.toLowerCase().startsWith("isf")) {
        isFile(command);
      } else if (command.toLowerCase().startsWith("isd")) {
        isDirectory(command);
      } else if (command.toLowerCase().startsWith("canw")) {
        canWrite(command);
      } else if (command.toLowerCase().startsWith("canr")) {
        canRead(command);
      } else if (command.toLowerCase().startsWith("del")) {
        deleteFile(command);
      } else if (command.toLowerCase().startsWith("s")) {
        getSize(command);
      } else if (command.toLowerCase().startsWith("m")) {
        makeDirectory(command);
      } else if (command.toLowerCase().startsWith("read")) {
        readInFile(command);
      } else if (command.toLowerCase().startsWith("ren")) {
        renameFile(command);
      } else if (command.toLowerCase().startsWith("ref")) {
        refresh();
      } else if (command.toLowerCase().startsWith("regstr")) {
        readStringRegValue(command);
      } else if (command.toLowerCase().startsWith("regint")) {
        readIntRegValue(command);
      } else if (command.toLowerCase().startsWith("regbin")) {
        readBytesRegValue(command);
      } else if (command.toLowerCase().startsWith("getenv")) {
        getEnvironmentVar(command);
      } else {
        interaction.println ("Unrecognized Command. Type ? for help.");
      }
    }
  }

} // end FileSourceSample

