package com.tivoli.twg.mscs;

import com.tivoli.twg.libs.*;

/**
 * Get full cluster configuration command
 * @author Mike Primm
 * @version 1.1 - 6/8/05
 */
public class MSCSGetFullConfigCommand extends MSCSAgentCommand {
    /**
     * Constructor for full configuration request - get cluster
     * object data based on enumeration mask
     * @param ipc_path - IPC path to target cluster agent
     * @param enum_mask - Enumeration mask for ClusterEnum
     */
    public MSCSGetFullConfigCommand(String ipc_path, long enum_mask) {
        super(MSCSAGT_GET_FULL_CONFIG_INFO, ipc_path);
        byte[] p0 = new byte[IntelByteBuffer.SIZEOF_ULONG];
        IntelByteBuffer.WriteULONG(p0, enum_mask, 0);
        AttachInputParm(p0);
    }
    /**
     * Constructor for full configuration request - get all attributes
     * @param ipc_path - IPC path to target cluster agent
     */
    public MSCSGetFullConfigCommand(String ipc_path) {
        this(ipc_path, CLUSTER_ENUM_ALL);
    }
    /**
     * Get cluster name
     * @returns cluster name
     */
    public String getClusterName() { return readCompUnicodeOutputParam(0); }
    /**
     * Get cluster major version
     * @returns major version
     */
    public int getClusterMajorVersion() { 
        return readWORDOutputParam(0, 
            IntelByteBuffer.SkipCompUnicode(OutputParm(0), 0) + 4); }
    /**
     * Get cluster minor version
     * @returns minor version
     */
    public int getClusterMinorVersion() { return readWORDOutputParam(0, 
        IntelByteBuffer.SkipCompUnicode(OutputParm(0), 0) + 6); }
    /**
     * Get cluster build number
     * @returns  build number
     */
    public int getClusterBuildNumber() { return readWORDOutputParam(0, 
        IntelByteBuffer.SkipCompUnicode(OutputParm(0), 0) + 8); }
    /**
     * Get cluster vendor ID
     * @returns vendor ID
     */
    public String getClusterVendorID() { return readUnicodeOutputParam(0, 
        IntelByteBuffer.SkipCompUnicode(OutputParm(0), 0) + 10); }
    /**
     * Get cluster CSD version
     * @returns CSD version
     */
    public String getClusterCSDVersion() { return readUnicodeOutputParam(0, 
        IntelByteBuffer.SkipCompUnicode(OutputParm(0), 0) + 138); }
    /**
     * Get cluster name resource ID (invariant resource ID for cluster name)
     * @returns name ID, or "" if not returned
     */
    public String getClusterNameResource() {
        int len = IntelByteBuffer.SkipCompUnicode(OutputParm(0), 0);
        int off = readWORDOutputParam(0, len) + len;
        if(off >= OutputParmLength(0))
            return "";
        else
            return IntelByteBuffer.ReadCompUnicode(OutputParm(0), off);
    }
    /**
     * Get number of elements returned
     * @returns number of elements
     */
    public int getNumberOfElements() { return NumOutputParms() - 1; }
    /**
     * Get type of element
     * @param index - index of element
     * @returns element type (CLUSTER_ENUM_*)
     */
    public long getElementType(int index) { return readDWORDOutputParam(index+1); }
    /**
     * Get name of element
     * @param index - index of element
     * @returns element name
     */
    public String getElementName(int index) { return readCompUnicodeOutputParam(index+1, 4); }
    /**
     * Get node state (for CLUSTER_ENUM_NODE elements)
     * @param index - index of element
     * @returns state (ClusterNode*)
     */
    public long getElementNodeState(int index) { 
        return readDWORDOutputParam(index+1, 
            IntelByteBuffer.SkipCompUnicode(OutputParm(index+1), 4)); 
    }
    /**
     * Get network interface list for node element (for CLUSTER_ENUM_NODE elements)
     * @param index - index of element
     * @returns list of network interface names
     */
    public String[] getElementNodeNetworkInterfaceList(int index) { 
        byte[] p = OutputParm(index+1);
        int cnt = (int)readDWORDOutputParam(index+1, 
            IntelByteBuffer.SkipCompUnicode(p, 4) + 4); 
        return readCompUnicodeListOutputParam(index+1, cnt, 
            IntelByteBuffer.SkipCompUnicode(p, 4) + 8);
    }
    /**
     * Get description property for node element (for CLUSTER_ENUM_NODE elements)
     * @param index - index of element
     * @returns description
     */
    public String getElementNodeDescription(int index) { 
        byte[] v = OutputParm(index+1);
        // skip type, name, state
        int off =  IntelByteBuffer.SkipCompUnicode(v, 4) + 4;
        // get num of net interfaces
        int cnt = IntelByteBuffer.ReadLONG(v, off);
        off += 4;
        // skip them
        for(int i = 0; i < cnt; i++) {
            off = IntelByteBuffer.SkipCompUnicode(v, off);
        }
        // now at the description - return it
        return readCompUnicodeOutputParam(index+1, off);
    }

    /**
     * Get network state (for CLUSTER_ENUM_NETWORK elements)
     * @param index - index of element
     * @returns state (ClusterNetwork*)
     */
    public long getElementNetworkState(int index) { 
        return readDWORDOutputParam(index+1, 
            IntelByteBuffer.SkipCompUnicode(OutputParm(index+1), 4)); 
    }
    /**
     * Get network interface list for network element (for CLUSTER_ENUM_NETWORK elements)
     * @param index - index of element
     * @returns list of network interface names
     */
    public String[] getElementNetworkNetworkInterfaceList(int index) { 
        byte[] p = OutputParm(index+1);
        int cnt = (int)readDWORDOutputParam(index+1, 
            IntelByteBuffer.SkipCompUnicode(p, 4) + 4); 
        return readCompUnicodeListOutputParam(index+1, cnt, 
            IntelByteBuffer.SkipCompUnicode(p, 4) + 8);
    }
    /**
     * Get address mask property for network element (for CLUSTER_ENUM_NETWORK elements)
     * @param index - index of element
     * @returns address mask
     */
    public String getElementNetworkAddressMask(int index) { 
        byte[] v = OutputParm(index+1);
        // skip type, name, state
        int off =  IntelByteBuffer.SkipCompUnicode(v, 4) + 4;
        // get num of net interfaces
        int cnt = IntelByteBuffer.ReadLONG(v, off);
        off += 4;
        // skip them
        for(int i = 0; i < cnt; i++) {
            off = IntelByteBuffer.SkipCompUnicode(v, off);
        }
        // now at the addressmask - return it
        return readCompUnicodeOutputParam(index+1, off);
    }
    /**
     * Get role property for network element (for CLUSTER_ENUM_NETWORK elements)
     * @param index - index of element
     * @returns role
     */
    public long getElementNetworkRole(int index) { 
        byte[] v = OutputParm(index+1);
        // skip type, name, state
        int off =  IntelByteBuffer.SkipCompUnicode(v, 4) + 4;
        // get num of net interfaces
        int cnt = IntelByteBuffer.ReadLONG(v, off);
        off += 4;
        // skip them
        for(int i = 0; i < cnt; i++) {
            off = IntelByteBuffer.SkipCompUnicode(v, off);
        }
        // now at the addressmask - skip it, and return the net role
        return readDWORDOutputParam(index+1, 
            IntelByteBuffer.SkipCompUnicode(v, off));
    }
    /**
     * Get description property for network element (for CLUSTER_ENUM_NETWORK elements)
     * @param index - index of element
     * @returns description
     */
    public String getElementNetworkDescription(int index) { 
        byte[] v = OutputParm(index+1);
        // skip type, name, state
        int off =  IntelByteBuffer.SkipCompUnicode(v, 4) + 4;
        // get num of net interfaces
        int cnt = IntelByteBuffer.ReadLONG(v, off);
        off += 4;
        // skip them
        for(int i = 0; i < cnt; i++) {
            off = IntelByteBuffer.SkipCompUnicode(v, off);
        }
        // skip the addressmask and the network role
        off = IntelByteBuffer.SkipCompUnicode(v, off) + 4;
        // now at the description - return it
        return readCompUnicodeOutputParam(index+1, off);
    }
    /**
     * Get network interface state (for CLUSTER_ENUM_NETINTERFACE elements)
     * @param index - index of element
     * @returns state (ClusterNetInterface*)
     */
    public long getElementNetInterfaceState(int index) { 
        return readDWORDOutputParam(index+1, 
            IntelByteBuffer.SkipCompUnicode(OutputParm(index+1), 4)); 
    }
    /**
     * Get network interface address (for CLUSTER_ENUM_NETINTERFACE elements)
     * @param index - index of element
     * @returns address
     */
    public String getElementNetInterfaceAddress(int index) { 
        return readCompUnicodeOutputParam(index+1, 
            IntelByteBuffer.SkipCompUnicode(OutputParm(index+1), 4) + 4); 
    }
    /**
     * Get network interface description (for CLUSTER_ENUM_NETINTERFACE elements)
     * @param index - index of element
     * @returns description
     */
    public String getElementNetInterfaceDescription(int index) { 
        byte[] v = OutputParm(index+1);
        // skip type, name, state
        int off =  IntelByteBuffer.SkipCompUnicode(v, 4) + 4;
        
        return readCompUnicodeOutputParam(index+1, 
            IntelByteBuffer.SkipCompUnicode(v, off)); 
    }
    /**
     * Get group state (for CLUSTER_ENUM_GROUP elements)
     * @param index - index of element
     * @returns state (ClusterGroup*)
     */
    public long getElementGroupState(int index) { 
        return readDWORDOutputParam(index+1, 
            IntelByteBuffer.SkipCompUnicode(OutputParm(index+1), 4)); 
    }
    /**
     * Get current node name for group (for CLUSTER_ENUM_GROUP elements)
     * @param index - index of element
     * @returns name of current node for group
     */
    public String getElementGroupNodeName(int index) { 
        return readCompUnicodeOutputParam(index+1, 
            IntelByteBuffer.SkipCompUnicode(OutputParm(index+1), 4) + 4); 
    }
    /**
     * Get resource list for group (for CLUSTER_ENUM_GROUP elements)
     * @param index - index of element
     * @returns list of resources for group
     */
    public String[] getElementGroupResourceList(int index) { 
        byte[] p = OutputParm(index+1);
        int off = IntelByteBuffer.SkipCompUnicode(p, 4) + 4;
        off = IntelByteBuffer.SkipCompUnicode(p, off);
        int cnt = IntelByteBuffer.ReadLONG(p, off);
        off += 4;
        return readCompUnicodeListOutputParam(index+1, cnt, off);
    }
    /**
     * Get preferred owner list for group (for CLUSTER_ENUM_GROUP elements)
     * @param index - index of element
     * @returns list of preferred owner list for group
     */
    public String[] getElementGroupPreferredOwnerList(int index) { 
        byte[] p = OutputParm(index+1);
        int off = IntelByteBuffer.SkipCompUnicode(p, 4) + 4;
        off = IntelByteBuffer.SkipCompUnicode(p, off);
        int cnt = IntelByteBuffer.ReadLONG(p, off);
        off += 4;
        for(int i = 0; i < cnt; i++)
            off = IntelByteBuffer.SkipCompUnicode(p, off);
        cnt = IntelByteBuffer.ReadLONG(p, off);
        off += 4;
        return readCompUnicodeListOutputParam(index+1, cnt, off);
    }
    /**
     * Get description for group (for CLUSTER_ENUM_GROUP elements)
     * @param index - index of element
     * @returns description for group
     */
    public String getElementGroupDescription(int index) { 
        byte[] p = OutputParm(index+1);
        int off = IntelByteBuffer.SkipCompUnicode(p, 4) + 4;
        off = IntelByteBuffer.SkipCompUnicode(p, off);
        int cnt = IntelByteBuffer.ReadLONG(p, off);
        off += 4;
        for(int i = 0; i < cnt; i++)
            off = IntelByteBuffer.SkipCompUnicode(p, off);
        cnt = IntelByteBuffer.ReadLONG(p, off);
        off += 4;
        for(int i = 0; i < cnt; i++)
            off = IntelByteBuffer.SkipCompUnicode(p, off);
        return readCompUnicodeOutputParam(index+1, off);
    }
    /**
     * Get resource state (for CLUSTER_ENUM_RESOURCE elements)
     * @param index - index of element
     * @returns state (ClusterResource*)
     */
    public long getElementResourceState(int index) { 
        return readDWORDOutputParam(index+1, 
            IntelByteBuffer.SkipCompUnicode(OutputParm(index+1), 4)); 
    }
    /**
     * Get resource owner node (for CLUSTER_ENUM_RESOURCE elements)
     * @param index - index of element
     * @returns owner node of resource
     */
    public String getElementResourceOwnerNode(int index) { 
        byte[] v = OutputParm(index+1);
        int off =  IntelByteBuffer.SkipCompUnicode(v, 4) + 4; 
        return IntelByteBuffer.ReadCompUnicode(v, off);
    }
    /**
     * Get resource current owner (for CLUSTER_ENUM_RESOURCE elements)
     * @param index - index of element
     * @returns current owner of resource
     */
    public String getElementResourceCurrentOwner(int index) { 
        byte[] v = OutputParm(index+1);
        int off =  IntelByteBuffer.SkipCompUnicode(v, 4) + 4; 
        off = IntelByteBuffer.SkipCompUnicode(v, off);
        return IntelByteBuffer.ReadCompUnicode(v, off);
    }
    /**
     * Get resource dependent network (for CLUSTER_ENUM_RESOURCE elements)
     * @param index - index of element
     * @returns dependent network of resource
     */
    public String getElementResourceDependentNetwork(int index) { 
        byte[] v = OutputParm(index+1);
        int off =  IntelByteBuffer.SkipCompUnicode(v, 4) + 4; 
        off = IntelByteBuffer.SkipCompUnicode(v, off);
        off = IntelByteBuffer.SkipCompUnicode(v, off);
        return IntelByteBuffer.ReadCompUnicode(v, off);
    }
    /**
     * Get list of dependencies of resource (for CLUSTER_ENUM_RESOURCE elements)
     * @param index - index of element
     * @returns list of dependencies of the resource
     */
    public String[] getElementResourceDependenciesList(int index) { 
        byte[] v = OutputParm(index+1);
        int off =  IntelByteBuffer.SkipCompUnicode(v, 4) + 4; 
        off = IntelByteBuffer.SkipCompUnicode(v, off);
        off = IntelByteBuffer.SkipCompUnicode(v, off);
        off = IntelByteBuffer.SkipCompUnicode(v, off);
        int cnt = IntelByteBuffer.ReadLONG(v, off);
        off += 4;
        return readCompUnicodeListOutputParam(index+1, cnt, off);
    }
    /**
     * Get list of dependents of resource (for CLUSTER_ENUM_RESOURCE elements)
     * @param index - index of element
     * @returns list of dependents of the resource
     */
    public String[] getElementResourceDependentsList(int index) { 
        byte[] v = OutputParm(index+1);
        int off =  IntelByteBuffer.SkipCompUnicode(v, 4) + 4; 
        off = IntelByteBuffer.SkipCompUnicode(v, off);
        off = IntelByteBuffer.SkipCompUnicode(v, off);
        off = IntelByteBuffer.SkipCompUnicode(v, off);
        int cnt = IntelByteBuffer.ReadLONG(v, off);
        off += 4;
        for(int i = 0; i < cnt; i++)
            off = IntelByteBuffer.SkipCompUnicode(v, off);
        cnt = IntelByteBuffer.ReadLONG(v, off);
        off += 4;
        return readCompUnicodeListOutputParam(index+1, cnt, off);
    }
    /**
     * Get list of hosting nodes of resource (for CLUSTER_ENUM_RESOURCE elements)
     * @param index - index of element
     * @returns list of hosting nodes of the resource
     */
    public String[] getElementResourceHostNodesList(int index) { 
        byte[] v = OutputParm(index+1);
        int off =  IntelByteBuffer.SkipCompUnicode(v, 4) + 4; 
        off = IntelByteBuffer.SkipCompUnicode(v, off);
        off = IntelByteBuffer.SkipCompUnicode(v, off);
        off = IntelByteBuffer.SkipCompUnicode(v, off);
        int cnt = IntelByteBuffer.ReadLONG(v, off);
        off += 4;
        for(int i = 0; i < cnt; i++)
            off = IntelByteBuffer.SkipCompUnicode(v, off);
        cnt = IntelByteBuffer.ReadLONG(v, off);
        off += 4;
        for(int i = 0; i < cnt; i++)
            off = IntelByteBuffer.SkipCompUnicode(v, off);
        cnt = IntelByteBuffer.ReadLONG(v, off);
        off += 4;
        return readCompUnicodeListOutputParam(index+1, cnt, off);
    }
    /**
     * Get resource type of resource (for CLUSTER_ENUM_RESOURCE elements)
     * @param index - index of element
     * @returns resource type name
     */
    public String getElementResourceType(int index) { 
        byte[] v = OutputParm(index+1);
        int off =  IntelByteBuffer.SkipCompUnicode(v, 4) + 4; 
        off = IntelByteBuffer.SkipCompUnicode(v, off);
        off = IntelByteBuffer.SkipCompUnicode(v, off);
        off = IntelByteBuffer.SkipCompUnicode(v, off);
        int cnt = IntelByteBuffer.ReadLONG(v, off);
        off += 4;
        for(int i = 0; i < cnt; i++)
            off = IntelByteBuffer.SkipCompUnicode(v, off);
        cnt = IntelByteBuffer.ReadLONG(v, off);
        off += 4;
        for(int i = 0; i < cnt; i++)
            off = IntelByteBuffer.SkipCompUnicode(v, off);
        cnt = IntelByteBuffer.ReadLONG(v, off);
        off += 4;
        for(int i = 0; i < cnt; i++)
            off = IntelByteBuffer.SkipCompUnicode(v, off);
        return IntelByteBuffer.ReadCompUnicode(v, off);
    }
    /**
     * Get description of resource (for CLUSTER_ENUM_RESOURCE elements)
     * @param index - index of element
     * @returns resource description
     */
    public String getElementResourceDescription(int index) { 
        byte[] v = OutputParm(index+1);
        int off =  IntelByteBuffer.SkipCompUnicode(v, 4) + 4; 
        off = IntelByteBuffer.SkipCompUnicode(v, off);
        off = IntelByteBuffer.SkipCompUnicode(v, off);
        off = IntelByteBuffer.SkipCompUnicode(v, off);
        int cnt = IntelByteBuffer.ReadLONG(v, off);
        off += 4;
        for(int i = 0; i < cnt; i++)
            off = IntelByteBuffer.SkipCompUnicode(v, off);
        cnt = IntelByteBuffer.ReadLONG(v, off);
        off += 4;
        for(int i = 0; i < cnt; i++)
            off = IntelByteBuffer.SkipCompUnicode(v, off);
        cnt = IntelByteBuffer.ReadLONG(v, off);
        off += 4;
        for(int i = 0; i < cnt; i++)
            off = IntelByteBuffer.SkipCompUnicode(v, off);
        off = IntelByteBuffer.SkipCompUnicode(v, off);
        return IntelByteBuffer.ReadCompUnicode(v, off);
    }
    /**
     * Get resource type resource dll (for CLUSTER_ENUM_RESTYPE elements)
     * @param index - index of element
     * @returns resource dll of resource
     */
    public String getElementResourceTypeResourceDLL(int index) { 
        byte[] v = OutputParm(index+1);
        // skip name, type
        int off =  IntelByteBuffer.SkipCompUnicode(v, 4);
        return IntelByteBuffer.ReadCompUnicode(v, off);
    }
    /**
     * Get resource type description (for CLUSTER_ENUM_RESTYPE elements)
     * @param index - index of element
     * @returns description of resource
     */
    public String getElementResourceTypeDescription(int index) { 
        byte[] v = OutputParm(index+1);
        // skip name, type
        int off =  IntelByteBuffer.SkipCompUnicode(v, 4);
        // skip res dll
        off =  IntelByteBuffer.SkipCompUnicode(v, off); 
        return IntelByteBuffer.ReadCompUnicode(v, off);
    }

}
