package com.tivoli.twg.mscs;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Locale;
import java.util.Enumeration;

import com.tivoli.twg.engine.cluster.TWGClusterManagedObject;
import com.tivoli.twg.engine.*;
import com.tivoli.twg.libs.DataValue;
import com.tivoli.twg.libs.IntelByteBuffer;
import com.tivoli.twg.libs.IntValueSet;
import com.tivoli.twg.libs.MultiLocaleBundle;
import com.tivoli.twg.libs.StringValueSet;
import com.tivoli.twg.libs.TWGString;
import com.tivoli.twg.libs.TWGStringArray;
import com.tivoli.twg.log.TWGRas;
import com.tivoli.twg.log.TWGOutput;

/**
 * Microsoft Cluster Services cluster managed object.
 * @author Mike Primm
 * @version 1.1 - 6/8/05
 */
public class MSCSManagedObject extends TWGClusterManagedObject {
    public static final String CLASSNAME = "com.tivoli.twg.mscs.MSCSManagedObject";

    /** Hashtable of cluster objects, hashed by cluster name                                */
    private static Hashtable clusters = new Hashtable();
    /** Hashtable of cluster objects, hashed by cluster resource ID         */
    private static Hashtable clusters_by_resid = new Hashtable();
    /* Name of cluster                                                                      */
    private String cluster_name;
    /* Version of cluster services                                          */
    private String cluster_version;
    /* Cluster resource ID (non-changing ID)                                */
    private String cluster_resid;
    /* Group error in cluster                                               */
    private boolean has_group_error;
    /* Object IDs of good cluster members (ones which successfully executed */
    /* refresh last time requested)                                         */
    private IntValueSet good_member_ids;
    private int last_preferred_node = 0;
    /* Last preferred path value returned                                                   */
    private String last_preferred = null;
    private boolean path_updated = false;
    /** Persistent record format
     * ULONG hdrlen
     * ULONG off_clustername
     * ULONG off_clusversion
     * ULONG off_clusterresid
     */
    public static final int BLK_HDRLEN_OFF = 0;
    public static final int BLK_CLUSTERNAMEOFF_OFF = 4;
    public static final int BLK_CLUSTERVERSIONOFF_OFF = 8;
    public static final int BLK_CLUSTERRESIDOFF_OFF = 12;
    public static final int BLK_HDRLEN = 16;
    public static final int BLK_HDRLEN_000 = 8;
    public static final int BLK_HDRLEN_001 = 12;

    /** List of supported attribute IDs                                     */
    private static final String[] attrib_ids = {
        // current best IPC path to client
        MSCSManagedObjectConstants.ATTRIB_IPCPATH,
        // all IPC paths to client
        MSCSManagedObjectConstants.ATTRIB_ALLIPCPATHS,
        // Cluster version
        MSCSManagedObjectConstants.ATTRIB_VERSION,
        // Cluster name resource ID
        MSCSManagedObjectConstants.ATTRIB_CLUSTER_RESID
    };
    /** List of attribute types                                             */
    private static final int[] attrib_types = {
        DataValue.STRING_TYPE,  // current best IPC path to client
        DataValue.STRING_ARRAY_TYPE,   // all IPC paths to client
        DataValue.STRING_TYPE,  // Cluster Version
        DataValue.STRING_TYPE   // Cluster name resource ID
    };
    /** List of string IFs for attribute labels                             */
    private static final String[] attrib_labels = {
        // current best IPC path to client
        "attrib.lbl.ipc_path",
        // all IPC paths to client
        "attrib.lbl.all_ipc_paths",
        // Cluster version
        "attrib.lbl.mscs_version",
        // Cluster name resource IDs
        "attrib.lbl.mscs_resid"
    };

    /** MSCS NLS bundle                                                    */
    private static MultiLocaleBundle nls_bundle =
        MultiLocaleBundle.getSharedBundle("com.tivoli.twg.mscs.MSCSNLS");

    /**
     * Default constructor for cluster managed object
     */
    public MSCSManagedObject() {
        super();
        setConsoleDeletable(true);      /* All MSCS MO's are console deletable      */
        cluster_version = "";
        cluster_resid = "";
        has_group_error = false;
        good_member_ids = new IntValueSet();
    }
    /**
     * Constructor for creating new cluster
     * @param name - name of cluster
     * @param resid - resource ID of cluster
     * @param init_moids - initial MOIDs which are member of cluster
     * @exception TWGPersistentObjectSaveException if error creating object
     */
    public MSCSManagedObject(String name, String resid, IntValueSet init_moids) throws
        TWGPersistentObjectSaveException {
        super(init_moids);
        setConsoleDeletable(true);      /* All MSCS MO's are console deletable      */
        setName(name);
        cluster_name = name;
        cluster_version = "";
        cluster_resid = resid;
        has_group_error = false;
        good_member_ids = new IntValueSet();
        setSaveRequired();
    }
    /**
     * Copy instance data to another managed object.
     * @param managedObject target managed object.
     */
    protected void copyTo(TWGManagedObject managedObject) {
        super.copyTo(managedObject);
        if (managedObject instanceof MSCSManagedObject) {
            ((MSCSManagedObject)managedObject).cluster_name = cluster_name;
            ((MSCSManagedObject)managedObject).cluster_resid = cluster_resid;
            ((MSCSManagedObject)managedObject).good_member_ids = new IntValueSet(good_member_ids);
            ((MSCSManagedObject)managedObject).cluster_name = cluster_name;
            ((MSCSManagedObject)managedObject).cluster_version = cluster_version;
            ((MSCSManagedObject)managedObject).cluster_resid = cluster_resid;
            ((MSCSManagedObject)managedObject).has_group_error = has_group_error;
        }
    }
    /**
     * Destructor for MSCSManagedObject: remove object from lists
     */
    public void Delete() {
        clusters_by_resid.remove(cluster_resid);
        clusters.remove(cluster_name);
        super.Delete();
    }
    /**
     * Destroy for MSCSManagedObject
     * @exception TWGObjectDestroyException if error during destroy
     */
    public void Destroy() throws TWGObjectDestroyException {
        super.Destroy();
    }
    /**
     * Save support for persistant object. This method saves all managed
     * object fields that are persistant in nature.
     * @exception TWGPersistentObjectSaveException if error on save
     */
    protected void saveData( TWGPersistentObjectDictionary dictionary )
        throws TWGPersistentObjectSaveException {
        /* Compute length needed                                                            */
        int len = BLK_HDRLEN + IntelByteBuffer.GetCompUnicodeLength(cluster_name) +
            IntelByteBuffer.GetCompUnicodeLength(cluster_version) + 
            IntelByteBuffer.GetCompUnicodeLength(cluster_resid);
        byte[] buf = new byte[len];
        /* Add fields to block                                                              */
        int off = BLK_HDRLEN;
        IntelByteBuffer.WriteLONG(buf, BLK_HDRLEN, BLK_HDRLEN_OFF);
        IntelByteBuffer.WriteLONG(buf, off, BLK_CLUSTERNAMEOFF_OFF);
        off = IntelByteBuffer.WriteCompUnicode(buf, cluster_name, off);
        IntelByteBuffer.WriteLONG(buf, off, BLK_CLUSTERVERSIONOFF_OFF);
        off = IntelByteBuffer.WriteCompUnicode(buf, cluster_version, off);
        IntelByteBuffer.WriteLONG(buf, off, BLK_CLUSTERRESIDOFF_OFF);
        off = IntelByteBuffer.WriteCompUnicode(buf, cluster_resid, off);
        /* Save our blob in the persistant store.                                           */
        dictionary.put(CLASSNAME, buf);
        /* Give our super classes a chance to save data.                                    */
        super.saveData( dictionary );
    }
    /**
     * Restore support for persistant object. This method restores managed
     * object fields that are persistant in nature.
     * @exception TWGPersistentObjectRestoreException if restore error
     */
    protected void restoreData( TWGPersistentObjectDictionary dictionary,
        boolean resolveObjectReferences ) throws
        TWGPersistentObjectRestoreException {
        byte[] buf;

        // Allow our super class to restore its data first.
        super.restoreData( dictionary, resolveObjectReferences );

        /* If first pass, we're loading our data                            */
        if(resolveObjectReferences == false) {
            // Get our data blob from which we will restore.
            buf = dictionary.get(CLASSNAME);
            // If no record, error
            if(buf == null)
                throw new TWGPersistentObjectRestoreException("no data");
            // Read length of header
            int hdrlen = IntelByteBuffer.ReadLONG(buf, BLK_HDRLEN_OFF);
            // If too short for minimum record, return error
            if(hdrlen < BLK_HDRLEN_000) {
                throw new TWGPersistentObjectRestoreException("too short");
            }
            int off;
            if(hdrlen > BLK_HDRLEN_000) {
                off = IntelByteBuffer.ReadLONG(buf, BLK_CLUSTERVERSIONOFF_OFF);
                cluster_version = IntelByteBuffer.ReadCompUnicode(buf, off);
            } else {
                cluster_version = "";
            }
            if(hdrlen > BLK_HDRLEN_001) {
                off = IntelByteBuffer.ReadLONG(buf, BLK_CLUSTERRESIDOFF_OFF);
                cluster_resid = IntelByteBuffer.ReadCompUnicode(buf, off);
            } else {
                cluster_resid = "";
            }
            off = IntelByteBuffer.ReadLONG(buf, BLK_CLUSTERNAMEOFF_OFF);
            cluster_name = IntelByteBuffer.ReadCompUnicode(buf, off);
            if(cluster_resid.length() > 0)
                clusters_by_resid.put(cluster_resid, this);
            clusters.put(cluster_name, this);
        }
    } // TWGManagedObject.restoreData  -  END
    /**
     * Set object ID. This method subclasses the TWGObject class's
     * SetObjectID. This call is needed so that Managed object class
     * instances can be registered with the appropriate TreeNode instances
     * within the ManagedObjectFactory.
     * @param oid - new object ID, or INVALID_OBJECT_ID if none
     * @exception DuplicateObjectIDException if ID is duplicate
     */
    public void SetObjectID(long oid) throws DuplicateObjectIDException {
        long old_oid = -1;

        if(isValid()) {
            try {
                // Get our object ID.
                old_oid = ObjectID();
                /* Remove node from cluster list                                            */
                if(cluster_resid.length() > 0)
                    clusters_by_resid.remove(cluster_resid);
                clusters.remove(cluster_name);
            } catch(InvalidObjectIDException ioidx) {
            }
        }
        // Process the new object ID.
        try {
            // Allow super to perform.
            super.SetObjectID(oid);
            /* If new one is valid, add to cluster list                                     */
            if(isValid()) {
                /* Remove node from cluster list                                            */
                if(cluster_resid.length() > 0)
                    clusters_by_resid.put(cluster_resid, this);
                clusters.put(cluster_name, this);
            }
        } catch (DuplicateObjectIDException doidx) {
            if(isValid() && (old_oid != -1)) {
                /* Remove node from cluster list                                            */
                if(cluster_resid.length() > 0)
                    clusters_by_resid.put(cluster_resid, this);
                clusters.put(cluster_name, this);
            }
            throw doidx;
        }
    }
    /**
     * Find MSCS cluster instance by name
     * @param cluster_name - Cluster Name
     * @returns MSCSManagedObject, or null if not found
     */
    public static MSCSManagedObject findMSCSObjectByName(String cluster_name) {
        return (MSCSManagedObject)clusters.get(cluster_name);
    }
    /**
     * Find MSCS cluster instance by resource ID
     * @param cluster_resid - Cluster resource ID
     * @returns MSCSManagedObject, or null if not found
     */
    public static MSCSManagedObject findMSCSObjectByNameResID(String cluster_resid) {
        return (MSCSManagedObject)clusters_by_resid.get(cluster_resid);
    }
    /**
     * Update cluster state based on config command content
     * @param cmd - TWGGetFullConfigCommand returned from agent in cluster
     */
    void updateMSCSCluster(MSCSGetFullConfigCommand cmd) {
        //TWGOutput.println(getClusterName()+".updateMSCSCluster(), rc=" + Long.toHexString(cmd.ReturnCode()));
        if(cmd.ReturnCode() == 0) {
            boolean updated = false;
            /* Update version information                                                   */
            String v = cmd.getClusterMajorVersion() + "." + cmd.getClusterMinorVersion() +
                "." + cmd.getClusterBuildNumber();
            if(!v.equals(cluster_version)) {    /* If changed, update value                 */
                cluster_version = v;
                updated = true;
            }
            String cname = cmd.getClusterName();
            setName(cname);     /* Update name, if changed                                  */
            /* If cluster renamed, fix hashing and name variable                            */
            if(!cname.equals(cluster_name)) {
                clusters.remove(cluster_name);
                cluster_name = cname;
                clusters.put(cluster_name, this);
                updated = true;
            }
            if(cmd.getClusterNameResource().equals(cluster_resid) == false) {
                if(cluster_resid.length() > 0) clusters_by_resid.remove(cluster_resid);
                cluster_resid = cmd.getClusterNameResource();
                if(cluster_resid.length() > 0) clusters_by_resid.put(cluster_resid, this);
                updated = true;
            }
            /* Update state, if available                                                   */
            boolean newerr = false;
            int cnt = cmd.getNumberOfElements();
            StringValueSet nodenames = new StringValueSet();
            for(int i = 0; i < cnt; i++) {
                /* For each group in the cluster                                            */
                if(cmd.getElementType(i) == cmd.CLUSTER_ENUM_GROUP) {
                    long grpstate = cmd.getElementGroupState(i);
                    /* If error, set error state                                            */
                    if((grpstate == cmd.ClusterGroupFailed) ||
                        (grpstate == cmd.ClusterGroupPartialOnline)) {  
                        newerr = true;
                    }
                }
                /* Accumulate nodes                                                         */
                else if(cmd.getElementType(i) == cmd.CLUSTER_ENUM_NODE) {
                    nodenames.InsertValue(cmd.getElementName(i));
                }
            }
            /* For each node, see if its in the list of nodes (for evicts)                  */
            int[] ids = getMemberIDs();
            for(int i = 0; i < ids.length; i++) {
                TWGObject o = TWGObject.FindObject(TWGPersistentObject.fromPersistID(ids[i]));
                if((o != null) && (o instanceof TWGNativeManagedObject)) {
                    TWGNativeManagedObject nmo = (TWGNativeManagedObject)o;
                    String n = nmo.getComputerName();
                    /* If name not found in node list, drop node                            */
                    if((n.length() > 0) && (nodenames.Find(n) == -1)) {
                        try { removeMember(ids[i]); } catch (InvalidObjectIDException ioidx) {}
                    }
                }
            }
            has_group_error = newerr;
            int new_state;
            boolean did_new_state = false;
            if(has_group_error) {
                new_state = TWGManagedObjectConstants.MOSTATE_ERROR_ONLINE;
            } else {
                new_state = TWGManagedObjectConstants.MOSTATE_NORMAL_ONLINE;
            }
            if(getState() != new_state) {
                setState(new_state);
                did_new_state = true;
                path_updated = false;
            }
            /* If any changes, save and notify                                              */
            if(updated) {
                setSaveRequired();
                try {
                    save();
                } catch (TWGPersistentObjectSaveException posx) {
                    TWGRas.error(TWGRas.ENGINE, "Error saving MSCSManagedObject");
                }
                ObjectChangedNotify(TWGManagedObjectConstants.CHANGED_ATTRIBUTE);
                path_updated = false;
            }
            /* If no state or lifecycle change notified, check for IPC path change          */
            if((!updated) && (!did_new_state)) {
                String pp = getPreferredPath();
                if(path_updated) {
                    path_updated = false;
                    ObjectAddressEntryChangeNotify();
                }
            }
            if(updated || did_new_state) {
                TWGActiveConsole.objectUpdated(this);
            }
        } 
    }
    private static String[] proto_priority = null;
    /**
     * Get current preferred IPC path from server to client
     * @return Preferred IPC path
     */
    public String getPreferredPath() { 
        TWGNativeManagedObject obj = getPreferredNode();
        String preferred = null;
        if(obj != null) {
            preferred = obj.getPreferredPath();
        }
        /* If different from last value returned                                            */
        if(((preferred == null) && (last_preferred != null)) ||
           ((preferred != null) && (last_preferred == null)) ||
           ((preferred != null) && (last_preferred != null) && (!preferred.equals(last_preferred)))) {
           last_preferred = preferred;
           path_updated = true;
        }
        return preferred;
    }
    /**
     * Get current preferred node to access cluster thorough
     * @returns best native MO node, or null if none available
     */
    public TWGNativeManagedObject getPreferredNode() {
        int i, j;
        /* Initialize protocol priority list first time called              */
        if(proto_priority == null) {
            /* Get protocol priority list                                   */
            String list = TWGConfigProperties.get(TWGConfigProperties.
                PROTOCOL_PRIORITY_LIST);
            list = list.trim();
            ArrayList v = new ArrayList();
            while(list.length() > 0) {
                int space = list.indexOf(' ');
                if(space > 0) {
                    v.add(list.substring(0, space));
                    list = list.substring(space+1).trim();
                } else {
                    v.add(list);
                    list = "";
                }
            }
            proto_priority = new String[v.size()];
            for(i = 0; i < v.size(); i++) {
                proto_priority[i] = (String)v.get(i);
            }
        }
        /* Get all available IPC paths for online, native managed objects   */
        TWGNativeManagedObject best = null;
        int best_index = proto_priority.length;
        int[] ids = getMemberIDs();
        for(i = 0; i < ids.length; i++) {
            if(good_member_ids.Find(ids[i]) == -1) {
                continue;
            }
            TWGManagedObject mo = TWGManagedObject.getManagedObjectByID(ids[i]);
            /* Skip missing or non-native MOs                               */
            if((mo == null) || (!(mo instanceof TWGNativeManagedObject))) {
                continue;
            }
            TWGNativeManagedObject nmo = (TWGNativeManagedObject)mo;
            String path = nmo.getPreferredPath();
            if(path == null)
                continue;
            /* Get protocol of path                                         */
            String proto = path.substring(0, path.indexOf("::"));
            /* See if its protocol is better than our current best          */
            if(proto != null) {
                for(j = 0; j < best_index; j++) {
                    if(proto_priority[j].equals(proto)) {
                        best_index = j;
                        best = nmo;
                    }
                }
            }
        }
        /* Keep track of last best answer                                   */
        if(best != null) {
            try {
                last_preferred_node = best.PersistID();
            } catch (InvalidObjectIDException ioidx) {
                best = null;
                last_preferred_node = 0;
            }
        } else {
            last_preferred_node = 0;
        }
        return best;
    }
    /**
     * Return all available IPC paths
     * @return list of IPC paths
     */
    public String[] GetAvailablePaths() {
        int i, cnt;
        StringValueSet list = new StringValueSet();
        int[] members = getMemberIDs();
   
        /* Go through all members                                           */
        for(i = 0; i < members.length; i++) {
            if(good_member_ids.Find(members[i]) == -1) {
                continue;
            }
            TWGManagedObject mo = TWGManagedObject.getManagedObjectByID(members[i]);
            if(mo != null) {
                DataValue val = mo.getAttributeValue(
                    TWGNativeManagedObjectConstants.ATTRIB_ALLIPCPATHS, 
                    Locale.getDefault());
                if((val != null) && (val instanceof TWGStringArray)) {
                    String[] v = ((TWGStringArray)val).getValue();
                    list.InsertArray(v, 0, v.length);
                }
            }
        }
        return list.GetValues();
    }
    /*
     * Attribute ID enumeration : this method is expected to return a
     * List of strings containing the String names of the attributes
     * supported for a given object.  Subclasses of base classes implementing
     * the interface should report additional attributes by adding them
     * to the list returned by the base class.
     * @returns list of attribute ID strings
     */
    public String[]     getAttributeIDList() {
        String[] s = super.getAttributeIDList();
        String[] new_s = new String[s.length + attrib_ids.length];
        System.arraycopy(s, 0, new_s, 0, s.length);
        System.arraycopy(attrib_ids, 0, new_s, s.length, attrib_ids.length);
        return new_s;
    }
    /**
     * Get value of attribute with given ID
     * @param id - attribute ID requested
     * @param loc - locale to use for value, if applicable
     * @return value object, or null if no value
     */
    public DataValue   getAttributeValue(String id, Locale loc) {
        if(id.equals(attrib_ids[0])) {  /* If request for path, return path */
            String pth = getPreferredPath();
            if(path_updated) {
                ObjectAddressEntryChangeNotify();
                path_updated = false;
            }
            if(pth != null)
                return new TWGString(pth);
            else
                return null;
        }
        else if(id.equals(attrib_ids[1])) {     /* If request for all paths,*/
            return new TWGStringArray(GetAvailablePaths());
        }
        else if(id.equals(attrib_ids[2])) {     /* If request for version   */
            return new TWGString(cluster_version);
        }
        else if(id.equals(attrib_ids[3])) {     /* If request for resource ID*/
            return new TWGString(cluster_resid);
        }
        else
            return super.getAttributeValue(id, loc);
    }
    /**
     * Set value of attribute with given ID
     * @param id - attribute ID to be set
     * @param val - attribute value to be set
     * @return true if set successful, false if not successful
     */
    public boolean     setAttributeValue(String id, DataValue val) {
        return super.setAttributeValue(id, val);
    }
    /**
     * Get type of attribute value for given attribute
     * @param id - attribute ID
     * @return DataValue type of attribute, or DataValue.NONE_TYPE if
     * undefined
     */
    public int getAttributeType(String id) {
        for(int i = 0; i < attrib_types.length; i++) {
            if(id.equals(attrib_ids[i])) {
                return attrib_types[i];
            }
        }
        return super.getAttributeType(id);
    }
    /**
     * Get description string for given attribute in given locale
     * @param id - attribute ID
     * @param loc - locale to use for description
     * @return description string, or null if not available
     */
    public String getAttributeDescription(String id, Locale loc) {
        for(int i = 0; i < attrib_types.length; i++) {
            if(id.equals(attrib_ids[i])) {
                return nls_bundle.getString(loc, attrib_labels[i],
                    attrib_labels[i]);
            }
        }
        return super.getAttributeDescription(id, loc);
    }
    /**
     * Get descriptive value string for given attribute in given locale
     * @param id - attribute ID
     * @param loc - locale to use for description
     * @return descriptive string for attribute value, or null if not available
     */
    public String getAttributeValueString(String id, Locale loc) {
        return super.getAttributeValueString(id, loc);
    }
    /**
     * Start a ping on this cluster.  This refreshes the cluster's configuration and
     * state.
     */
    public boolean startPing() {
        int i, cnt;
        StringValueSet list = new StringValueSet();
        int[] members = getMemberIDs();
        boolean attempted = false;
   
        /* Go through all members                                           */
        for(i = 0; i < members.length; i++) {
            TWGManagedObject mo = TWGManagedObject.getManagedObjectByID(members[i]);
            if((mo != null) && (mo instanceof TWGNativeManagedObject))  {
                TWGNativeManagedObject nmo = (TWGNativeManagedObject)mo;
                String pth = nmo.getPreferredPath();
                if(pth != null) {
                    MSCSManagedObjectFactory.startClusterValidation(nmo, this);
                    attempted = true;
                }
            }
        }
        if(!attempted) {
            int new_state;
            /* If we had a group error last time we communicated, report error*/
            if(has_group_error) {
                new_state = TWGManagedObjectConstants.MOSTATE_ERROR_OFFLINE;
            } 
            else {      /* Else, normal offline                              */
                new_state = TWGManagedObjectConstants.MOSTATE_NORMAL_OFFLINE;
            }
            if(getState() != new_state) {
                path_updated = false;
                setState(new_state);
                TWGActiveConsole.objectUpdated(this);
            } else {    /* Check for IPC path change                         */
                String pp = getPreferredPath();
                if(path_updated) {
                    path_updated = false;
                    ObjectAddressEntryChangeNotify();
                }
            }
        }
        return true;       
    }
    /**
     * Get name of cluster
     * @returns cluster name
     */
    public String getClusterName() {
        return cluster_name;
    }
    /**
     * Get cluster resource ID
     * @returns cluster resource ID
     */
    public String getClusterResourceID() {
        return cluster_resid;
    }
    /**
     * Get version of cluster services : format is "major.minor.build"
     * @returns cluster services version
     */
    public String getClusterServicesVersion() {
        return cluster_version;
    }
    /**
     * Find cluster containing given object
     * @param obj - object contained in MSCS cluster
     * @return cluster found, or null if not found
     */
    public static MSCSManagedObject findClusterByObject(TWGManagedObject obj) {
        Enumeration enum = clusters.elements();
        while(enum.hasMoreElements()) {
            MSCSManagedObject mo = (MSCSManagedObject)enum.nextElement();
            if(mo.isMember(obj)) {
                return mo;
            }
        }
        return null;
    }
    /**
     * Mark given node as bad (don't use for agent communications)
     * @param node - managed object of node
     */
    public void setNodeBad(TWGManagedObject node) {
        try {
            setNodeBad(node.PersistID());
        } catch (InvalidObjectIDException ioidx) {
        }
    }
    /**
     * Mark given node as bad (don't use for agent communications)
     * @param id - persistent ID of node
     */
    public void setNodeBad(int id) {
        good_member_ids.RemoveValue(id);
        if(last_preferred_node == id) { /* If was best              */
            last_preferred_node = 0;    /* Drop it                  */
            if(getPreferredNode() == null) {    /* If no-one left   */
                /* If we had a group error last time we communicated, report error*/
                int new_state;
                if(has_group_error) {
                    new_state = TWGManagedObjectConstants.MOSTATE_ERROR_OFFLINE;
                }  
                else {  /* Else, normal offline                     */
                    new_state = TWGManagedObjectConstants.MOSTATE_NORMAL_OFFLINE;
                }
                if(getState() != new_state) {
                    setState(new_state);
                    path_updated = false;
                    TWGActiveConsole.objectUpdated(this);
                } else {
                    String pp = getPreferredPath();
                    if(path_updated) {
                        path_updated = false;
                        ObjectAddressEntryChangeNotify();
                    }
                } 
                startPing();            /* Try a ping to find one   */
            }
        }
    }
    /**
     * Mark given node as good (use for agent communications)
     * @param node - managed object of node
     */
    public void setNodeGood(TWGManagedObject node) {
        try {
            if(isMember(node)) {
                good_member_ids.InsertValue(node.PersistID());
                if(last_preferred_node == 0) {  /* If no good node known    */
                    last_preferred_node = node.PersistID();     /* Use this */
                }
            }
        } catch (InvalidObjectIDException ioidx) {
        }
    }
    /**
     * Remove member managed object from cluster : 
     * @param rem_moid - Persistent ID of managed object to be removed
     * @return true if was member, false if was not
     */
    public boolean removeMember(int rem_moid) throws InvalidObjectIDException {
        boolean was_member = super.removeMember(rem_moid);
        if(was_member) {
            //TWGOutput.println("removeMember(" + rem_moid + ") for " + getName());
            setNodeBad(rem_moid);
            ObjectChangedNotify(TWGManagedObjectConstants.CHANGED_ATTRIBUTE);
        }
        return was_member;
    }
    /**
     * Get computer name of cluster node
     * @param id - persistent ID of node 
     * @return computer name, or "" if not found/available
     */
    public String getClusterNodeName(int id) {
        TWGObject o = TWGObject.FindObject(TWGPersistentObject.fromPersistID(id));
        if((o != null) && (o instanceof TWGNativeManagedObject)) {
            return ((TWGNativeManagedObject)o).getComputerName();
        } else {
            return "";
        }
    }
}
