//*****************************************************************************
// (C) Copyright 2000 by IBM
//*****************************************************************************
package com.BobCo.monitors;

import  com.tivoli.twg.monitor.TWGMonAttributePath;
import  com.tivoli.twg.monitor.TWGMonAttributeNode;
import  com.tivoli.twg.monitor.TWGMonAttributeNodeList;
import  com.tivoli.twg.monitor.TWGMonEnumStrings;
import  com.tivoli.twg.monitor.TWGMonAttributeDataSink;
import  com.tivoli.twg.monitor.TWGMonAttributeMetaDataSink;
import  com.tivoli.twg.monitor.TWGMonConstants;
import  com.tivoli.twg.monitor.TWGMonCurrentData;
import  com.tivoli.twg.monitor.TWGMonIndepCmd;
import  com.tivoli.twg.monitor.TWGMonMetaData;
import  com.tivoli.twg.monitor.TWGMonMOAddressInfoProvider;
import  com.tivoli.twg.monitor.TWGMonMOAttributeInfoProvider;
import  com.tivoli.twg.monitor.TWGMonMOStateInfoProvider;
import  com.tivoli.twg.monitor.TWGMonRC;
import  com.tivoli.twg.monitor.TWGMonRecordedData;
import  com.tivoli.twg.monitor.TWGMonRecordingDataSink;
import  com.tivoli.twg.monitor.TWGMonSubSys;
import  com.tivoli.twg.monitor.TWGMonSubSysCapabilities;
import  com.tivoli.twg.monitor.TWGMonSubSysInitException;
import  com.tivoli.twg.monitor.TWGMonThreshold;
import  com.tivoli.twg.monitor.TWGMonThresholdEvent;
import  com.tivoli.twg.monitor.TWGMonThresholdEventSink;
import  com.tivoli.twg.monitor.TWGMonThresholdId;

import  com.tivoli.twg.libs.ServiceNode;
import  com.tivoli.twg.libs.ServiceNodeException;
import  com.tivoli.twg.libs.ServiceNodeClosedException;
import  com.tivoli.twg.libs.BadServiceNodeImplException;
import  com.tivoli.twg.libs.MultiLocaleBundle;
import  com.tivoli.twg.libs.LongKeyTable;
import  com.tivoli.twg.libs.StringSortSet;
import  com.tivoli.twg.libs.QueueTimer;
import  com.tivoli.twg.libs.QueueElement;
import  com.tivoli.twg.libs.DataValue;

import  com.tivoli.twg.engine.TWGNativeAddressEntryShadow;
import  com.tivoli.twg.engine.AddressEntryFormatException;

import  java.util.Date;
import  java.util.Vector;
import  java.util.Locale;
import  java.util.Enumeration;

/**
* A class that extends the TWGMonSubSys abstract class to provide a sample of
* what a monitor subsystem implemenation should include.
* For example:
* <pre>
*    MonSubsysSample subsys =  new MonSubSysSample(...);
* </pre>
*
* @author  Bill Trautman
* @version Dec 2000
*/

public class MonSubSysSample extends TWGMonSubSys {

   /* static class constants */

   /** root node programmattic path string for this subsystem */
   private static final String   MY_ROOT_NAME = "SampleSubsystem";
   /** programmatic name for the IPC path attribute the subsystem returns for moid */
   private static final String   MY_IPC_PATH_NODE_NAME = "attr.BestIPCPath";
   /** programmatic name for the State attribute the subsystem returns for moid */
   private static final String   MY_STATE_NODE_NAME = "attr.SystemStateAttr";
   /** normal time between data point refreshes */
   private static final long     REFRESH_INTERVAL = 15000;
   /**  Bundle name that contains the root qualifier name. */
   private static final String  ROOT_QUALIFIER_BUNDLE_NAME = "com.BobCo.monitors.MonSubSysSampleNLS";
   /**  Key used to retrieve the root qualifier name from the root qualifier bundle. */
   private static final String  ROOT_QUALIFIER_KEY = "RootNodeName";


   /** reference to the service node that the server provides for client
       subsystems use to contact the main engine */
   private ServiceNode           helperServiceNode = null;

   /** reference to resource bundle object that holds our NLS strings */
   private MultiLocaleBundle     nlsResourcesMlb = null;

   /** hold list of handles we gave out. */
   private StringSortSet         handleSet  = new StringSortSet();

   /** hold list of recodings we are doing */
   private StringSortSet         recordingSet  = new StringSortSet();

   /** list of currently in place thresholds */
   private Vector                threshList = new Vector();

   /** quest to keep track of timing to use to check threshold information. */
   private QueueTimer            timer      = new QueueTimer();

   /** object to control refresh timing */
   private RefresherElement      refresher  = null;

   public static class RefresherElement implements QueueElement {

      private QueueTimer myTimer = null;
      private MonSubSysSample mySubsys = null;

      public RefresherElement(MonSubSysSample subsys,
                              QueueTimer      timer) {
         this.mySubsys = subsys;
         mySubsys.refresher = this;
         this.myTimer = timer;
         myTimer.enqueue(this, REFRESH_INTERVAL, false);
      }

      public void shutdown() {
         myTimer.remove(this);
         myTimer = null;
         mySubsys = null;
      }
      /**
       * Method called when this element is removed from a queue.
       * @parm qType Queue type from which this method has been called.
       *             Valid types are "QTYPE_TIMER" and "QTYPE_BATCH".
       *
       */
      public void process(int qType) {
         /* need to to call the subsystem to have
            it process the threshold information */
         mySubsys.checkThresholds();
         mySubsys.checkRecordings();

      }
   } // end class RefresherElement

   /**
    * Constructor for the Director Monitor Subsystem
    *
    * @param     mainEventSink       reference to the object to report events to
    * @param     mainAddressSource   reference to the object to get moid addressing information from
    * @param     mainStateSource     reference to the object to get moid state information from
    * @param     mainAttributeSource reference to the object to get moid attribute information from
    */
   public MonSubSysSample(TWGMonThresholdEventSink      mainEventSink,
                          TWGMonMOAddressInfoProvider   mainAddressSource,
                          TWGMonMOStateInfoProvider     mainStateSource,
                          TWGMonMOAttributeInfoProvider mainAttributeSource ) {
      // invoke constructor for TWGMonSubSys
      super(mainEventSink, mainAddressSource, mainStateSource, mainAttributeSource );
   }  // end method MonSubSysSample

   //==============================================================
   // Interfaces requried of TWGMonSubsystem subclass
   //==============================================================

   /**
    * The method implements the behavior to initialize a subsystem.
    *
    * <P>most important thing to get done in this routine is to tuck away
    * the service node that is passed in if the subsystem needs to request
    * services over one and it does not need the overhead of creating
    * its own for that purpose.
    *
    * <P>On this call other things that should be done include
    * setting up instance data for the monitor subsystem.
    *
    * @param helperServiceNode reference to a server service node that
    *                          subsystems can use to communicate with the main
    *                          server engine, etc.
    * @exception TWGMonSubSysInitException
    *                   Indicates the Subsystem had a failure during
    *                   initialization
    */

   public void startup(ServiceNode helperServiceNode)
   throws TWGMonSubSysInitException
   {

      // tuck away the helper service node
      this.helperServiceNode = helperServiceNode;

      // Load the resource bundle for the NLS text.
      nlsResourcesMlb = MultiLocaleBundle.getSharedBundle("com.BobCo.monitors.MonSubSysSampleNLS");

      this.refresher = new RefresherElement(this, timer);

   } // end method startup

   /**
   * The method implements the shutdown behavior for the subsystem.
   * <P>during this call instance data should be freed, work queues emptied and
   * quiesed.
   */
   public void shutdown() {

      // shut down the threshold refresher object
      refresher.shutdown();
      // null out the references found in the base part of the class
      addressServer   = null;
      eventServer     = null;
      stateServer     = null;
      attributeServer = null;
      handleSet       = null;
      threshList      = null;
      refresher       = null;
      timer           = null;
      return;
   }  // end method shutdown

   /**
    * The method implements the behavior to get a report on the
    * capabilities of a subsystem.
    *
    * @param abilities object to fill in with list of capabilities the
    *                  subsystem provides.
    * @see com.tivoli.twg.monitor.TWGMonSubSysCapabilities
    */
   public void reportCapabilities(TWGMonSubSysCapabilities abilities) {

      // This flag indicates the sub-system's ability to support persistent
      // data store. For example, If this flag is set true, thresholds will
      // be pushed at each launch of the product.
      abilities.refreshAtServerLaunch = true;
      return;

   } // end method reportCapabilities

   /**
    * The method implements the behavior to get the root string
    * for the attributes available from this subsystem
    *
    * @param testRoot string value that is to be tested to see if it is a root
    *                 path segment provided by this subsystem
    * @return boolean indicating if it is a root or not.  True if it is
    */
   public boolean isRoot(String testRoot) {
      return testRoot.equals(MY_ROOT_NAME);
   } // end method isRoot

   /**
    * This method provides the root path to use with the moid if the subsystem
    * can reach the managed obect.
    * @param                 moid  managed object id for the object to check
    *                              reachability for
    * @return TWGMonAttributePath  object if the subsystem can reach the object,
    *                              null if it can not. The path returned will
    *                              be the root path that would be used to talk
    *                              with the object
    */
   public TWGMonAttributePath getRootForManagedObject(long moid) {

      TWGMonAttributePath result = null;
      // verify the object is a reachable by this subsystem
      // get the address for the nodes in the request
      String ipcPath = null;
      byte[] addrInfo = addressServer.getAddressInfoForMO(moid);
      if ( addrInfo!=null ) {
         TWGNativeAddressEntryShadow tmp = new TWGNativeAddressEntryShadow();
         try {

            tmp.initAddressEntry(addrInfo,0,addrInfo.length);
            ipcPath = tmp.getIPCPath();

            // if we got here the address is for a Native managed Object
            // return an attribute path with our root name
            result = new TWGMonAttributePath();
            result.addLevelToPath( MY_ROOT_NAME );
         } catch ( AddressEntryFormatException aefx ) {
            // if we could not format the node address, go onto next node
         }
      } else {
         // we could not format the node address
      } // end if addrInfo not null

      return result;
   }  // end method getRootForManagedObject

   /**
    * Checks whether the systems identified by the objectArray
    * are supported by this subsystem and then adds the supported object
    * ids to the list of systems in the ourNode object.
    *
    * @param ourNode addtribute node that is to have the list of
    *                supported systems, added to its system list
    * @param objectArray list of systems to check to see if they should be
    *                    added to to the attribute node
    * @return int with count of number of object ids added to ourNode
    */
   private int checkObjectList(TWGMonAttributeNode ourNode,
                               long                objectArray[]) {

      // count of number of systems that support IPC addressing
      int      numSys = 0;
      String   ipcPath = null;
      byte[]   addrInfo = null;

      //  now we need to add in the other systems in the list
      for ( int index = 0 ; index <objectArray.length; index ++ ) {

         // verify the object is a reachable by this subsystem
         // get the address for the nodes in the request
         ipcPath = null;
         addrInfo = addressServer.getAddressInfoForMO(objectArray[index]);
         if ( addrInfo!=null ) {
            TWGNativeAddressEntryShadow tmp = new TWGNativeAddressEntryShadow();
            try {

               tmp.initAddressEntry(addrInfo,0,addrInfo.length);
               ipcPath = tmp.getIPCPath();

               // if we got here the address is for a native managed object
               ourNode.addSystem(objectArray[index]);
               numSys++;
            } catch ( AddressEntryFormatException aefx ) {
               // if we could not format the node address, go onto next managed object
            }
         } else {
            // if we could not format the node address, go onto next managed object
         } // end if addrInfo not null
         ipcPath = null;
         addrInfo = null;

      } // end for loop of systems to check
      return numSys;
   } // end method checkObjectList

   /**
   * The method implements the find attribute nodes behavior for the subsystem.
   *
   * @param     managedObject[] the managed to check for attribute nodes.
   * @param     pathinfo        object with path information for the search.
   * @param     requestedLocale locale we want the displayble node names returned in.
   *
   * @return    TWGAttributeNodeList - indicating whether the command worked
   */
   public TWGMonAttributeNodeList getAttributeNodeList(long                objectArray[],
                                                       TWGMonAttributePath pathInfo,
                                                       Locale              neededLocale) {
      TWGMonAttributeNodeList outputNodeList = new TWGMonAttributeNodeList(neededLocale);

      if ( pathInfo.getPathLevels()==0 ) {
         // user is looking for the first level under the root.  for the Native
         // client subsystem we provide it in the subsystem itself.

         //  Need translation stuff for display name and good locale info
         TWGMonAttributeNode ourRootNode =
            new TWGMonAttributeNode(nlsResourcesMlb.getString(neededLocale,"RootNodeName"),
                                    MY_ROOT_NAME,
                                    neededLocale,
                                    TWGMonAttributeNode.PATH_TYPE);

         // check if the objects passed in are ok to include in list of objects
         // that the node says it is available on
         int countSupporting = checkObjectList(ourRootNode, objectArray);

         // check if any of the systems were reachable via a Native Managed Object
         // IPC path
         if ( countSupporting != 0 ) {
            // now add the single element to the list with its non null set of systems
            outputNodeList.addNodeToList(ourRootNode);
         } else {
            // no systems have a native managed object like address,  don't
            // respond with a node in the list
         } // end if systems with a netfinity ipc address

      } else {

         // seemes like are looking for something further into the tree of attributes

         if ( pathInfo.getPathAtLevel(0).equals(MY_ROOT_NAME) ) {
            // looks like the request really is for us (do this check
            // in case the caller is sloppy about which subsystems
            // are to be queried)

            if ( pathInfo.getPathLevels() == 1 ) {
               /// if asking for stuff under root name...

               //  Need translation stuff for display name and good locale info
               TWGMonAttributeNode ipcNode =
                  new TWGMonAttributeNode(nlsResourcesMlb.getString(neededLocale,"IpcPathNodeName"),
                                          MY_IPC_PATH_NODE_NAME,
                                          neededLocale,
                                          TWGMonAttributeNode.ATTRIBUTE_TYPE,
                                          TWGMonConstants.STRING_NODEDATA);

               {
                  // check if the objects passed in are ok to include in list of objects
                  // that the node says it is available on
                  int countSupporting = checkObjectList(ipcNode, objectArray);

                  // check if any of the systems were reachable via a Native Managed Object
                  // IPC path
                  if ( countSupporting != 0 ) {
                     // now add the single element to the list with its non null set of systems
                     outputNodeList.addNodeToList(ipcNode);
                  } else {
                     // no systems have a native managed object like address,  don't
                     // respond with a node in the list
                  } // end if systems with a netfinity ipc address
               }

               TWGMonAttributeNode stateNode =
                  new TWGMonAttributeNode(nlsResourcesMlb.getString(neededLocale,"StateNodeName"),
                                          MY_STATE_NODE_NAME,
                                          neededLocale,
                                          TWGMonAttributeNode.ATTRIBUTE_TYPE,
                                          TWGMonConstants.DOUBLE_NODEDATA);

               {
                  // check if the objects passed in are ok to include in list of objects
                  // that the node says it is available on
                  int countSupporting = checkObjectList(stateNode, objectArray);

                  // check if any of the systems were reachable via a Native Managed Object
                  // IPC path
                  if ( countSupporting != 0 ) {
                     // now add the single element to the list with its non null set of systems
                     outputNodeList.addNodeToList(stateNode);
                  } else {
                     // no systems have a native managed object like address,  don't
                     // respond with a node in the list
                  } // end if systems with a netfinity ipc address
               }

            } else {
               ;
            } // end if

         } else {
            // server request was not for this subsystem, ignore it by not adding
            // nodes to the list
         }
      }

      return outputNodeList;
   } // end method getAttributeNodeList

   /**
    * The method implements the retreive enumeration strings in a specific
    * local for an enumerated attribute node from the subsystem.
    *
    * @param     pathinfo object with path information for the search
    * @param     requestedLocale locale we want the enumeration strings to be returned in
    *
    * @return    TWGMonEnumStrings  enumeration strings if available
    */
   public TWGMonEnumStrings getEnumStringsByLocale(TWGMonAttributePath  pathInfo,
                                                   Locale               requestedLocale) {
      /*-  -*/
      return null;
   } // end method getEnumStringsByLocale

   /**
    * The method retrieves the resource bundle name of enumeration strings in a specific
    * locale for an enumerated attribute node from the subsystem.
    *
    * @param     pathinfo object with path information for the search
    *
    * @return    String  resource bundle name of enumeration strings if available
    */
   public String getEnumBundle( TWGMonAttributePath  pathInfo ) {
      /*-  -*/
      return null;
   } // end method getEnumBundle

   /**
    * The method retrieves the property filename that contains the
    * Threshold Template for the given attribute node from the subsystem.
    * This is an abstract interface that must be provided by derived classes
    *
    * @param     pathinfo object with path information for the search
    *
    * @return    String  threshold template filename if available otherwise null
    */
   public String getThresholdTemplateFilename( TWGMonAttributePath  pathInfo ) {
      // let our Enumeration Manager class do the work
      /*-  -*/
      return null;
   } // end method getThresholdTemplateFilename

   /**
    * The method implements the acquire handle behavior for the subsystem
    *
    * @param     pathinfo the path of the attribute to start monitoring on
    * @param  objectArray  the list of system ids to start monitoring the attribute on
    * @param  handleArray  array that is filled in by function with handles of started monitors
    * @param  resultArray  array that is filled in with result code to indicate success or failure of request
    *
    * @return    int indicating whether the command worked
    */
   public int acquireHandle(TWGMonAttributePath pathInfo, /* input */
                            long[] objectArray,           /* input */
                            String[] handleArray,         /* output */
                            int[]  resultArray)           /* output */ {

      int resultCode = TWGMonRC.SUCCESS;

      if ( !isRoot(pathInfo.getPathAtLevel(0)) ) {
         // dang it, someone passed us root that was not ours!!!
         for ( int i = 0; i<objectArray.length; i++ ) {
            // marke them all bad!
            resultArray[i] = TWGMonRC.HANDLE_NOT_AVAILABLE;
         }
         return resultCode;
      }

      // seemes like we have to talk to the native client systems since we
      // are looking for something further down the tree

      if ( pathInfo.getPathLevels()==2 ) {
         // we might be interested in this one...

         if ( pathInfo.getPathAtLevel(1).equals(MY_IPC_PATH_NODE_NAME) ) {

            // register interest in IPC Path info for the list of managed objects
            for ( int index = 0; index < objectArray.length; index++ ) {
               String tempHandle = ""+ objectArray[index]+ MY_IPC_PATH_NODE_NAME;
               handleArray[index] = tempHandle;
               resultArray[index] = TWGMonRC.GOT_HANDLE;
               handleSet.InsertValue(tempHandle,null);
            } // end for index < objectArray.length

         } else if ( pathInfo.getPathAtLevel(1).equals(MY_STATE_NODE_NAME) ) {
            // register interest in State info for the list of managed objects

            for ( int index = 0; index < objectArray.length; index++ ) {
               String tempHandle = ""+ objectArray[index]+ MY_STATE_NODE_NAME;
               handleArray[index] = tempHandle;
               resultArray[index] = TWGMonRC.GOT_HANDLE;
               handleSet.InsertValue(tempHandle,null);
            } // end for index < objectArray.length
         } // end if
      } else {
         ;
      } // end if

      return resultCode;
   } // end acquireHandle

   /**
    * The method implements the release handle behavior for the subystem
    *
    * @param  handleArray  array that provides the list of handles the server is no longer interested in
    * @param  resultArray  array that is filled in with result code to indicate success or failure of request
    * @return    int indicating whether the command worked
    */
   public int releaseHandle(String[] handleArray,         /* input */
                            int[]    resultArray)         /* output */ {

      String tempHandle = null;
      // interate over each handle in the list.
      for ( int index = 0; index < handleArray.length; index++ ) {
         resultArray[index] = TWGMonRC.GENERAL_ERROR;
         tempHandle = handleArray[index];
         if ( handleSet.Find(tempHandle,null) !=-1 ) {
            resultArray[index] = TWGMonRC.SUCCESS;
         } else {
            resultArray[index] = TWGMonRC.SUCCESS;
         }
      }  // end for loop on handle array

      return TWGMonRC.SUCCESS;
   } // end method releaseHandle

   /**
    * The method implements the startMonitor behavior for the subsystem
    *
    * @param  handleArray  array that is filled in by function with handles of started monitors
    * @param  resultArray  array that is filled in with result code to indicate success or failure of request
    *
    * @return    int indicating whether the command worked
    */
   public int startMonitor(String[] handleArray,         /* input  */
                           int[]  resultArray)           /* output */ {

      int      resultCode = TWGMonRC.SUCCESS;
      String   tempHandle;

      for ( int index = 0; index < handleArray.length; index++ ) {
         // check that the handle looks like a native client handle
         tempHandle = handleArray[index];
         if ( handleSet.Find(tempHandle,null) !=-1 ) {
            resultArray[index] = TWGMonRC.SUCCESS;
         } else {
            resultArray[index] = TWGMonRC.BAD_HANDLE;
         }

      }

      return resultCode;
   }// end method startMonitor


   /**
    * The method implements the get meta data behavior for the subystem
    *
    * @param  handleArray         array that is filled in by function with handles of started monitors
    * @param  AttributePathArray  array that is filled in the attribute path we want meta data for
    * @param  attributeArray      array that is filled in with result meta data objects with its rc set
    *
    * @return         int  indicating whether the command worked
    */
   public int getMetaData(String[]                       handleArray,         /* input */
                          TWGMonAttributePath[]          attributePathArray,  /* input */
                          TWGMonAttributeMetaDataSink[]  attributeArray)      /* input */ {

      String tempHandle = null;

      // interate over each handle in the list.
      for ( int index = 0; index < handleArray.length; index++ ) {
         tempHandle = handleArray[index];
         TWGMonMetaData aMData = null;

         if ( handleSet.Find(tempHandle,null) !=-1 ) {
            // create the server version of MetaData to return
            aMData = new TWGMonMetaData( TWGMonRC.SUCCESS,
                                         TWGMonConstants.DOUBLE_NODEDATA,
                                         REFRESH_INTERVAL, // 15 seconds
                                         false, /*- -*/
                                         1,
                                         0,  /*- this needs to be the real count -*/
                                         "",
                                         "",
                                         0);


         } else {
            // create the server version of MetaData to return
            aMData = new TWGMonMetaData( TWGMonRC.BAD_HANDLE );
         }
         attributeArray[index].updateMetaData(aMData);

      } // end for

      return TWGMonRC.SUCCESS;
   } // end method getMetaData


   /**
    * The method implements the get Attribute data behavior for the subsystem
    * it enqueues a work request to get the job done.
    *
    * @param  handleArray  array with handles of monitors to stop arranged as array of handles
    * @param  attributeArray  array with references to the attributes that should have data returned to them
    * @return int          indicating whether the command worked
    */
   public int getAttributeData(String[]                  handleArray,      /* input */
                               TWGMonAttributeDataSink[] attributeArray)   /* input */ {
      int resultCode = TWGMonRC.SUCCESS;


      // based on the handle passed in in the handle array, send the data
      //   back to the data sink in the data sink the data sink array
      for (int index = 0; index < handleArray.length; index++) {
         String tempHandle = handleArray[index];
         TWGMonAttributeDataSink sink = attributeArray[index];

         // default data point
         TWGMonCurrentData dataPoint = getDataPointForHandle(tempHandle);


         sink.updateCurrentData(dataPoint);

      } // end for index < handleArray.length

      return resultCode;
   } // end method getAttributeData

   /**
    * The method implements the stop monitoring behavior for the subsystem
    *
    * @param  handleArray  array with handles of monitors to stop arranged as array of handles
    * @param  resultArray  array that is filled in with result code to indicate success or failure of request
    * @return int          indicating whether the command worked
    */
   public int stopMonitor(String[] handleArray,        /* input  */
                          int[]    resultArray)        /* output */ {
      int resultCode = TWGMonRC.SUCCESS;
      String tempHandle;

      for ( int index = 0; index < handleArray.length; index++ ) {
         // check that the handle looks like a native client handle
         tempHandle = handleArray[index];
         if ( handleSet.Find(tempHandle,null) !=-1 ) {
            resultArray[index] = TWGMonRC.SUCCESS;
         } else {
            resultArray[index] = TWGMonRC.BAD_HANDLE;
         }
      }

      return TWGMonRC.SUCCESS;
   } // end method stopMonitor

   private TWGMonCurrentData getDataPointForHandle(String tempHandle) {
      // default data point
      TWGMonCurrentData dataPoint = new TWGMonCurrentData(TWGMonRC.GENERAL_ERROR);

      if (tempHandle.endsWith(MY_IPC_PATH_NODE_NAME)) {
         int endstring= tempHandle.lastIndexOf(MY_IPC_PATH_NODE_NAME);
         long moid = Long.valueOf(tempHandle.substring(0,endstring)).longValue();
         String ipcPath = null;
         byte[] addrInfo = addressServer.getAddressInfoForMO(moid);
         if ( addrInfo!=null ) {
            TWGNativeAddressEntryShadow tmp = new TWGNativeAddressEntryShadow();
            try {

               tmp.initAddressEntry(addrInfo,0,addrInfo.length);
               ipcPath = tmp.getIPCPath();

               dataPoint = new TWGMonCurrentData(TWGMonRC.SUCCESS,
                                                 TWGMonConstants.STRING_NODEDATA,
                                                 0,
                                                 ipcPath,
                                                 "",
                                                 new Date().getTime()
                                                 );

            } catch ( AddressEntryFormatException aefx ) {
               // if we could not format the node address, go onto next node
            }
         } else {
            // we could not format the node address
         } // end if addrInfo not null

      } else if (tempHandle.endsWith(MY_STATE_NODE_NAME)) {
         int endstring= tempHandle.lastIndexOf(MY_STATE_NODE_NAME);
         long moid = Long.valueOf(tempHandle.substring(0,endstring)).longValue();

         dataPoint = new TWGMonCurrentData(TWGMonRC.SUCCESS,
                                           TWGMonConstants.DOUBLE_NODEDATA,
                                           stateServer.getStateInfoForMO(moid),
                                           "",
                                           "",
                                           new Date().getTime()
                                           );
         ;
      } // end if
      return dataPoint;
   }

   /**
    * The method implements the start Recording behavior for the subsystem.
    *
    * @param    handleArray   array with handles of monitors to stop arranged as array of handles
    * @param startTimeArray   array of timeStamps of when the client started the recording operation in msecs since 1/1/1970
    * @param    resultArray   array that is filled in with result code to indicate success or failure of request
    *
    * @return int             indicating whether the command worked
    */
   public int startRecording(String[]     handleArray,         /* input */
                             long[]       startTimeArray,      /* output */
                             int[]        resultArray)         /* output */ {

      int                     resultCode = TWGMonRC.SUCCESS;
      String tempHandle;

      for ( int index = 0; index < handleArray.length; index++ ) {
         // check that the handle looks like a native client handle
         tempHandle = handleArray[index];
         // set the return container's values
         resultArray[index] = TWGMonRC.SUCCESS;
         // set the time we started getting data for this guy.
         startTimeArray[index] = new Date().getTime();

         recordingSet.InsertValue(tempHandle, new LongKeyTable());
      }  // end for loop

      return resultCode;
   } // end method startRecording

   /**
    * method to periodically process the recording list
    */
   private void checkRecordings() {

      // don't want the list changing while we are working with it
      synchronized(recordingSet) {
         // check current recordings now
         for (int index = 0; index < recordingSet.Length(); index++) {
            String tempHandle = recordingSet.GetValue(index);
            LongKeyTable dataPoints = (LongKeyTable)recordingSet.GetObject(index);
            TWGMonCurrentData dataPoint = getDataPointForHandle(tempHandle);
            dataPoints.InsertObject(dataPoint.getTimeStamp(),dataPoint);
         } // end for index < recordingSet.
      }
   } // end method checkRecordings

   /**
    * The method implements the get Recorded data behavior for the subsystem
    * it enqueues a work request to get the job done.
    *
    * @param  handleArray     array with handles of monitors to stop
    *                         arranged as array of handles
    * @param  attributeArray  array with references to the attributes
    *                         that should have recorded data returned to them
    * @param  startTimeArray  array with timeStamps in msecs since 1/1/1970
    *                         to specifiy the cut-off time for recorded data
    *                         points to retrieve
    *
    * @return int             indicating whether the command worked
    */
   public int getRecordedData(String[]                   handleArray,        /* input */
                              TWGMonRecordingDataSink[]  attributeArray,     /* input */
                              long[]                     startTimeArray)     /* input */ {
      int resultCode = TWGMonRC.SUCCESS;

      // take the list of handles and objects to return data to and place it on
      // the recoreded data gatherer queue
      for (int index = 0; index < handleArray.length; index++) {
         String tempHandle = handleArray[index];
         TWGMonRecordedData newPoints  =
            new TWGMonRecordedData(TWGMonRC.SUCCESS,
                                   new Date().getTime());
         LongKeyTable points = (LongKeyTable)recordingSet.GetObject(tempHandle);
         if (points!=null) {
            Object[] pointArray = points.GetAllObjects();
            for (int ptindex = 0; ptindex < pointArray.length; ptindex++) {
               newPoints.addPoint((TWGMonCurrentData)pointArray[ptindex] );
            } // end for index < points.
            points.EmptyTable();
         } // end if

         attributeArray[index].updateRecordedData(newPoints);
      } // end for index < handleArray.length

      return resultCode;
   } // end method getRecordedData

   /**
    * The method implements the stop Recording behavior for the server.
    *
    * @param  handleArray  array with handles of monitors to stop arranged as array of handles
    * @param  resultArray  array that is filled in with result code to indicate success or failure of request
    * @return int          indicating whether the command worked
    */
   public int stopRecording(String[]   handleArray,         /* input */
                            int[]      resultArray)         /* output */ {
      int                     resultCode = TWGMonRC.SUCCESS;
      String                  tempHandle = null;

      for ( int index = 0; index < handleArray.length; index++ ) {
         // check that the handle looks like a native client handle
         tempHandle = handleArray[index];
         // set the return container's values if we got something valid from the client
         resultArray[index] = TWGMonRC.SUCCESS;
         // remove from recording set
         recordingSet.RemoveValue(tempHandle);
      }  // end for loop

      return resultCode;
   } // end method stopRecording


   /***************************************************************************
    * The method implements the set threshold behavior for the subystem
    *
    * @param     pathinfo  the path of the attribute to set a threshold for
    * @param   threshinfo  the threshold being set on a list of MOIDs
    * @param    mObjectID  the list of system ids to set the threshold on
    *
    * @return         int  indicating the result of the command using TWGMonRC
    *                      SUCCESS, GENERAL_ERROR, INVALID_MOID, INVALID_PATH
    **************************************************************************/
   public int setThreshold(TWGMonAttributePath pathInfo, /* input */
                           TWGMonThreshold threshInfo,   /* input */
                           long mObjectID)               /* input */ {

      int resultCode = TWGMonRC.GENERAL_ERROR;

      if ( !isRoot(pathInfo.getPathAtLevel(0)) ) {
         // dang it, someone passed us root that was not ours!!!
         resultCode = TWGMonRC.INVALID_PATH;
         return resultCode;
      } // end if

      TWGMonThresholdId id = threshInfo.getThreshIDClass();

      SubsysThreshInfo infoBlock = new SubsysThreshInfo();
      infoBlock.id = id;
      infoBlock.thresh = threshInfo;
      infoBlock.moid = mObjectID;

      synchronized(threshList) {
         threshList.addElement( infoBlock );
      }

      resultCode = TWGMonRC.SUCCESS;
      return resultCode;
   } // end method setThreshold

   public static class SubsysThreshInfo {
      public TWGMonThresholdId id = null;
      public long moid = 0;
      public TWGMonThreshold thresh = null;
      public byte lastAlert = 0x00;
      public double numericValue = 0;
      public String stringValue = "";
      public long duration = 0;  // in milliseconds
   } // end class SubsysThreshInfo

   /**
    * method to periodically process the threshold list to see if any events should be generated
    */
   private void checkThresholds() {

      // don't want the list changing while we are working with it
      synchronized(threshList) {
         /*- check current thresholds for violations now -*/
      }
   } // end method checkThresholds

   /**
    * The method implements the delete threshold id behavior for the subystem
    *
    * @param     pathinfo  the path of the attribute to delete a threshold for
    * @param     threshID  the unique threshold ID to delete
    * @param    mObjectID  the system id to set the threshold on
    *
    * @return         int  indicating the result of the command using TWGMonRC
    *                      SUCCESS, GENERAL_ERROR, INVALID_MOID, INVALID_PATH
    */
   public int deleteThresholdID(TWGMonAttributePath   pathInfo,   /* input */
                                TWGMonThresholdId     threshID,   /* input */
                                long                  mObjectID)  /* input */ {

      int resultCode = TWGMonRC.GENERAL_ERROR;

      if ( !isRoot(pathInfo.getPathAtLevel(0)) ) {
         // dang it, someone passed us root that was not ours!!!
         resultCode = TWGMonRC.INVALID_PATH;
         return resultCode;
      }

      synchronized(threshList) {
         /* examine the threshold ids of all the thresholds                   */
         /* that we are holding onto and remove the one that is the target of */
         /* request */
         for ( Enumeration e = threshList.elements(); e.hasMoreElements() ; ) {
            SubsysThreshInfo tempObj = (SubsysThreshInfo)e.nextElement();
            if ( tempObj.id.equals(threshID) &&
                 tempObj.moid == mObjectID ) {
               threshList.removeElement(tempObj);
               resultCode = TWGMonRC.SUCCESS;
               break;
            } //end if
         } // end for
      }

      return resultCode;
   } // end method deleteThresholdID


   /**
    * The method implements the get threshold IDs behavior for the subystem
    *
    * @param     pathinfo  the path of the attribute to get threshold IDs for
    * @param    mObjectID  the system id to get the threshold on
    * @param    threshIDs  array is filled in with the returned threshold IDs
    *
    * @return         int  indicating whether the command worked
    */
   public int getThresholdIDs(TWGMonAttributePath pathInfo, /* input */
                              long   mObjectID,             /* input */
                              Vector threshIDs)             /* output */ {

      int  resultCode = TWGMonRC.GENERAL_ERROR;

      if ( !isRoot(pathInfo.getPathAtLevel(0)) ) {
         // dang it, someone passed us root that was not ours!!!
         resultCode = TWGMonRC.INVALID_PATH;
         return resultCode;
      }

      synchronized(threshList) {
         // check all the threshold information we have right now
         for ( Enumeration e = threshList.elements(); e.hasMoreElements() ; ) {
            SubsysThreshInfo tempObj = (SubsysThreshInfo)e.nextElement();
            // see if we have threshold that is assocated with the managed object
            if ( tempObj.moid == mObjectID ) {
               // now attach our threshold id to return
               threshIDs.addElement(tempObj.id);
            } else {
            } // end if
         }
      }

      resultCode = TWGMonRC.SUCCESS;
      return resultCode;
   } // end method getThresholdIDs

   /**
    * The method implements the get default threshold paths behavior for
    * the subsystem.
    *
    * <p>The subsystem is requested via this method to return a list of paths
    * for which the it desires to have default threshold information on the
    * server to applied against the return paths.  This application of threshold
    * policy will only occur for those paths returned and only if the default
    * information exists on the server
    *
    * @param mObjectID the managed object to get the default threshold paths for
    * @param symPaths  vector of programatic paths that should have default
    *                  thresholds set
    * @param guiPaths  vector of displayable paths that should have default
    *                  thresholds set
    * @return int indicating whether the command worked
    */
   public int getDefaultThresholdPaths(long      mObjectID,     /* input  */
                                       Vector    symPaths,      /* output */
                                       Vector    guiPaths) {    /* output */

      int resultCode = TWGMonRC.SUCCESS;

      // nothing filled in, we will not advertise any paths that should have
      // default thresholds applied, that is not part of this example.

      return resultCode;
   } // end method getDefaultThresholdPaths

   /**
    * The method implements the sending of independent command blobs for
    * the subystem.  The ability to send an independent command to a managed
    * object via a subsystem call is not available for plug in subsystems at
    * this time.
    *
    * @param     pathinfo     the progromatic path of the attribute to send
    *                         an independent command to
    * @param     indepCmd     the indep command class instance to send
    * @param    mObjectID     the system id to send the indepCmd to
    *
    * @return TWGMonIndepCmd  the resulting container class for the indepCmd
    */
   public TWGMonIndepCmd sendIndepCmd( TWGMonAttributePath   pathInfo,   /* input */
                                       TWGMonIndepCmd        indepCmd,   /* input */
                                       long                  mObjectID)  /* input */ {

      // create our IndepCmd container to return and set the rc to error for now
      TWGMonIndepCmd resultCmd = new TWGMonIndepCmd();
      resultCmd.setRC(TWGMonRC.GENERAL_ERROR);

      // we do not currently support independent commands for plug in subsystems
      return resultCmd;
   }  // end method sendIndepCmd

   //==============================================================
   // Additional Interfaces for this subsystem
   //==============================================================

   /**
    * The method implements the threshold event recieved behaivor for the
    * subsystem.
    *
    * @param     info  the information that describes the event that
    *                  just occured
    * @return    int   indicating whether the command worked
    */
   private int thresholdEvent(SubsysThreshInfo info) {

      int                     serverRC;
      TWGMonThresholdEvent    event = new TWGMonThresholdEvent();

      Date eventTime = new Date();

      // now set all of our values
      event.setThreshIDandCTAD( info.thresh.getThreshID(), info.thresh.getThreshCTAD());
      event.setDataType(        info.thresh.getDataType());
      event.setAlertType(       info.lastAlert);
      event.setManagedObjID( info.moid );
      event.setTime( (int)eventTime.getTime()/1000 );
      event.setEventTime( (int)eventTime.getTime()/1000 );
      event.setDuration( (int)info.duration/1000 );
      if ( (info.thresh.getDataType() & TWGMonThreshold.NUMERIC_THRESHOLD) == TWGMonThreshold.NUMERIC_THRESHOLD ) {
         event.setActualValue( info.numericValue );
      } else if ( (info.thresh.getDataType() & TWGMonThreshold.STRING_THRESHOLD) == TWGMonThreshold.STRING_THRESHOLD ) {
         event.setValueString( info.stringValue );
      } // end if
      event.setValid(true);

      return eventServer.reportThresholdEvent(event);
   } // end method thresholdEvent

   /**
    * The method returns the NLS bundle name for the bundle containing the root
    * qualifier name for the subsystem.
    *
    * <p>This is an abstract interface that must be provided by derived classes
    *
    * @return   String The name of the NLS bundle that contains the root qualifier
    *           name for the subsystem.
    */
   public String getRootQualifierBundleName() {
      return ROOT_QUALIFIER_BUNDLE_NAME;
   } // end method getRootQualifierBundleName

   /**
    * The method returns the NLS key that should be used to retrieve from the "root
    * qualifier bundle", the qualifier name for the subsystem.
    * See "getRootQualifierBundleName"
    *
    * <p>This is an abstract interface that must be provided by derived classes
    *
    * @return String The key that will retrieve the root qualifier name.
    *                      subsystem provides.
    */
   public String getRootQualifierKey() {
      return ROOT_QUALIFIER_KEY;
   } // end method getRootQualifierKey

   /**
    * This method returns the root name for the sub-system. This name is used as the
    * first level qualifier under "Director". For example, bossman sub-system
    * has a root name of "Director Agent".
    *
    * <p>This is an abstract interface that must be provided by derived classes
    *
    * <p>This is an abstract interface that must be provided by derived classes
    *
    * @return String The root name.
    */
   public String getRoot() {
      return MY_ROOT_NAME;
   } // end method getRoot

   /**
   * The method implements the behavior to get the root GUI string for the
   * subsystem based on the locale passed in
   *
   * @param  neededLocale     locale to get the string in
   * @return String           subsystem GUI name from our NLS resource bundle
   */
   public String getDispRootName( Locale neededLocale ) {
      // Load the resource bundle for the NLS text.
      MultiLocaleBundle    nlsResources = MultiLocaleBundle.getSharedBundle(ROOT_QUALIFIER_BUNDLE_NAME);
      // now return our gui name
      return nlsResources.getString( neededLocale, ROOT_QUALIFIER_KEY);
   } // end method getDispRootName

} // end class MonSubSysSample

