/******************************************************************************
 * Licensed Materials - Property of IBM                            *
 *                                                                            *
 * PID (5697-TKT)                                                             *
 *                                                                            *
 * (C) Copyright by IBM 1999. All Rights Reserved. *
 *****************************************************************************/

package com.BobCo.snapins;

import java.awt.*;
import java.beans.*;
import java.util.*;
import javax.swing.*;
import com.tivoli.twg.engine.*;
import com.tivoli.twg.cim.*;
import com.tivoli.twg.libs.*;
import com.tivoli.twg.guilibs.*;

/**
 * The ProgramGroupSnapIn class is a sample CIM instance browser.
 * It implements the TWGCIMInstanceBrowserInterface and is
 * registered to display instances of Win32_ProgramGroup.
 *
 * @version 1.1 - 6/8/05
 */
public class ProgramGroupSnapIn implements TWGCIMInstanceBrowserInterface,
                                           UFTreeListViewListener
{

   /* Private data */
   private Font                    boldFont         = null; 
   private TWGTreeListCellRenderer renderer         = null; 
   private JPanel                  panel            = null;
   private ImageLabel              panel_label      = null;
   private UFTreeListView          tree             = null;
   private int                     status           = TWGCIMConstants.UNKNOWN;
   private String                  statusText       = null;
   private aCIMClass               cimClass         = null;
   private aCIMInstance            selected         = null;
   private WindowAttributes        windowAttributes = null;
   private PropertyChangeSupport   prop             = null;
   private ListResourceBundle      bundle           = null;
   private ProgramGroupResponse    response         = null;
   private Vect                    nodes            = null;
   private ImageIcon               pgmGroupIcon     = null;
   private ImageIcon               sysGroupIcon     = null;
   private static final String     pgmGroupIconPath = "com/BobCo/snapins/images/pgmgrp16.gif";
   private static final String     sysGroupIconPath = "com/BobCo/snapins/images/sysgrp16.gif";

   /**
    * Constructor.  The snap-in interface requires a default
    * constructor that does not have any parameters.
    */
   public ProgramGroupSnapIn ()
   {
      /* Create a Java Bean property change support object */
      /* to manage the property change listeners           */
      prop = new PropertyChangeSupport( this );
   }

   /**
    * The getDisplayWindow method is used to return a JPanel 
    * that will be displayed by the caller.  The snap-in is  
    * responsible for the contents and appearance of the panel.
    */
   public JPanel getDisplayWindow ()
   {
      /* If we have already created a panel, just re-use it*/
      if (panel == null)
      {

         /* Load the default program group image */
         if (pgmGroupIcon == null)
         {
            pgmGroupIcon = new ImageIcon(TWGRemoteSnapInManager.getImage(pgmGroupIconPath));
         }
         /* Load the system program group image */
         if (sysGroupIcon == null)
         {
            sysGroupIcon = new ImageIcon(TWGRemoteSnapInManager.getImage(sysGroupIconPath));
         }

         /* Load the NLS resource bundle for display strings */
         if (bundle == null)
         {
            bundle = (ListResourceBundle)ListResourceBundle.getBundle
                     ( "com.BobCo.snapins.BobCoSnapInsResources",
                     Locale.getDefault() );
         }

         /* Create a bold font object to use for the area label */
         /* and a tree cell renderer for our tree.              */
         boldFont = new Font("Dialog", Font.BOLD, 12);
         renderer = new TWGTreeListCellRenderer();

         /* Create the panel with a Border layout */
         panel = new JPanel();
         panel.setLayout(new BorderLayout());

         /* Create an image label for the area label and add it */
         /* to the top                                          */
         panel_label = new ImageLabel();
         panel_label.setHorizontalAlignment( JLabel.CENTER );
         panel_label.setAlignmentX(JPanel.CENTER_ALIGNMENT);
         panel_label.setFont( boldFont );
         panel.add(panel_label,BorderLayout.NORTH);

         /* Create a tree view and add it to the center */
         tree = new UFTreeListView(UFTreeListView.TREE_VIEW);
         tree.setTreeCellRenderer(renderer);
         tree.setRowHeight(20);
         tree.setAsSorted(UFTreeListView.SORT_ASCENDING);
         tree.addTreeListViewListener(this);
         panel.add(tree,BorderLayout.CENTER);

      }

      /* Return the panel */
      return panel;
   }

   /**
    * Retrieve the current status of the snap-in.  Status values 
    * should be one of the constants defined in TWGCIMConstants.
    */
   public int getStatus()
   {
      return status;
   }

   /**
    * Set the current status of the snap-in.  Status values 
    * should be one of the constants defined in TWGCIMConstants.
    * After changing the status, any property change listeners
    * are notified of the change.
    */
   public void setStatus( int newStatus )
   {
      /* Load the NLS resource bundle for display strings */
      if (bundle == null)
      {
         bundle = (ListResourceBundle)ListResourceBundle.getBundle
                  ( "com.BobCo.snapins.BobCoSnapInsResources",
                  Locale.getDefault() );
      }

      /* Take actions based on the new status */
      switch ( newStatus )
      {
         /* If we are closing, then remove all references to    */
         /* our data objects so that they are garbage collected */
         /* by the Java run-time.                               */
         case TWGCIMConstants.CLOSE:
            if (response != null)
            {
               /* If we are in the middle of a CIM response, then */
               /* cancel the response processing.                 */
               response.cancel();
               response = null;
            }
            tree = null;
            panel_label = null;
            panel = null;
            renderer = null;
            boldFont = null;
            bundle = null;
            pgmGroupIcon = null;
            sysGroupIcon = null;
            statusText = null;
            cimClass = null;
            selected = null;
            windowAttributes = null;
            break;
            
         /* If we are in-progress, then update the status text */
         case TWGCIMConstants.IN_PROGRESS:
            setStatusText(bundle.getString("InProgress"));
            break;

         /* Otherwise we must be ready, so update the status text */
         default:
            setStatusText(bundle.getString("Ready"));
            response = null;
            break;
      }
      
      /* Notify any property change listeners */
      int oldStatus = status;
      status = newStatus;
      if (newStatus != TWGCIMConstants.CLOSE)
      {
         prop.firePropertyChange( "statusChange",
                                  new Integer(oldStatus),
                                  new Integer(newStatus));
      }
   }

   /**
    * Retrieve a text string that describes the current status
    * of the snap-in.  This string may be used by the parent of
    * the snap-in to update the status area at the bottom of
    * the window.
    */
   public String getStatusText()
   {
      /* Load the NLS resource bundle for display strings */
      if (bundle == null)
      {
         bundle = (ListResourceBundle)ListResourceBundle.getBundle
                  ( "com.BobCo.snapins.BobCoSnapInsResources",
                  Locale.getDefault() );
      }

      /* If we have never set the status text, then assume */
      /* we are ready                                      */
      if ( statusText == null )
      {
         statusText = bundle.getString("Ready");
      }

      /* Return the latest status text string */
      return statusText;
   }

   /**
    * Set the current status text message for the snap-in.
    */
   public void setStatusText( String newStatusText )
   {
      statusText = newStatusText;
   }

   /**
    * Retrieve the CIM class object that is being displayed 
    * by the snap-in. 
    */
   public aCIMClass getCIMClass()
   {
      return cimClass;
   }

   /**
    * Set the CIM class that the snap-in will display instance
    * for.  Calling this method will start the CIM enumeration
    * process to get the list of instances and populate the
    * display window.
    */
   public void setCIMClass( aCIMClass newCIMclass )
   {
      /* Save the class object */
      cimClass = newCIMclass;

      /* Be sure that we have a display window to update */
      getDisplayWindow();

      /* Update the area label with the class string */
      panel_label.setText(cimClass.GetClassString());

      /* If a previous CIM enumeration response is still */
      /* in progress, then we need to cancel it.         */
      if (response != null)
      {
         response.cancel();
      }

      /* Create a new CIM enumeration response to get the */
      /* instance data and update the display             */
      response = new ProgramGroupResponse(cimClass);
   }

   /**
    * Retrieve the currently selected instance.
    */
   public aCIMInstance getSelectedCIMInstance()
   {
      return selected;
   }

   /**
    * This method is used to select a given CIM instance in
    * the current display.  
    */
   public void setSelectedCIMInstance( aCIMInstance newCIMinstance )
   {
      Enumeration      enum;
      ProgramGroupData data;
      
      /* Select nothing by default */
      selected = null;

      /* Make sure we have already populated the display */
      if (nodes != null) 
      {
         /* Get the name of the current instance */
         String name = getInstanceName(newCIMinstance);

         /* Try to find a node with the name */
         data = findProgramGroup(name);

         /* If we find a match, then selecte the node */
         if (data != null)
         {
            selected = data.getCIMInstance();
         }
      }
   }

   /**
    * This method is used to get an icon image to display for
    * a CIM class. 
    */
   public ImageIcon getIconForCIMClass( aCIMClass c )
   {
      /* Load the default program group image */
      if (pgmGroupIcon == null)
      {
         pgmGroupIcon = new ImageIcon(TWGRemoteSnapInManager.getImage(pgmGroupIconPath));
      }

      /* Return the default program group image */
      return pgmGroupIcon;
   }

   /**
    * This method is called by the parent of the snap-in to
    * update the window attributes so that they match the
    * rest of the current user console.
    */
   public void setWindowAttributes( WindowAttributes wa )
   {
      /* Save the new attributes */
      windowAttributes = wa;

      /* Ensure that we have a display window */
      getDisplayWindow();

      /* Apply the new attributes to our window */
      JComponent panels[] = {tree};
      ImageLabel labels[] = {panel_label};
      JComponent others[] = {};
      windowAttributes.applyColors(panels, labels, others);

      /* Repaint the window */
      panel.invalidate();
      panel.validate();
      panel.repaint();
   }

   /**
    * Retrieve the current window attributes 
    */
   public WindowAttributes getWindowAttributes()
   {
      return windowAttributes;
   }

   /**
    * Add a property change listener to the snap-in.
    */
   public void addPropertyChangeListener( PropertyChangeListener listener )
   {
      prop.addPropertyChangeListener( listener );
   }

   /**
    * Remove a property change listener from the snap-in.
    */
   public void removePropertyChangeListener( PropertyChangeListener listener )
   {
      prop.removePropertyChangeListener( listener );
   }

   /**
    * This method is part of the UFTreeListViewListener interface
    * and is used to handle selection events in the tree.
    */
   public void ufTreeListViewEvent( UFTreeListViewEvent evt )
   {
      /* Get the object being selected */
      Object evt_object = evt.getTopic();

      switch ( (int)evt.getType( ) )
      {
         /* If no items are selected, then reset the selected */
         /* CIM instance                                      */
         case UFTreeListViewEvent.TLVE_NO_ITEMS_SELECTED:
            selected = null;
            break;
         /* If an item is being selected, then set the selected */   
         /* CIM instance from the node data.                    */
         case UFTreeListViewEvent.TLVE_SELECT_ITEM:
            selected = null;
            if ((evt_object != null) && (evt_object instanceof ProgramGroupData))
            {
               selected = ((ProgramGroupData)evt_object).getCIMInstance();
            }
            break;
      }
   }

   /**
    * This is an internal method used to find a given instance
    * in the current display.                  
    */
   private ProgramGroupData findProgramGroup ( String name )
   {
      ProgramGroupData data = null;
      Enumeration enum = nodes.elements();
      while ((data == null) && enum.hasMoreElements())
      {
         data = (ProgramGroupData) enum.nextElement();
         if (!data.getFullName().equals(name))
         {
            data = null;
         }
      }
      return data;
   }

   /**
    * This internal method is used to add a program group
    * instance to the display window.  The method returns false
    * if the instance cannot be added (because the parent has
    * not been added yet).
    */
   private boolean addProgramGroupInstance ( aCIMInstance inst )
   {
      String           parentName = null;
      ProgramGroupData parent;
      ProgramGroupData data;

      /* Get a unique name for the instance */
      String name = getInstanceName(inst);

      /* Get the default program group icon */
      ImageIcon        icon = pgmGroupIcon;

      /* Check if we need to use the system group icon */
      if (name.startsWith("All Users"))
      {
         icon = sysGroupIcon;
      }

      /* Check if this is a root level program group.  We */
      /* do this by looking for a \ character after the   */
      /* user prefix.                                     */
      if ( name.indexOf("\\\\",name.indexOf(":")) == -1 )
      {
         /* Get the user prefix */
         parentName = name.substring(0,name.indexOf(":"));

         /* Check if the prefix has been added to the tree */
         parent = findProgramGroup(parentName);
         if (parent == null)
         {
            /* Add the prefix to the tree */
            parent = new ProgramGroupData(parentName,icon,null);
            tree.addNode(tree.getRoot(),parent);
            nodes.addElement(parent);
         }

         /* Add the program group as a child of the user prefix */
         data = new ProgramGroupData(name,icon,inst);
         tree.addNode(parent,data);
         nodes.addElement(data);
         return true;
      }

      /* Determine the name of the parent instance */
      parentName = name.substring(0,name.lastIndexOf("\\\\"));
      /* Check if the parent is in the tree */
      parent = findProgramGroup(parentName);
      if (parent != null)
      {
         /* Add the program group as a child of the parent */
         data = new ProgramGroupData(name,icon,inst);
         tree.addNode(parent,data);
         nodes.addElement(data);
         return true;
      }

      /* We should only get here if this is a child program */
      /* group and the parent has not yet been added.       */
      return false;
   }

   /**
    * This internal method is used to add a list of program
    * group instances to the tree.
    */
   public void addProgramGroups ( Vect list )
   {
      Enumeration          enum;
      aCIMInstance         inst = null;
      String               name;
      String               prefix;
      String               parent_name;
      int                  tmp;
      ImageIcon            icon = pgmGroupIcon;
      ProgramGroupData     data;
      UFTreeListViewObject parent;
      boolean              done = false;
      boolean              looping = false;

      /* Create a list to track nodes added to the tree */
      nodes = new Vect();

      /* Try to add the list elements */
      while (!done && !looping)
      {
         done = true;
         looping = true;
         enum = list.elements();
         /* Loop through the list */
         while (enum.hasMoreElements())
         {
            /* Get the next instance to add */
            inst = (aCIMInstance) enum.nextElement();
            /* Try to add the instance */
            if (addProgramGroupInstance (inst))
            {
               /* The instance was added to the tree, so */
               /* remove it from our list, and reset our */
               /* loop to start over (since we modified  */
               /* the list                               */
               list.removeElement(inst);
               enum = list.elements();
               looping = false;
            }
            else
            {
               /* We could not add the item, so set the */
               /* done flag to false so that we will go */
               /* through the list again later to add   */
               /* the node (after the parent has been   */
               /* added.                                */
               done = false;
            }
         }
      }

      /* Refresh the tree to reflect the added instances */
      tree.refresh();
   }

   /**
    * This is an internal method used to generate a unique name
    * for an instance of Win32_ProgramGroup.  We build this name
    * by removing the key name and quotation marks from the
    * key string.  Note that we only use the first key string
    * for this example since Win32_ProgramGroup only has one
    * key property.
    */
   private static String getInstanceName ( aCIMInstance inst )
   {
      String[] keys = inst.GetKeys();
      String name = keys[0].substring(keys[0].indexOf('=')+1);
      if (name.startsWith("\"") && name.endsWith("\""))
      {
         name = name.substring(1);
         name = name.substring(0,name.length()-1);
      }
      return name;
   }

   /**
    * ProgramGroupResponse is an internal class used to enumerate
    * through the list of instances for a class.
    */
   class ProgramGroupResponse implements iCIMEnumerationResponse
   {
      Vect     list      = null;
      boolean  cancelled = false;

      /**
       * The constructor for this class creates a list for the
       * instances, changes the status to reflect that we have 
       * a query in progress, and then starts the enumeration
       * process.
       */
      public ProgramGroupResponse ( aCIMClass c )
      {
         list = new Vect();
         setStatus(TWGCIMConstants.IN_PROGRESS);
         if (CIMEnumeration.createInstanceEnum(((CIMClass)c).GetMOID(),c, false, this) == null)
         {
            /* We had an error, so just reset the status to ready */
            setStatus(TWGCIMConstants.READY);
         }
      }

      /**
       * This method is called by the CIM enumeration when we 
       * have an enumeration ready for processing.
       */
      public synchronized void enumReady( iCIMEnumeration enum )
      {
         /* Check if we have been cancelled while we were */
         /* waiting for the enumeration to be ready       */
         if (!cancelled)
         {
            /* If the enumeration is not empty, then process */
            /* the first entry.  Otherwise set the status to */
            /* be ready (since there is nothing to do).      */
            if (enum.hasMoreElements())
            {
               enum.nextElement(this);
            }
            else
            {
               setStatus(TWGCIMConstants.READY);
            }
         }
      }

      /**
       * This callback is not used in this sample snap-in.
       */
      public void enumResponse( iCIMEnumeration enum, aCIMClass Class )
      {
      }

      /**
       * This method is called by the CIM enumeration to process the
       * next entry in the enumeration list.
       */
      public synchronized void enumResponse( iCIMEnumeration enum, aCIMInstance cimInstance )
      {
         /* Check if our response has been cancelled. */
         if (!cancelled)
         {
            /* Add the instance to our internal list */
            list.addElement ( cimInstance );
            if ( enum.hasMoreElements() )
            {
               /* Process the next entry */
               enum.nextElement(this);
            }
            else
            {
               /* There are no more entrys, so update the */
               /* display and set the status to ready.    */
               addProgramGroups(list);
               setStatus( TWGCIMConstants.READY );
            }
         }
      }

      /**
       * This callback is not used in this sample snap-in.
       */
      public void enumResponse( iCIMEnumeration enum, aCIMMethod Method )
      {
      }

      /**
       * This callback is not used in this sample snap-in.
       */
      public void enumResponse( iCIMEnumeration enum, aCIMProperty Property )
      {
      }

      /**
       * This callback is not used in this sample snap-in.
       */
      public void enumResponse( iCIMEnumeration enum, aCIMQualifier Qualifier )
      {
      }

      /**
       * This callback is called when an error occurs during the
       * enumeration processing.
       */
      public void enumResponse( iCIMEnumeration enum, aCIMError Error )
      {
         /* Just set the status to ready */
         setStatus(TWGCIMConstants.READY);
      }

      /**
       * This callback is not used in this sample snap-in.
       */
      public void enumResponse( iCIMEnumeration enum, aCIMNameSpace Name )
      {
      }

      /**
       * This method is used to cancel an enumeration that is in-progress.
       */
      public synchronized void cancel ()
      {
         cancelled = true;
      }
   }
}
