
package com.ibm.sysmgt.sdk.tools;

import com.tivoli.twg.engine.cli.*;
import com.tivoli.twg.engine.*;
import com.tivoli.twg.libs.*;
import com.tivoli.twg.log.TWGRas;
import com.tivoli.twg.util.DataStoreException;
import java.math.BigInteger;
import java.io.*;
import java.util.*;
import java.text.DateFormat;

/**
 * Event access CLI Bundle<p>
 * This CLI bundle implementation provides access to alert manager
 * event log and event action plan functionality.<p>
 * <li>Help - provies event help
 * <li>List - will print a list of all event commands
 * <li>listFilters [-r|-report|-t|-terse]
 * <li>listEvents [-r|-report|-t|-terse] [-f filterName] [-h hours] [moid(1) ... <moid(n)]
 * <li>listEventTypes [-r|-report|-t|-terse]
 * <li>ListEventActionPlans
 * <li>ListEventActions
 * <li>ApplyEventActionPlan <PlanName> system-oid(1)>...<system-oid(N)>
 * <li>CreateEventActionPlan <PlanName>
 *                           [-f filterName(1) ActionName(1) ... ActionName(N)]
 *                           [-f filterName(N) ActionName(1) ... ActionName(N)]
 *
 * @see ICliBundle for the bundle interface
 *
 * @author Dave Wormald
 *  * @version 4.2 - 05/1/03
 */
public class SQLTableDoc implements ICliBundle {

    private static final String LanguageBundle = "com.ibm.sysmgt.sdk.tools.SDKToolsNLS";
    private Writer clientOut = null;
    private Locale clientLocale = null;
    private ResourceBundle nlsBundle = null;

    private int reportingType;
    private static final int TERSE_LISTING = 1;
    private static final String TERSE_SWITCH = "-t";
    private static final int REPORT_LISTING = 2;
    private static final String REPORT_SWITCH = "-r";
    private static final int STANDARD_LISTING = 3;

    public SQLTableDoc() {

        TWGRas.error(TWGRas.EX0," AAA SQLTableDoc()");
    }


    /**
     * CLI Usage command
     * Argument Input Format: no arguments expected
     */
    public int help(Locale locale, String[] args, Writer out) throws Exception {

        nlsBundle = getBundle(LanguageBundle, locale);
        out.write( nlsBundle.getString("TITLE") + "\r\n");
        return( list(locale, args, out) );
    }

    /**
     * CLI List command
     * Argument Input Format: no arguments expected
     */
    public int list(Locale locale, String[] args, Writer out) throws Exception
    {
        nlsBundle = getBundle(LanguageBundle, locale);
        try {
            int idx = 1;
            String str = nlsBundle.getString( "CLI_COMMANDS." +  idx );
            while (str != null) {
                out.write( str + "\r\n" );
                str = nlsBundle.getString( "CLI_COMMANDS." +  ++idx );
            }
        } catch (java.util.MissingResourceException e) {
        }

        return ICliBundle.OK;
    }

    /**
     * Cli command to retrieve the list of Director database tables
     * Argument Input Format: [-r|-report|-t|-terse]
     */
    public int listTables(Locale locale, String args[], Writer out) throws Exception
    {

        setReportingLevel(args);
        nlsBundle = getBundle(LanguageBundle, locale);
        try {
            Hashtable allTables = TWGDatabaseTable.getTables();

            if (allTables != null) {
                Enumeration tableObjects = allTables.elements();


                if (reportingType != TERSE_LISTING) {
                    out.write(nlsBundle.getString("TABLES") + "\r\n\r\n");
                }
                while (tableObjects.hasMoreElements()) {
                    TWGDatabaseTable table = (TWGDatabaseTable)tableObjects.nextElement();
                    if (table != null) {
                        // now print the columns

                        if (reportingType == TERSE_LISTING) {
                            out.write(table.getRealName() + "\r\n");
                        } else if (reportingType == STANDARD_LISTING) {
                            long oid = 0; 
                            try {
                                oid = table.ObjectID();
                            } catch (InvalidObjectIDException ioe) {
                                oid = 0;
                            }
                            out.write("" + oid
                                      + " " +  table.getRealName()
                                      + " \"" + table.getDisplayName(locale) +"\"\r\n");
                        } else if (reportingType == REPORT_LISTING) {

                            out.write(getString(nlsBundle,"TABLE_REAL_NAME") 
                                      + table.getRealName() + "\r\n"
                                      + nlsBundle.getString("DISPLAY_TABLE_NAME")
                                      + "\"" + table.getDisplayName(locale) +"\"" 
                                      +"\r\n");

                            int numberOfColumns = table.getNumberOfColumns();

                            out.write(getString(nlsBundle,"TABLE_COLUMN_COUNT") 
                                      + numberOfColumns +"\r\n\r\n");


                            Vector columnTokens = table.getTokenColumnNames();

                            for (int i = 0; i < columnTokens.size();i++) {

                                TWGDbColumn column = (TWGDbColumn)table.getColumn((String)columnTokens.elementAt(i));

                                out.write(getString(nlsBundle,"TABLE_COLUMN_REALNAME") + column.getRealColumnName() +"\r\n");
                                out.write(getString(nlsBundle,"TABLE_COLUMN_DISPLAYNAME") + table.getDisplayColumnName( (String)columnTokens.elementAt(i), locale ) +"\r\n");

                                int columnType =  column.getColumnType();

                                String tempType = null;

                                Enumeration enum = TWGObject.AllObjects(Class.forName("com.tivoli.twg.engine.TWGDatabase"));
                                if (enum.hasMoreElements()) {
                                    TWGDatabase db = (TWGDatabase)enum.nextElement();
                                    tempType = db.getColumnTypeName(columnType);
                                }

                                if (tempType == null) {
                                    switch (columnType) {
                                    case TWGDatabase.INTEGER:
                                        tempType = "INTEGER";
                                        break;
                                    case TWGDatabase.CHAR:
                                        tempType = "CHAR";
                                        break;
                                    case TWGDatabase.VARCHAR:
                                        tempType = "VARCHAR";
                                        break;
                                    case TWGDatabase.REAL:
                                        tempType = "REAL";
                                        break;
                                    case TWGDatabase.DOUBLE:
                                        tempType = "double";
                                        break;
                                    case TWGDatabase.SMALLINT:
                                        tempType = "SMALLINT";
                                        break;
                                    case TWGDatabase.DATE:
                                        tempType = "date";
                                        break;
                                    case TWGDatabase.DATETIME:
                                        tempType = "datetime";
                                        break;
                                    default:
                                        tempType = "UNKNOWN";
                                    }
                                }


                                out.write(getString(nlsBundle,"TABLE_COLUMN_TYPE") + tempType );

                                if (column.getColumnlength().intValue() > 0) {

                                    out.write(" " + getString(nlsBundle,"TABLE_COLUMN_LEN") + column.getColumnlength() +"\r\n");
                                } else {
                                    out.write("\r\n");
                                }
                                out.write(getString(nlsBundle,"TABLE_COLUMN_FILTER") + column.isFilterColumn() +"\r\n");
                                out.write(getString(nlsBundle,"TABLE_COLUMN_MAPPED") + column.isMappedColumn() +"\r\n");
                                out.write(getString(nlsBundle,"TABLE_COLUMN_KEYNUM") + column.getKeyNum() +"\r\n");
                                if (column.getKeyNum().intValue() == 0) {
                                    out.write(getString(nlsBundle,"TABLE_COLUMN_DEFAULT") + column.getColumnDefaultValue() +"\r\n");
                                }
                                out.write(getString(nlsBundle,"TABLE_COLUMN_DISTINCT") + column.queryDistinctValues() +"\r\n");
                                out.write(getString(nlsBundle,"TABLE_COLUMN_GTLT") + column.allowsGTLT() +"\r\n");
                                out.write("\r\n");

                            }
                            out.write("\r\n");
                        }




                    } else {
                        TWGRas.error(TWGRas.EX0," AAA listTables (table == null)");
                    }
                }
            } else {
                TWGRas.error(TWGRas.EX0," AAA listTables (allTables == null)");

            }
        }

        catch (java.util.MissingResourceException e) {

            TWGRas.error(TWGRas.EX0," AAA java.util.MissingResourceException", e);
        }




        return 0;
    }


    /**
     * Cli command to write out a web page describing the Director database tables
     */
    public int webReport(Locale locale, String args[], Writer out) throws Exception
    {


        nlsBundle = getBundle(LanguageBundle, locale);
        try {

            Hashtable allTables = TWGDatabaseTable.getTables();


            // First, print the HTML table containing the list of database tables
            if (allTables != null) {
                Enumeration tableObjects = allTables.elements();


                printOutputHeader(out);
                printTableHeader(out);

                while (tableObjects.hasMoreElements()) {
                    TWGDatabaseTable table = (TWGDatabaseTable)tableObjects.nextElement();
                    if (table != null) {
                        printTable(table ,  locale,  out);
                    } else {
                        TWGRas.error(TWGRas.EX0," AAA webReport (table == null)");
                    }
                }
                printTableTrailer(out);
            } else {
                TWGRas.error(TWGRas.EX0," AAA webReport (allTables == null)");
            }

            // Print the details for each database table
            if (allTables != null) {
                Enumeration tableObjects = allTables.elements();

                printTableDetailsSectionHeader(out);
                while (tableObjects.hasMoreElements()) {
                    TWGDatabaseTable table = (TWGDatabaseTable)tableObjects.nextElement();
                    if (table != null) {
                        printTableColumnsHeader(table, locale, out);

                        Vector columnTokens = table.getTokenColumnNames();

                        for (int i = 0; i < columnTokens.size();i++) {

                            TWGDbColumn column = (TWGDbColumn)table.getColumn((String)columnTokens.elementAt(i));

                            if (column != null) {
                                printColumn(table, column, locale, out);
                            }

                        }

                        printTableColumnsTrailer(out);

                    } else {
                        TWGRas.error(TWGRas.EX0," AAA webReport (table == null)");
                    }
                }
                printOutputTrailer(out);
            } else {
                TWGRas.error(TWGRas.EX0," AAA webReport (allTables == null)");

            }

        }

        catch (java.util.MissingResourceException e) {

            TWGRas.error(TWGRas.EX0," AAA java.util.MissingResourceException", e);
        }

        return 0;
    }




    private void printOutputHeader(Writer out)  throws IOException
    {
        out.write("<html>\r\n<head>\r\n<title>Database Tables</title>\r\n</head>\r\n<body>\r\n");
        out.write("<h1>IBM Director Database</h1>\r\n<P>&nbsp;</p>\r\n");
    }

    private void printOutputTrailer(Writer out)  throws IOException
    {
        out.write("<P></P>\r\n</body>\r\n</html>\r\n");
    }

    private void printTableHeader(Writer out) throws IOException
    {
        out.write("<h2>Database Tables</h2>\r\n");
        out.write("<table border=\"1\" width=\"70%\">\r\n");
        out.write("<tr>\r\n<td width=\"35%\"><b>Display Name</b></td>\r\n");
        out.write("<td width=\"65%\"><b>Real Table Name</b></td>\r\n</tr>\r\n");
    }



    private void printTable(TWGDatabaseTable table, Locale locale, Writer out) throws IOException
    {
        if (table != null) {
            out.write("<tr>\r\n<td width=\"35%\">" +table.getDisplayName(locale) + "</td>\r\n");
            out.write("<td width=\"65%\"><a href=\"#" +table.getRealName() +"\">" 
                      +table.getRealName()+ "</a></td>\r\n</tr>\r\n");

        }
    }


    private void printTableTrailer(Writer out) throws IOException
    {
        out.write("</table>\r\n");
    }



    private void printTableDetailsSectionHeader(Writer out) throws IOException
    {
        out.write("<P>&nbsp;</p><P>&nbsp;</p><h2>Table Details</h2>\r\n<P>&nbsp;</p>\r\n");
    }


    private void printTableColumnsHeader(TWGDatabaseTable table , Locale locale, Writer out) throws IOException
    {
        out.write("<P>&nbsp;</p><h3>Table: <a name=\"" 
                  + table.getRealName() + "\">" 
                  + table.getRealName() + "</a></h3>\r\n");

        out.write("<ul>\r\n");
        out.write("<li><b>"
                  + "Display Name:</b> " + table.getDisplayName(locale)
                  + "</li>");
        out.write("<li><b>Number of Columns:</b> " + table.getNumberOfColumns() + "</li>");
        out.write("<li><b>Table Version:</b> " + table.getTableVersion() + "</li>");
        String tableClassname = table.getTableName(); 
        if (tableClassname.startsWith("com.tivoli.twg")) {
            tableClassname = tableClassname.substring(tableClassname.lastIndexOf('.')+1);
        }
        out.write("<li><b>Classname:</b> " + tableClassname + "</li>");
        out.write("</ul>\r\n");


        out.write("<table border=\"1\" width=\"99%\">\r\n"
                 );

        out.write("<tr>\r\n"
                  + "<td width=\"17%\"><b>Real Name</b></td>\r\n"
                  + "<td width=\"23%\"><b>Display Name</b></td>\r\n"
                  + "<td width=\"19%\"><b>Type</b></td>\r\n"
                  + "<td width=\"8%\"><b>Filter Column</b></td>\r\n"
                  + "<td width=\"11%\"><b>Mapped Column</b></td>\r\n"
                  + "<td width=\"9%\"><b>Key Number</b></td>\r\n"
                  + "<td width=\"15%\"><b>Default Value</b></td>\r\n"
                  + "<td width=\"8%\"><b>Query For Distinct Values</b></td>\r\n"
                  + "<td width=\"7%\"><b>Allows GT &amp; LT</b></td>\r\n"
                  + "</tr>\r\n"
                 );

    }

    private void printColumn(TWGDatabaseTable table, TWGDbColumn column, Locale locale, Writer out) throws IOException
    {
        if (column != null) {
            out.write("<tr>\r\n");
            out.write("<td width=\"17%\">"
                      +column.getRealColumnName()
                      +"</td>\r\n");

            String columnDisplayName;
            try {
                columnDisplayName = table.getDisplayColumnName(column.getTokenColumnName(), locale );
            } catch (TWGDbException dbe) {
                columnDisplayName = column.getRealColumnName();
            }

            out.write("<td width=\"23%\">"
                      + columnDisplayName
                      + "</td>\r\n");

            out.write("<td width=\"19%\">"
                      +getTypeString(column.getColumnType()));


            if (column.getColumnlength().intValue() > 0) {
                out.write(" " + getString(nlsBundle,"TABLE_COLUMN_LEN") + column.getColumnlength());
            }

            out.write("</td>\r\n");

            out.write("<td width=\"8%\">"+column.isFilterColumn()+"</td>\r\n");
            out.write("<td width=\"11%\">"+column.isMappedColumn()+"</td>\r\n");
            out.write("<td width=\"9%\">"+column.getKeyNum()+"</td>\r\n");

            if (column.getKeyNum().intValue() == 0) {
                out.write("<td width=\"15%\">"+column.getColumnDefaultValue()+"&nbsp;</td>\r\n");
            } else {
                out.write("<td width=\"15%\">&nbsp;</td>\r\n");
            }
            out.write("<td width=\"8%\">"+column.queryDistinctValues()+"</td>\r\n");
            out.write("<td width=\"7%\">"+column.allowsGTLT()+"</td>\r\n");
            out.write("</tr>\r\n");
            out.write("");

        }
    }


    private void printTableColumnsTrailer(Writer out) throws IOException
    {
        out.write("</table>\r\n<P></P>\r\n");
    }




    /**
     * Cli command to retrieve the list of Director database columns
     * Argument Input Format: [-r|-report|-t|-terse]
     */
    public int listColumns(Locale locale, String args[], Writer out) throws Exception
    {

        setReportingLevel(args);
        nlsBundle = getBundle(LanguageBundle, locale);
        try {
            Hashtable allTables = TWGDatabaseTable.getTables();

            if (allTables != null) {
                Enumeration tableObjects = allTables.elements();

                if (reportingType != TERSE_LISTING) {
                    out.write(nlsBundle.getString("COLUMNS") + "\r\n\r\n");
                }


                while (tableObjects.hasMoreElements()) {
                    TWGDatabaseTable table = (TWGDatabaseTable)tableObjects.nextElement();
                    if (table != null) {
                        Vector tableColumns = table.getAllFullColumnNames();
                        for (int i = 0; i < tableColumns.size(); i++) {
                            out.write("" + tableColumns.elementAt(i) + "\r\n");
                        }
                    } else {
                        TWGRas.error(TWGRas.EX0," AAA listTables (table == null)");
                    }
                }
            } else {
                TWGRas.error(TWGRas.EX0," AAA listTables (allTables == null)");

            }
        }

        catch (java.util.MissingResourceException e) {

            TWGRas.error(TWGRas.EX0," AAA java.util.MissingResourceException", e);
        }


        return 0;
    }


    private static TWGDatabase db = null;
    private String getTypeString(int columnType) 
    {
        String tempType = null;

        if (db == null) {
            try {
                Enumeration enum = TWGObject.AllObjects(Class.forName("com.tivoli.twg.engine.TWGDatabase"));
                if (enum.hasMoreElements()) {
                    db = (TWGDatabase)enum.nextElement();
                }
            } catch (ClassNotFoundException cnfe) {
                TWGRas.error(TWGRas.EX0," AAA  ClassNotFoundException", cnfe);
            }
        }

        try {
            tempType = db.getColumnTypeName(columnType);
            if (tempType == null) {
                switch (columnType) {
                case TWGDatabase.INTEGER:
                    tempType = "INTEGER";
                    break;
                case TWGDatabase.CHAR:
                    tempType = "CHAR";
                    break;
                case TWGDatabase.VARCHAR:
                    tempType = "VARCHAR";
                    break;
                case TWGDatabase.REAL:
                    tempType = "REAL";
                    break;
                case TWGDatabase.DOUBLE:
                    tempType = "double";
                    break;
                case TWGDatabase.SMALLINT:
                    tempType = "SMALLINT";
                    break;
                case TWGDatabase.DATE:
                    tempType = "date";
                    break;
                case TWGDatabase.DATETIME:
                    tempType = "datetime";
                    break;
                default:
                    tempType = "UNKNOWN";
                }
            }
        } catch (TWGDbException dbe) {
        }

        return tempType;
    }

    /**
     * Look at command line argument and set the repoting level for listings<p>
     * Method sets instance variable reportingType
     * @param Srting[] args - CLI arguments
     */
    private void setReportingLevel(String[] args) {
        reportingType = STANDARD_LISTING;
        if (args.length > 0) {
            if (args[0].startsWith(REPORT_SWITCH)) {
                reportingType = REPORT_LISTING;
            } else if (args[0].startsWith(TERSE_SWITCH)) {
                reportingType = TERSE_LISTING;
            }
        }

        TWGRas.error(TWGRas.EX0, "AAA  reportingType = " + reportingType);
    }

    /**
     * load NLS messages
     */
    private ResourceBundle getBundle(String cls, Locale locale) {

        ResourceBundle resBundle = null;

        try {
            resBundle = ResourceBundle.getBundle(cls, locale);
        } catch (Exception e) {

            Locale defaultLocale = new Locale(new String("en"), new String("US"));
            resBundle = ResourceBundle.getBundle(cls, defaultLocale);
        }
        return resBundle;
    }


    private String getString(ResourceBundle nlsBundle, String s) 
    {
        String nlsString;
        try {
            nlsString = nlsBundle.getString(s);
        } catch (MissingResourceException mre) {
            nlsString = s;
        }

        return nlsString;

    }



    public static void main(String args[])
    {
        System.out.println("SQLTableDoc main");
    }

}
