/*****************************************************************************
 * Licensed Materials - Property of IBM                            
 * PID (5697-TKT)                                                             
 * (C) Copyright by IBM 1999. All Rights Reserved.
 ****************************************************************************/
#ifndef _DIRGRAPH_
#define _DIRGRAPH_

#include <linklist.h>

template<class T>
class DirectedGraphElement
{
public:
  DirectedGraphElement( const T &element );
  ~DirectedGraphElement();

  DirectedGraphElement
    *fNextElement;

  LinkList<DirectedGraphElement<T> *> 
    fList;
  T
    fElement;
};


template <class T> class DirectedGraphIterator;
template <class T> class DirectedGraph
{
public:
  DirectedGraph();
  ~DirectedGraph();

  void
    addNode( const T &root),
    addNode( const T &parent, const T &child ),
    addNodeAfterCursor( const T &child, 
                        DirectedGraphIterator<T> &cursor ), 
    addNodeBeforeCursor( const T &child, 
                         DirectedGraphIterator<T> &cursor ); 
  void
    removeNodeAt( DirectedGraphIterator<T> &cursor );
  T
    elementAt( DirectedGraphIterator<T> &cursor);
  unsigned long
    numberOfChildrenFor( const T& parent);

  T
    rootOfTree();
private:
  DirectedGraphElement<T> 
    *fRootNode;
  DirectedGraphElement<T>
    *findElementInList( const T &node,
                        DirectedGraphElement<T> *DGElement);
  void
    deleteElements( DirectedGraphElement<T> *DGElement);
  friend class DirectedGraphIterator<T>;
};

template <class T> class DirectedGraphIterator
{
public:
  DirectedGraphIterator( DirectedGraph<T> *tree,
                         const T& parent);
  ~DirectedGraphIterator( );
  void
    setToFirst( ),
    setToNext( );
  BOOL
    isValid( );
private:
  DirectedGraph<T>
    *fDirectedGraph;
  DirectedGraphElement<T>
    *fDGElement;
  LinkListIterator<DirectedGraphElement<T> *>
    *fCursor;
  friend class DirectedGraph<T>;
};


template<class T>
DirectedGraph<T>::DirectedGraph()
: fRootNode( 0 )
{
}

template<class T>
DirectedGraph<T>::~DirectedGraph()
{
  if ( fRootNode )
  {
    deleteElements( fRootNode );
    delete fRootNode;
  }
}

template<class T> void
  DirectedGraph<T>::addNode( const T &root)
{
  // create the root node.
  fRootNode = new DirectedGraphElement<T>( root);
}

template<class T> void
  DirectedGraph<T>::addNode( const T &parent, 
                             const T &child )
{
  DirectedGraphElement<T> *fParent = 
    findElementInList( parent, fRootNode );
  if ( fParent )
  {
    fParent->fList.addElementToEndOfList(new DirectedGraphElement<T>(child));
  }
}

template<class T> void
  DirectedGraph<T>::addNodeAfterCursor( const T &child,
                                        DirectedGraphIterator<T> &cursor )
{
  DirectedGraphElement<T> *fParent = cursor.fDGElement;
  if ( fParent )
  {
    fParent->fList.insertElementAfterCursor(new DirectedGraphElement<T>(child),
                                            *cursor.fCursor );
  }
}

template<class T> void
  DirectedGraph<T>::addNodeBeforeCursor( const T &child,
                                         DirectedGraphIterator<T> &cursor )
{
  DirectedGraphElement<T> *fParent = cursor.fDGElement;
  if ( fParent )
  {
    fParent->fList.insertElementBeforeCursor(new DirectedGraphElement<T>(child),
                                             *cursor.fCursor );
  }
}

template<class T> void
  DirectedGraph<T>::removeNodeAt( DirectedGraphIterator<T> &cursor )
{
  if ( cursor.fDGElement )
  {
    DirectedGraphElement<T> *fDGElement = 
      cursor.fDGElement->fList.elementAt( *cursor.fCursor);
    cursor.fDGElement->fList.removeElement( *cursor.fCursor );
    deleteElements( fDGElement );
    delete fDGElement;
  }
}

template<class T> DirectedGraphElement<T>* 
  DirectedGraph<T>::findElementInList( const T &node,
                                       DirectedGraphElement<T> *DGElement) 
{
  if ( DGElement->fElement == node )
    return DGElement;

  DirectedGraphElement<T> *fFoundElement = 0;
  LinkListIterator<DirectedGraphElement<T> *> fCursor( &DGElement->fList);
  for ( fCursor.setToFirst();
      fCursor.isValid();
      fCursor.setToNext() )
  {
    DirectedGraphElement<T> *fDGElement = DGElement->fList.elementAt( fCursor);
    // check the element before traversing down to the children.
    if ( fDGElement->fElement == node )
      return fDGElement;
    fFoundElement = findElementInList( node, fDGElement );
    if ( fFoundElement )
    {
      // found it
      break;
    }
  }
  return fFoundElement;
}

template<class T> void 
  DirectedGraph<T>::deleteElements( DirectedGraphElement<T> *DGElement) 
{
  LinkListIterator<DirectedGraphElement<T> *> fCursor( &DGElement->fList);
  for ( fCursor.setToFirst();
      fCursor.isValid();
      fCursor.setToNext() )
  {
    DirectedGraphElement<T> *fDGElement = DGElement->fList.elementAt(fCursor);
    deleteElements( fDGElement );
    delete fDGElement;
  }
}

template<class T> unsigned long
  DirectedGraph<T>::numberOfChildrenFor( const T& parent ) 
{
  DirectedGraphElement<T> 
    *fDGElement = findElementInList( parent, fRootNode );
  if ( fDGElement )
    return fDGElement->fList.numberOfElements();
  else
  {
    return 0;
  }
}

template<class T> T
  DirectedGraph<T>::rootOfTree( )
{
  return fRootNode->fElement;
}

template<class T> 
DirectedGraphElement<T>::DirectedGraphElement( const T &element )
: fNextElement( 0 )
{
  fElement = element;
}

template<class T> 
DirectedGraphElement<T>::~DirectedGraphElement( )
{
}

template<class T>
DirectedGraphIterator<T>::DirectedGraphIterator( DirectedGraph<T>* tree,
                                                 const T& node )
: fDirectedGraph( tree )
, fCursor( 0 )
{
  fDGElement = fDirectedGraph->findElementInList( node, 
                                                  fDirectedGraph->fRootNode );
  if ( fDGElement )
    fCursor = new LinkListIterator<DirectedGraphElement<T> *>(&fDGElement->fList);
}

template<class T>
DirectedGraphIterator<T>::~DirectedGraphIterator( )
{
  if ( fCursor )
    delete fCursor;
}

template<class T> void
  DirectedGraphIterator<T>::setToFirst(  )
{
  if ( fCursor )
    fCursor->setToFirst();
}

template<class T> void
  DirectedGraphIterator<T>::setToNext( )
{
  if ( fCursor )
    fCursor->setToNext();
}

template<class T> BOOL
  DirectedGraphIterator<T>::isValid( )
{
  if ( fCursor )
    return fCursor->isValid();
  else
    return FALSE;
}


template<class T>
T DirectedGraph<T>::elementAt( DirectedGraphIterator<T> &cursor )
{
  T fT;
  if ( cursor.fDGElement )
  {
    DirectedGraphElement<T> *fDGElement 
      = cursor.fDGElement->fList.elementAt( *cursor.fCursor );
    if ( fDGElement )
    {
      fT = fDGElement->fElement;
    }

  }
  return fT;
}

#endif
