/*****************************************************************************
 * Licensed Materials - Property of IBM
 * PID (5697-TKT)
 * (C) Copyright by IBM 1999. All Rights Reserved.
 ****************************************************************************/
#include <keyinfob.hpp>
#include <cp_conv.hpp>
#include <fstream.h>
#include <time.h>
#include <windows.h>

/**
 * This private class contains all of the data associated with a Info Block.
 * This object is reference counted by using the KeyInfoBlock objects. When the
 * KeyInfoBlock copy ctr or assignment operator is used a reference to the
 * KeyInfoBlockData object is created and the reference count is modified. All
 * of the data members are public so they are accessible to the KeyInfoBlock
 * class.
 */
class KeyInfoBlockData
{
public:
/*-------------------------------Constructors--------------------------------*/
  KeyInfoBlockData();
  ~KeyInfoBlockData();

/*---------------------------Data Members------------------------------------*/
  int refCount;
  // these all become str when class gets exported.
  char
    *fDisplayableName,
    *fProgrammaticName,
    *fJavaLangString,
    *fUnitsOfMeasure,
    *fFormat;
  unsigned long
    fBasePulse,
    fPulsesPerData,
    fNumberRecorded,
    fHandle;
  KeyInfoBlock::EDataType
    fDataType;
  BOOL
    fLeafNode,
    fConvert,
    fBlockInUse,
    fDataValid;
  time_t
    fLastTime;      // last time we collected data,
  double
    fDoubleValue;
  char
    *fStringValue;

  static const unsigned long
    fAttributeNodeSize;
  static const unsigned long
    fNodeDataSize;
  static const unsigned long
    fNodeParmSize;
  static unsigned long
    fCurrentHandle;
  static unsigned long
    fNumberOfObjects;
  static char
    fDefaultJavaLangString[6];
};

/*
 * Constants used in class
 */
const unsigned long
  KeyInfoBlockData::fAttributeNodeSize( sizeof(AttributeNode) );
const unsigned long
  KeyInfoBlockData::fNodeDataSize( sizeof(NodeData));
const unsigned long
  KeyInfoBlockData::fNodeParmSize( sizeof(NodeParms) );
unsigned long
  KeyInfoBlockData::fCurrentHandle = 100;
unsigned long
  KeyInfoBlockData::fNumberOfObjects = 0;
char
  KeyInfoBlockData::fDefaultJavaLangString[6] = "en_US";

/**
 * Constructor for private data class.
 */
KeyInfoBlockData::KeyInfoBlockData()
: refCount(1)
, fDataType( KeyInfoBlock::kInvalid )
, fBasePulse( 30000 )   // default to once every 30 seconds
, fPulsesPerData( 1 )   // default to recording data every pulse
, fNumberRecorded( 10 ) // default to keeping 5 minutes worth
, fBlockInUse( FALSE )
, fDataValid ( FALSE )
, fLastTime( 0 )
, fLeafNode( TRUE )
, fJavaLangString( 0 )
, fDisplayableName( 0 )
, fProgrammaticName( 0 )
, fUnitsOfMeasure( 0 )
, fFormat( 0 )
, fHandle( fCurrentHandle++ )
, fStringValue( 0 )
, fDoubleValue( 0.0 )
{
  fNumberOfObjects++;

  fJavaLangString = new char[ strlen(fDefaultJavaLangString) + 1];
  strcpy( fJavaLangString, fDefaultJavaLangString );

#ifdef _DEBUG
  ofstream to("C:\\temp\\test.out", ios::out | ios::app );
  to << "  KeyInfoBlockData::KeyInfoBlockData() - number of outstanding "
    << fNumberOfObjects << endl;
#endif
}

/**
 * Destructor for private data class. Free memory used for strings.
 */
KeyInfoBlockData::~KeyInfoBlockData( )
{
  if ( fDisplayableName )
  {
    delete [] fDisplayableName;
  }
  if ( fProgrammaticName )
  {
    delete [] fProgrammaticName;
  }
  if ( fJavaLangString )
  {
    delete [] fJavaLangString;
  }
  if ( fUnitsOfMeasure )
  {
    delete [] fUnitsOfMeasure;
  }
  if ( fFormat )
  {
    delete [] fFormat;
  }
  if ( fStringValue )
  {
    delete [] fStringValue;
  }
  fNumberOfObjects--;
#ifdef _DEBUG
  ofstream to("C:\\temp\\test.out", ios::out | ios::app );
  to << "  KeyInfoBlockData::~KeyInfoBlockData() - number still outstanding "
    << fNumberOfObjects << endl;
#endif
}


KeyInfoBlock::KeyInfoBlock( )
: fData( new KeyInfoBlockData )
{
}

KeyInfoBlock::KeyInfoBlock( char *displayableName,
                            char *programmaticName )
: fData( new KeyInfoBlockData )
{
  fData->fDisplayableName = new char[strlen( displayableName ) + 1];
  strcpy( fData->fDisplayableName, displayableName );

  fData->fProgrammaticName = new char[strlen( programmaticName ) + 1];
  strcpy( fData->fProgrammaticName, programmaticName );
}

KeyInfoBlock::KeyInfoBlock( const KeyInfoBlock& node )
: fData( node.fData )
{
  ++fData->refCount;
#ifdef _DEBUG
  ofstream to("C:\\temp\\test.out", ios::out | ios::app );
  to << "  KeyInfoBlockData::KeyInfoBlockData( copy ctr) - ref count is "
    << fData->refCount << endl;
#endif
}

KeyInfoBlock &KeyInfoBlock::operator = ( const KeyInfoBlock& node )
{
#ifdef _DEBUG
  ofstream to("C:\\temp\\test.out", ios::out | ios::app );
#endif
  if ( fData == node.fData )
    return *this;

  if ( --fData->refCount == 0 )
    delete fData;
  fData = node.fData;
  ++fData->refCount;

#ifdef _DEBUG
  to << "  KeyInfoBlockData::operator = ref count is "
    << fData->refCount << endl;
#endif

  return *this;
}

BOOL KeyInfoBlock::operator == ( const KeyInfoBlock& node )
{
  // the blocks are reference counted so if the fData are the same then
  // they are equal !
  if ( fData == node.fData )
    return TRUE;
  else
    return FALSE;

/*
  // the only thing we check to verify that two KeyInfoBlocks are equivalent
  // is the programmatic name.
  if (strcmp( fData->fProgrammaticName, node.fData->fProgrammaticName))
  {
    return FALSE;
  }
  else
  {
    return TRUE;
  }
*/
}

KeyInfoBlock::~KeyInfoBlock( )
{
#ifdef _DEBUG
  ofstream to("C:\\temp\\test.out", ios::out | ios::app );
#endif
  --fData->refCount;
#ifdef _DEBUG
  to << "KeyInfoBlock::~KeyInfoBlock for" << getDisplayableName()
  << "and ref count is " << fData->refCount << endl;
#endif
  if ( fData->refCount  == 0 )
    delete fData;
}

void KeyInfoBlock::setDefaultJavaLangString( const char * javaLangString )
{
  strcpy( KeyInfoBlockData::fDefaultJavaLangString, javaLangString );
}

const char * KeyInfoBlock::getDefaultJavaLangString( )
{
  return KeyInfoBlockData::fDefaultJavaLangString;
}

void KeyInfoBlock::setJavaLangString( const char * javaLangString )
{
  if ( fData->fJavaLangString )
  {
    delete [] fData->fJavaLangString;
  }
  fData->fJavaLangString = new char[strlen(javaLangString) + 1];
  strcpy( fData->fJavaLangString , javaLangString);
}


void KeyInfoBlock::setDisplayableName( const char * displayableName )
{
  if ( fData->fDisplayableName )
  {
    delete [] fData->fDisplayableName;
  }
  fData->fDisplayableName = new char[strlen(displayableName) + 1];
  strcpy( fData->fDisplayableName, displayableName);
}

void KeyInfoBlock::setProgrammaticName( const char * programmaticName )
{
  if ( fData->fProgrammaticName )
  {
    delete [] fData->fProgrammaticName;
  }
  fData->fProgrammaticName = new char[strlen(programmaticName ) + 1];
  strcpy( fData->fProgrammaticName, programmaticName );
}

void KeyInfoBlock::setUnitsOfMeasure( const char * unitsOfMeasure )
{
  if ( fData->fUnitsOfMeasure )
  {
    delete [] fData->fUnitsOfMeasure;
  }
  fData->fUnitsOfMeasure = new char[strlen(unitsOfMeasure) + 1];
  strcpy( fData->fUnitsOfMeasure, unitsOfMeasure );
}

void KeyInfoBlock::setFormat( const char * format )
{
  if ( fData->fFormat )
  {
    delete [] fData->fFormat;
  }
  fData->fFormat = new char[strlen(format) + 1];
  strcpy( fData->fFormat, format );
}

void KeyInfoBlock::setDataType( EDataType dataType )
{
  fData->fDataType = dataType;
}

KeyInfoBlock::EDataType KeyInfoBlock::getDataType() const
{
  return fData->fDataType;
}

const char * KeyInfoBlock::getJavaLangString( ) const
{
  return fData->fJavaLangString;
}

const char * KeyInfoBlock::getDisplayableName( ) const
{
  return fData->fDisplayableName;
}

const char * KeyInfoBlock::getProgrammaticName( ) const
{
  return fData->fProgrammaticName;
}

const char * KeyInfoBlock::getUnitsOfMeasure( ) const
{
  return fData->fUnitsOfMeasure;
}

const char * KeyInfoBlock::getFormat( ) const
{
  return fData->fFormat;
}

void KeyInfoBlock::setLeafNode( BOOL leafNode )
{
  fData->fLeafNode = leafNode;
}

BOOL KeyInfoBlock::isLeafNode() const
{
  return fData->fLeafNode;
}

NodeData * KeyInfoBlock::orphanNodeData( )
{
#ifdef _DEBUG
  ofstream to("C:\\temp\\test.out", ios::out | ios::app );
  to << "KeyInfoBlock::orphanNodeData " << endl;
#endif

  NodeData
    *fNodeData = 0;

  if ( dataValid() )
  {
    switch ( getDataType() )
    {
      case KeyInfoBlock::kDouble:
        {
          // allocate a NodeData struct to return
          // set the struct info
          fNodeData = (NodeData *) new char[fData->fNodeDataSize];
          if ( fNodeData )
          {
            fNodeData->size = fData->fNodeDataSize;
            fNodeData->version = NODEDATA_VERSION_1;
            fNodeData->dataType = DOUBLE_NODEDATA;
            fNodeData->data.dValue = fData->fDoubleValue;
            fNodeData->rc = RC_OK;
          }
          break;
        }
      case KeyInfoBlock::kString:
        {
          // the compressed unicode logic will be switched to use the str
          // class when we get the code moved over. For now we deal with
          // NF calls
          unsigned long
            fStringValueLength( NFConvertLocalToCompUnicode(
                                                           fData->fStringValue,
                                                           NULL, 0, L'*', NULL) );

          unsigned long
            fTotalNodeDataSize( fData->fNodeDataSize
                                + fStringValueLength + 1 );

          fNodeData = (NodeData *) new char[fTotalNodeDataSize];

          if ( fNodeData )
          {
            fNodeData->dataType = STRING_NODEDATA;
            fNodeData->data.strOffset = fData->fNodeDataSize;

            // set the struct info
            fNodeData->size = fTotalNodeDataSize;
            fNodeData->version = NODEDATA_VERSION_1;
            fNodeData->dataType = STRING_NODEDATA;
            // return the latest data
            unsigned long fStringLength =
              NFConvertLocalToCompUnicode( fData->fStringValue,
                                           (char *)(((char *)fNodeData)+fNodeData->data.strOffset),
                                           fStringValueLength + 1, L'*', NULL);
            fNodeData->rc = RC_OK;
          }
          break;
        }
    }
  }
  else
  {
#ifdef _DEBUG
    to << "  data is not valid" << endl;
#endif
    // data is not valid, so create a NodeData with invalid data rc.
    fNodeData = (NodeData *) new char[fData->fNodeDataSize];
    if ( fNodeData )
    {
      fNodeData->size = fData->fNodeDataSize;
      fNodeData->version = NODEDATA_VERSION_1;
      fNodeData->rc = RC_INVALID_DATA;
    }
  }
  return fNodeData;
}


NodeParms* KeyInfoBlock::orphanNodeParms()
{
#ifdef _DEBUG
  ofstream to("C:\\temp\\test.out", ios::out | ios::app );
  to << "KeyInfoBlock::orphanNodeParms " << endl;
#endif

  NodeParms *fNodeParms = 0;

  // build the NodeParms data structure from the information contained
  // in the KeyInfoBlock

  // Fix the strings when we move the string class over into the SDK.
  unsigned long
    fUOMLength( 0 ),
    fFormatLength( 0 );

  if ( fData->fUnitsOfMeasure )
    fUOMLength = NFConvertLocalToCompUnicode( fData->fUnitsOfMeasure,
                                              NULL, 0, L'*', NULL);
  if ( fData->fFormat )
    fFormatLength = NFConvertLocalToCompUnicode(fData->fFormat,
                                                NULL, 0, L'*', NULL);

  unsigned long
    fTotalSize( fData->fNodeParmSize
                + fUOMLength + 1
                + fFormatLength + 1 );

  fNodeParms = (NodeParms *) new char[fTotalSize];
  if ( fNodeParms )
  {
    // clear out memory
    memset( fNodeParms, 0, fTotalSize);
    fNodeParms->size = fTotalSize;
    fNodeParms->version = NODEPARMS_VERSION_2;
    fNodeParms->rc = RC_OK;
    fNodeParms->base_pulse = fData->fBasePulse;
    fNodeParms->pulses_per_data = fData->fPulsesPerData;
    fNodeParms->num_recorded = fData->fNumberRecorded;
    fNodeParms->attrib_flags = 0;
    fNodeParms->output_type = 0;
    fNodeParms->number_data_levels = 0;
    fNodeParms->data_levels = NULL;

    if ( fUOMLength )
    {
      // copy over variable length string data
      fNodeParms->units_label_offset = fData->fNodeParmSize;
      NFConvertLocalToCompUnicode(fData->fUnitsOfMeasure,
                                  (char *)(((char *)fNodeParms)+fNodeParms->units_label_offset),
                                  fUOMLength, L'*', NULL);
    }
    else
    {
      fNodeParms->units_label_offset = 0;
    }

    if ( fFormatLength )
    {
      if ( fUOMLength )
      {
        fNodeParms->value_format_offset =
          fNodeParms->units_label_offset + fUOMLength;
      }
      else
      {
        fNodeParms->value_format_offset = fData->fNodeParmSize;
      }
      NFConvertLocalToCompUnicode(fData->fFormat,
                                  (char *)(((char *)fNodeParms)+fNodeParms->value_format_offset),
                                  fFormatLength, L'*', NULL);
    }
  }
  return fNodeParms;
}


unsigned long KeyInfoBlock::getDisplayableNameLength() const
{
  // for now everything is stored as local strings. They are converted
  // to Compressed Unicode when requested by the monitor.
  return
    NFConvertLocalToCompUnicode( fData->fDisplayableName,
                                 NULL, 0, L'*', NULL);

}

unsigned long KeyInfoBlock::getProgrammaticNameLength() const
{
  // for now everything is stored as local strings. They are converted
  // to Compressed Unicode when requested by the monitor.
  return
    NFConvertLocalToCompUnicode( fData->fProgrammaticName,
                                 NULL, 0, L'*', NULL);

}

unsigned long KeyInfoBlock::getAttributeNodeSize() const
{
  // we need to calculate how much memory is needed for an AttributeNode
  //the represents this KeyInfoBlock
  return fData->fAttributeNodeSize + getDisplayableNameLength()
  + getProgrammaticNameLength();
}

void KeyInfoBlock::copyDataTo( AttributeNode *fAttributeNode )
{
#ifdef _DEBUG
  ofstream to("C:\\temp\\test.out", ios::out | ios::app );
  to << "KeyInfoBlock::copyDataTo = " << endl;
#endif

  fAttributeNode->size = getAttributeNodeSize();

  strncpy( fAttributeNode->lang,
           fData->fJavaLangString,
           sizeof(fAttributeNode->lang));
  fAttributeNode->version = NODE_VERSION_1;
  fAttributeNode->isLeaf = isLeafNode();

  switch ( fData->fDataType )
  {
    case KeyInfoBlock::kDouble:
      fAttributeNode->type = DOUBLE_NODEDATA;
      break;
    case KeyInfoBlock::kString:
      fAttributeNode->type = STRING_NODEDATA;
      break;
    case KeyInfoBlock::kInvalid:
      fAttributeNode->type = INVALID_NODEDATA;
      break;
  }
  fAttributeNode->nameOffset = fData->fAttributeNodeSize;

  // copy strings to the end of the data structure. Need to convert them
  // to compressed unicode.
  NFConvertLocalToCompUnicode( fData->fDisplayableName,
                               (char *)(((char *)fAttributeNode) + fAttributeNode->nameOffset),
                               getDisplayableNameLength(), L'*', NULL);

  fAttributeNode->symNameOffset = fData->fAttributeNodeSize +
                                  getDisplayableNameLength();

  NFConvertLocalToCompUnicode( fData->fProgrammaticName,
                               (char *)(((char *)fAttributeNode) + fAttributeNode->symNameOffset),
                               getProgrammaticNameLength(), L'*', NULL);
}

void KeyInfoBlock::setDoubleValue( double value )
{
  fData->fDoubleValue = value;
}

double KeyInfoBlock::getDoubleValue( ) const
{
  return fData->fDoubleValue;
}


void KeyInfoBlock::setStringValue( const char *value )
{
#ifdef _DEBUG
  ofstream to("C:\\temp\\test.out", ios::out | ios::app );
  to << "KeyInfoBlock::setStringValue - " << value << endl;
#endif

  if ( fData->fStringValue )
    delete [] fData->fStringValue;
  fData->fStringValue = new char[strlen(value) + 1];
  strcpy( fData->fStringValue, value );
}

const char * KeyInfoBlock::getStringValue( ) const
{
  return fData->fStringValue;
}

void KeyInfoBlock::setLastTime( unsigned long lastTime )
{
  fData->fLastTime = lastTime;
}

unsigned long KeyInfoBlock::getLastTime( ) const
{
  return fData->fLastTime;
}

void KeyInfoBlock::setBlockUsage( BOOL usage )
{
  fData->fBlockInUse = usage;
}

BOOL KeyInfoBlock::inUse() const
{
  return fData->fBlockInUse;
}

BOOL KeyInfoBlock::dataValid() const
{
  return fData->fDataValid;
}

void KeyInfoBlock::setDataValidity( BOOL valid )
{
  fData->fDataValid = valid;
}

void KeyInfoBlock::setBasePulse( unsigned long basePulse )
{
  fData->fBasePulse = basePulse;
}

unsigned long KeyInfoBlock::getBasePulse( ) const
{
  return fData->fBasePulse;
}

void KeyInfoBlock::setPulsesPerData( unsigned long pulsesPerData)
{
  fData->fPulsesPerData = pulsesPerData;
}

void KeyInfoBlock::setNumberRecorded( unsigned long numberRecorded)
{
  fData->fNumberRecorded = numberRecorded;
}

unsigned long KeyInfoBlock::getHandle( ) const
{
  return fData->fHandle;
}
