/*****************************************************************************
 * Licensed Materials - Property of IBM                            
 * PID (5697-TKT)                                                             
 * (C) Copyright by IBM 1999. All Rights Reserved.
 ****************************************************************************/
#include <windows.h>
#include <mon_sdk.h>

class KeyInfoBlock;
class ValueGenerator;
class MonitorLibraryPrivate;

/**
 * This class is designed to orchestrate the monitor agent interaction between
 * the monitor dll and the user data. The KeyInfoBlock objects are created and
 * added to the MonitorLibrary object. The KeyInfoBlocks are an encapsulation of
 * all of the data needed to describe a node to the Monitor Console. The Monitor 
 * library object then has member functions that are called by the monsamp dll
 * to obtain the data at the pulse rate described in the KeyInfoBlock objects.
 */
class MonitorLibrary
{
public:
  /**
   * Constructor for the Monitor Library Class
   */
  MonitorLibrary();
  /**
   * Destructor for the Monitor Library Class. The destructor is not virtual 
   * because inheritance was not anticipated. 
   */
  ~MonitorLibrary();

  /**
   * Add the root KeyInfoBlock to the MonitorLibrary. These blocks are 
   * maintained until the MonitorLibrary is destructed.
   *
   * @param root - Root node
   */
  void
  addKeyInfoBlock( const KeyInfoBlock& root );
  /**
   * Add KeyInfoBlock objects to the MonitorLibrary with a parent child 
   * relationship. These blocks are maintained for the lifetime of the 
   * MonitorLibrary object.
   *
   * @param parent - Parent KeyInfoBlock that was previously added
   * @param child - child of parent KeyInfoBlock
   */
  void
  addKeyInfoBlock( const KeyInfoBlock& parent, 
                   const KeyInfoBlock& child );

  /**
   * Adopt the value generator. The value generator is the object intended to be
   * overriden by a monitor agent author. The value generator is called to 
   * add the KeyInfoBlock objects to the MonitorLibrary object and to supply 
   * data for a specific KeyInfoBlock object.
   *
   * @param valueGenerator - ValueGenerator that was new'd. MonitorLibrary
   *                         will control lifetime of object.
   */
  void
  adoptValueGenerator( ValueGenerator *valueGenerator );
  /**
    * This member function is called when KeyInfoBlocks should be added to the 
    * monitor library. This member function is just a pass through to the 
    * ValueGenerator object's member function.
    */
  void
  addNodes();

  /**
    * This member function is called when a folder is opened and the children
    * nodes are requested. This gives the Monitor author the opportunity to 
    * add nodes or modify the current nodes to support a dynamic monitor.
    *
    * @param queriedPath - the path that may need updating or the addition
    *                      of new nodes.
    */
  void
  updateNodesFor( AttributePath *queriedPath );

  /**
   * News the memory for the AttributeNodeList and orphans the data structure
   * for consumption by Monitor Agent.
   */
  AttributeNodeList
  *orphanAttributeNodeListFor( AttributePath *queriedPath );

  /**
   * The queried path is used to search the collection of KeyInfoBlock objects
   * to find the corresponding KeyInfoBlock object. The handle, which is a 
   * unique id, is returned. This handle can be used to find the corresponding
   * KeyInfoBlock object in future calls.
   */
  unsigned long
  getKeyFor( AttributePath *queriedPath );

  /**
    * Called on the MonitorLibrary object to set the value for the passed
    * KeyInfoBlock object. This KeyInfoBlock object is passed to the 
    * ValueGenerator object, which will set the objects value.
    *
    * @param queriedPath - Object that has the programmatic name of the 
    *                      KeyInfoBlock and its parent KeyInfoBlocks.
    */
  void
  setValue( KeyInfoBlock& block );

  /**
   * Returns a pointer to the KeyInfoBlock object with the key handle. The
   * handle is determined by a call getKeyFor.
   *
   * @param handle - unique id that can be used to find a specific KeyInfoBlock
   *                 object.
   */
  KeyInfoBlock
  *getKeyInfoBlockWith( unsigned long handle );

  /**
   * Returns a pointer to the KeyInfoBlock object with queriedPath.
   *
   * @param queriedPath - Object that has the programmatic name of the 
   *                      KeyInfoBlock and its parent KeyInfoBlocks.
   */
  KeyInfoBlock
  *getKeyInfoBlockFor( AttributePath *queriedPath );

private:
  MonitorLibraryPrivate
  *fData;
};
